/*
 *  CLRadeonExtender - Unofficial OpenCL Radeon Extensions Library
 *  Copyright (C) 2014-2016 Mateusz Szpakowski
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <CLRX/Config.h>
#include "GCNAsmOpc.h"

const GCNAsmOpcodeCase encGCNOpcodeCases[] =
{
    { "    s_add_u32  s21, s4, s61", 0x80153d04U, 0, false, true, "" },
    /* registers and literals */
    { "    s_add_u32  vcc_lo, s4, s61", 0x806a3d04U, 0, false, true, "" },
    { "    s_add_u32  vcc_hi, s4, s61", 0x806b3d04U, 0, false, true, "" },
    { "    s_add_u32  tba_lo, s4, s61", 0x806c3d04U, 0, false, true, "" },
    { "    s_add_u32  tba_hi, s4, s61", 0x806d3d04U, 0, false, true, "" },
    { "    s_add_u32  tma_lo, s4, s61", 0x806e3d04U, 0, false, true, "" },
    { "    s_add_u32  tma_hi, s4, s61", 0x806f3d04U, 0, false, true, "" },
    { "    s_add_u32  m0, s4, s61", 0x807c3d04U, 0, false, true, "" },
    { "    s_add_u32  ttmp0, s4, s61", 0x80703d04U, 0, false, true, "" },
    { "    s_add_u32  ttmp1, s4, s61", 0x80713d04U, 0, false, true, "" },
    { "    s_add_u32  ttmp[2:2], s4, s61", 0x80723d04U, 0, false, true, "" },
    { "    s_add_u32  ttmp[2], s4, s61", 0x80723d04U, 0, false, true, "" },
    { "    s_add_u32  s[21:21], s4, s61", 0x80153d04U, 0, false, true, "" },
    { "    s_add_u32  s[21], s[4], s[61]", 0x80153d04U, 0, false, true, "" },
    { "zx=21; ss=2; b=60;s_add_u32  s[zx], s[ss*2], s[b+1]",
            0x80153d04U, 0, false, true, "" },
    /* register's symbols */
    { "zx=%s[20:23]; ss=%s4; b=%s[57:67];s_add_u32  zx[1], ss, b[4]",
            0x80153d04U, 0, false, true, "" },
    { "xv=2; zx=%s[xv+18:xv+19]; ss=%s[xv+2]; b=%s[xv+55:xv+65];"
        "s_add_u32  zx[1], ss, b[4]", 0x80153d04U, 0, false, true, "" },
    { "zx=%s[20:23]; ss=%execz; b=%s[57:67];s_add_u32  zx[1], ss, b[4]",
            0x80153dfcU, 0, false, true, "" },
    { "    s_add_u32  s21, s4, 0", 0x80158004U, 0, false, true, "" },
    { "lit=3;s_add_u32  s21, s4, lit*5", 0x80158f04U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, 1", 0x80158104U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, 0x2a", 0x8015aa04U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, -7", 0x8015c704U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, lit(-7)", 0x8015ff04U, 0xfffffff9, true, true, "" },
    { "    s_add_u32  s21, s4, lit(0)", 0x8015ff04U, 0x0, true, true, "" },
    { "    s_add_u32  s21, s4, lit(32)", 0x8015ff04U, 0x20, true, true, "" },
    { "    s_add_u32  s21, s4, lit  (  32  )  ", 0x8015ff04U, 0x20, true, true, "" },
    { "    s_add_u32  s21, s4, LiT  (  32  )  ", 0x8015ff04U, 0x20, true, true, "" },
    { "    s_add_u32  s21, s4, lit(0.0)", 0x8015ff04U, 0x0, true, true, "" },
    { "    s_add_u32  s21, s4, lit(-.5)", 0x8015ff04U, 0xbf000000, true, true, "" },
    { "    s_add_u32  s21, s4, lit(2.0)", 0x8015ff04U, 0x40000000, true, true, "" },
    { "    s_add_u32  s21, s4, lit  ( 2.0 )", 0x8015ff04U, 0x40000000, true, true, "" },
    { "    s_add_u32  s21, scc, s61", 0x80153dfdU, 0, false, true, "" },
    { "    s_add_u32  s21, execz, s61", 0x80153dfcU, 0, false, true, "" },
    { "    s_add_u32  s21, vccz, s61", 0x80153dfbU, 0, false, true, "" },
    { "t=0x2b;    s_add_u32  s21, s4, t", 0x8015ab04U, 0, false, true, "" },
    { "s0=0x2b;    s_add_u32  s21, s4, @s0", 0x8015ab04U, 0, false, true, "" },
    { "s0=0x2c;    s_add_u32  s21, s4, @s0-1", 0x8015ab04U, 0, false, true, "" },
    { "s123x=0x2b;    s_add_u32  s21, s4, s123x", 0x8015ab04U, 0, false, true, "" },
    { "ttmp123x=0x2b;    s_add_u32  s21, s4, ttmp123x", 0x8015ab04U, 0, false, true, "" },
    { "s_add_u32  s21, s4, @s0-1; s0=600", 0x8015ff04U, 599, true, true, "" },
    { "s_add_u32  s21, s4, @s0-1; s0=40", 0x8015ff04U, 39, true, true, "" },
    { "s_add_u32  s21, s4, 3254500000000", 0x8015ff04U, 0xbf510100U, true, true,
        "test.s:1:21: Warning: Value 0x2f5bf510100 truncated to 0xbf510100\n" },
    { "s_add_u32  s21, s4, xx\nxx=3254500000000", 0x8015ff04U, 0xbf510100U, true, true,
        "test.s:1:21: Warning: Value 0x2f5bf510100 truncated to 0xbf510100\n" },
    /* parse second source as expression ('@' force that) */
    { ".5=0x2b;    s_add_u32       s21, s4, @.5", 0x8015ab04U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, .5", 0x8015f004U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, -.5", 0x8015f104U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, 1.", 0x8015f204U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, -1.", 0x8015f304U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, 2.", 0x8015f404U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, -2.", 0x8015f504U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, 4.", 0x8015f604U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, -4.", 0x8015f704U, 0, false, true, "" },
    { "    s_add_u32  s21, s4, 1234", 0x8015ff04U, 1234, true, true, "" },
    { "    s_add_u32  s21, 1234, s4", 0x801504ffU, 1234, true, true, "" },
    { "    s_add_u32  s21, s4, -4.5", 0x8015ff04U, 0xc0900000U, true, true, "" },
    { "    s_add_u32  s21, s4, -4.5s", 0x8015ff04U, 0xc0900000U, true, true, "" },
    { "    s_add_u32  s21, s4, -4.5h", 0x8015ff04U, 0xc480U, true, true, "" },
    { "    s_add_u32  s21, s4, 3e-7", 0x8015ff04U, 0x34a10fb0U, true, true, "" },
    /* 64-bit registers and literals */
    { "    s_xor_b64 s[22:23], s[4:5], s[62:63]\n", 0x89963e04U, 0, false, true, "" },
    { "s1=22; s2=4;s3=62;s_xor_b64 s[s1:s1+1], s[s2:s2+1], s[s3:s3+1]\n",
        0x89963e04U, 0, false, true, "" },
    { "    s_xor_b64 vcc, s[4:5], s[62:63]\n", 0x89ea3e04U, 0, false, true, "" },
    { "    s_xor_b64 tba, s[4:5], s[62:63]\n", 0x89ec3e04U, 0, false, true, "" },
    { "    s_xor_b64 tma, s[4:5], s[62:63]\n", 0x89ee3e04U, 0, false, true, "" },
    { "    s_xor_b64 ttmp[4:5], s[4:5], s[62:63]\n", 0x89f43e04U, 0, false, true, "" },
    { "    s_xor_b64 exec, s[4:5], s[62:63]\n", 0x89fe3e04U, 0, false, true, "" },
    { "    s_xor_b64 exec, s[4:5], 4000\n", 0x89feff04U, 4000, true, true, "" },
    { "    s_xor_b64 s[22:23], 0x2e, s[62:63]\n", 0x89963eaeU, 0, false, true, "" },
    { "    s_xor_b64 s[22:23], -12, s[62:63]\n", 0x89963eccU, 0, false, true, "" },
    { "    s_xor_b64 s[22:23], 1.0, s[62:63]\n", 0x89963ef2U, 0, false, true, "" },
    { "    s_xor_b64 s[22:23], vccz, s[62:63]\n", 0x89963efbU, 0, false, true, "" },
    { "    s_xor_b64 s[22:23], execz, s[62:63]\n", 0x89963efcU, 0, false, true, "" },
    { "    s_xor_b64 s[22:23], scc, s[62:63]\n", 0x89963efdU, 0, false, true, "" },
    /* long var names */
    { "sx1111111111111111111222222222=67777\n"
      "s_add_u32  s21, s4, sx1111111111111111111222222222\n",
      0x8015ff04U, 67777, true, true, "" },
    { "vx1111111111111111111222222222=67777\n"
      "s_add_u32  s21, s4, vx1111111111111111111222222222\n",
      0x8015ff04U, 67777, true, true, "" },
    /* errors */
    { "    s_add_u32  xx, s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Expected 1 scalar register\n" },
    { "    s_add_u32  s[2:3], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Required 1 scalar register\n" },
    { "    s_add_u32  ttmp[2:3], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Required 1 scalar register\n" },
    { "    s_add_u32  s[3:4], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Required 1 scalar register\n" },
    { "    s_add_u32  ttmp[3:4], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Required 1 scalar register\n" },
    { "    s_add_u32  s104, s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Scalar register number out of range\n" },
    { "    s_add_u32  ttmp12, s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: TTMPRegister number out of range (0-11)\n" },
    { "    s_add_u32  s[104:105], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Some scalar register number out of range\n" },
    { "    s_add_u32  ttmp[12:13], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Some TTMPRegister number out of range (0-11)\n" },
    { "    s_add_u32  s[44:42], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Illegal scalar register range\n" },
    { "    s_add_u32  ttmp[10:8], s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Illegal TTMPRegister range\n" },
    { "    s_add_u32  s[z], s4, s61", 0, 0, false, false,
        "test.s:1:18: Error: Expression have unresolved symbol 'z'\n"
        "test.s:1:19: Error: Expected ',' before argument\n" },
    { "    s_add_u32  ttmp[z], s4, s61", 0, 0, false, false,
        "test.s:1:21: Error: Expression have unresolved symbol 'z'\n"
        "test.s:1:22: Error: Expected ',' before argument\n" },
    { "    s_add_u32  sxzz, s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Expected 1 scalar register\n" },
    { "    s_add_u32  ttmpxzz, s4, s61", 0, 0, false, false,
        "test.s:1:16: Error: Expected 1 scalar register\n" },
    { "    s_add_u32  s30, s[4, s[61", 0, 0, false, false,
        "test.s:1:21: Error: Unterminated scalar register range\n"
        "test.s:1:26: Error: Unterminated scalar register range\n" },
    { "    s_xor_b64  s[30:31], s[3:4], s[15:16]", 0, 0, false, false,
        "test.s:1:26: Error: Unaligned scalar register range\n"
        "test.s:1:34: Error: Unaligned scalar register range\n" },
    { "    s_xor_b64  s[30:31], vcc_lo, s[14:15]", 0, 0, false, false,
        "test.s:1:26: Error: Required 2 scalar registers\n" },
    { "    s_xor_b64  s[30:31], vcc_hi, s[14:15]", 0, 0, false, false,
        "test.s:1:26: Error: Required 2 scalar registers\n" },
    { "    s_add_u32  s10, xfdd+a*, s61", 0, 0, false, false,
        "test.s:1:28: Error: Unterminated expression\n" },
    { "    s_add_u32  s10, , ", 0, 0, false, false,
        "test.s:1:21: Error: Expected instruction operand\n"
        "test.s:1:23: Error: Expected instruction operand\n" },
    { "  s_add_u32  s35, 400000, 1111116", 0, 0, false, false,
        "test.s:1:27: Error: Only one literal can be used in instruction\n" },
    { "    s_xor_b64  s[30:31], vcc_hi, s[14:15]", 0, 0, false, false,
        "test.s:1:26: Error: Required 2 scalar registers\n" },
    { "k=%s5; s_xor_b64  s[30:31], k, s[14:15]", 0, 0, false, false,
        "test.s:1:29: Error: Required 2 scalar registers\n" },
    { "k=%execz; s_xor_b64  s[30:31], k, s[14:15]", 0, 0, false, false,
        "test.s:1:32: Error: Required 2 scalar registers\n" },
    { "k=%v[10:11]; s_xor_b64  s[30:31], k, s[14:15]", 0, 0, false, false,
        "test.s:1:35: Error: Expression have register symbol\n"
        "test.s:1:35: Error: Expected ',' before argument\n" },
    /* SOP2 encodings */
    { "    s_sub_u32  s21, s4, s61", 0x80953d04U, 0, false, true, "" },
    { "    s_add_i32  s21, s4, s61", 0x81153d04U, 0, false, true, "" },
    { "    s_sub_i32  s21, s4, s61", 0x81953d04U, 0, false, true, "" },
    { "    s_addc_u32  s21, s4, s61", 0x82153d04U, 0, false, true, "" },
    { "    s_subb_u32  s21, s4, s61", 0x82953d04U, 0, false, true, "" },
    { "    s_min_i32  s21, s4, s61", 0x83153d04U, 0, false, true, "" },
    { "    s_min_u32  s21, s4, s61", 0x83953d04U, 0, false, true, "" },
    { "    s_max_i32  s21, s4, s61", 0x84153d04U, 0, false, true, "" },
    { "    s_max_u32  s21, s4, s61", 0x84953d04U, 0, false, true, "" },
    { "    s_cselect_b32  s21, s4, s61", 0x85153d04U, 0, false, true, "" },
    { "    s_cselect_b64  s[20:21], s[4:5], s[62:63]", 0x85943e04U, 0, false, true, "" },
    { "    s_and_b32  s21, s4, s61", 0x87153d04U, 0, false, true, "" },
    { "    s_and_b64  s[20:21], s[4:5], s[62:63]", 0x87943e04U, 0, false, true, "" },
    { "    s_or_b32  s21, s4, s61", 0x88153d04U, 0, false, true, "" },
    { "    s_or_b64  s[20:21], s[4:5], s[62:63]", 0x88943e04U, 0, false, true, "" },
    { "    s_xor_b32  s21, s4, s61", 0x89153d04U, 0, false, true, "" },
    { "    s_xor_b64  s[20:21], s[4:5], s[62:63]", 0x89943e04U, 0, false, true, "" },
    { "    s_andn2_b32  s21, s4, s61", 0x8a153d04U, 0, false, true, "" },
    { "    s_andn2_b64  s[20:21], s[4:5], s[62:63]", 0x8a943e04U, 0, false, true, "" },
    { "    s_orn2_b32  s21, s4, s61", 0x8b153d04U, 0, false, true, "" },
    { "    s_orn2_b64  s[20:21], s[4:5], s[62:63]", 0x8b943e04U, 0, false, true, "" },
    { "    s_nand_b32  s21, s4, s61", 0x8c153d04U, 0, false, true, "" },
    { "    s_nand_b64  s[20:21], s[4:5], s[62:63]", 0x8c943e04U, 0, false, true, "" },
    { "    s_nor_b32  s21, s4, s61", 0x8d153d04U, 0, false, true, "" },
    { "    s_nor_b64  s[20:21], s[4:5], s[62:63]", 0x8d943e04U, 0, false, true, "" },
    { "    s_xnor_b32  s21, s4, s61", 0x8e153d04U, 0, false, true, "" },
    { "    s_xnor_b64  s[20:21], s[4:5], s[62:63]", 0x8e943e04U, 0, false, true, "" },
    { "    s_lshl_b32  s21, s4, s61", 0x8f153d04U, 0, false, true, "" },
    { "    s_lshl_b64  s[20:21], s[4:5], s61", 0x8f943d04U, 0, false, true, "" },
    { "    s_lshr_b32  s21, s4, s61", 0x90153d04U, 0, false, true, "" },
    { "    s_lshr_b64  s[20:21], s[4:5], s61", 0x90943d04U, 0, false, true, "" },
    { "    s_ashr_i32  s21, s4, s61", 0x91153d04U, 0, false, true, "" },
    { "    s_ashr_i64  s[20:21], s[4:5], s61", 0x91943d04U, 0, false, true, "" },
    { "    s_bfm_b32  s21, s4, s61", 0x92153d04U, 0, false, true, "" },
    { "    s_bfm_b64  s[20:21], s4, s62", 0x92943e04U, 0, false, true, "" },
    { "    s_mul_i32  s21, s4, s61", 0x93153d04U, 0, false, true, "" },
    { "    s_bfe_u32  s21, s4, s61", 0x93953d04U, 0, false, true, "" },
    { "    s_bfe_i32  s21, s4, s61", 0x94153d04U, 0, false, true, "" },
    { "    s_bfe_u64  s[20:21], s[4:5], s61", 0x94943d04U, 0, false, true, "" },
    { "    s_bfe_i64  s[20:21], s[4:5], s61", 0x95143d04U, 0, false, true, "" },
    { "    s_cbranch_g_fork  s[4:5], s[62:63]", 0x95803e04U, 0, false, true, "" },
    { "    s_absdiff_i32  s21, s4, s61", 0x96153d04U, 0, false, true, "" },
    /* SOP1 */
    { "    s_mov_b32  s86, s20", 0xbed60314U, 0, false, true, "" },
    { "    s_mov_b32  s86, 0xadbc", 0xbed603ff, 0xadbc, true, true, "" },
    { "    s_mov_b32  s86, xx; xx=0xadbc", 0xbed603ff, 0xadbc, true, true, "" },
    { "    s_mov_b64  s[86:87], s[20:21]", 0xbed60414U, 0, false, true, "" },
    { "    s_cmov_b32  s86, s20", 0xbed60514U, 0, false, true, "" },
    { "    s_cmov_b64  s[86:87], s[20:21]", 0xbed60614U, 0, false, true, "" },
    { "    s_not_b32  s86, s20", 0xbed60714U, 0, false, true, "" },
    { "    s_not_b64  s[86:87], s[20:21]", 0xbed60814U, 0, false, true, "" },
    { "    s_wqm_b32  s86, s20", 0xbed60914U, 0, false, true, "" },
    { "    s_wqm_b64  s[86:87], s[20:21]", 0xbed60a14U, 0, false, true, "" },
    { "    s_brev_b32  s86, s20", 0xbed60b14U, 0, false, true, "" },
    { "    s_brev_b64  s[86:87], s[20:21]", 0xbed60c14U, 0, false, true, "" },
    { "    s_bcnt0_i32_b32  s86, s20", 0xbed60d14U, 0, false, true, "" },
    { "    s_bcnt0_i32_b64  s86, s[20:21]", 0xbed60e14U, 0, false, true, "" },
    { "    s_bcnt1_i32_b32  s86, s20", 0xbed60f14U, 0, false, true, "" },
    { "    s_bcnt1_i32_b64  s86, s[20:21]", 0xbed61014U, 0, false, true, "" },
    { "    s_ff0_i32_b32  s86, s20", 0xbed61114U, 0, false, true, "" },
    { "    s_ff0_i32_b64  s86, s[20:21]", 0xbed61214U, 0, false, true, "" },
    { "    s_ff1_i32_b32  s86, s20", 0xbed61314U, 0, false, true, "" },
    { "    s_ff1_i32_b64  s86, s[20:21]", 0xbed61414U, 0, false, true, "" },
    { "    s_flbit_i32_b32  s86, s20", 0xbed61514U, 0, false, true, "" },
    { "    s_flbit_i32_b64  s86, s[20:21]", 0xbed61614U, 0, false, true, "" },
    { "    s_flbit_i32  s86, s20", 0xbed61714U, 0, false, true, "" },
    { "    s_flbit_i32_i64  s86, s[20:21]", 0xbed61814U, 0, false, true, "" },
    { "    s_sext_i32_i8  s86, s20", 0xbed61914U, 0, false, true, "" },
    { "    s_sext_i32_i16  s86, s20", 0xbed61a14U, 0, false, true, "" },
    { "    s_bitset0_b32  s86, s20", 0xbed61b14U, 0, false, true, "" },
    { "    s_bitset0_b64  s[86:87], s20", 0xbed61c14U, 0, false, true, "" },
    { "    s_bitset1_b32  s86, s20", 0xbed61d14U, 0, false, true, "" },
    { "    s_bitset1_b64  s[86:87], s20", 0xbed61e14U, 0, false, true, "" },
    { "    s_getpc_b64  s[86:87]", 0xbed61f00U, 0, false, true, "" },
    { "    s_setpc_b64  s[20:21]", 0xbe802014U, 0, false, true, "" },
    { "    s_swappc_b64  s[86:87], s[20:21]", 0xbed62114U, 0, false, true, "" },
    { "    s_rfe_b64  s[20:21]", 0xbe802214U, 0, false, true, "" },
    { "    s_and_saveexec_b64 s[86:87], s[20:21]", 0xbed62414U, 0, false, true, "" },
    { "    s_or_saveexec_b64 s[86:87], s[20:21]", 0xbed62514U, 0, false, true, "" },
    { "    s_xor_saveexec_b64 s[86:87], s[20:21]", 0xbed62614U, 0, false, true, "" },
    { "    s_andn2_saveexec_b64 s[86:87], s[20:21]", 0xbed62714U, 0, false, true, "" },
    { "    s_orn2_saveexec_b64 s[86:87], s[20:21]", 0xbed62814U, 0, false, true, "" },
    { "    s_nand_saveexec_b64 s[86:87], s[20:21]", 0xbed62914U, 0, false, true, "" },
    { "    s_nor_saveexec_b64 s[86:87], s[20:21]", 0xbed62a14U, 0, false, true, "" },
    { "    s_xnor_saveexec_b64 s[86:87], s[20:21]", 0xbed62b14U, 0, false, true, "" },
    { "    s_quadmask_b32  s86, s20",  0xbed62c14U, 0, false, true, "" },
    { "    s_quadmask_b64  s[86:87], s[20:21]",  0xbed62d14U, 0, false, true, "" },
    { "    s_movrels_b32  s86, s20",  0xbed62e14U, 0, false, true, "" },
    { "    s_movrels_b64  s[86:87], s[20:21]",  0xbed62f14U, 0, false, true, "" },
    { "    s_movreld_b32  s86, s20",  0xbed63014U, 0, false, true, "" },
    { "    s_movreld_b64  s[86:87], s[20:21]",  0xbed63114U, 0, false, true, "" },
    { "    s_cbranch_join  s20", 0xbe803214U, 0, false, true, "" },
    { "    s_mov_regrd_b32 s86, s20", 0xbed63314U, 0, false, true, "" },
    { "    s_abs_i32  s86, s20", 0xbed63414U, 0, false, true, "" },
    { "    s_mov_fed_b32  s86, s20", 0xbed63514U, 0, false, true, "" },
    /* SOPC */
    { "    s_cmp_eq_i32  s29, s69", 0xbf00451dU, 0, false, true, "" },
    { "    s_cmp_eq_i32  12222, s69", 0xbf0045ffU, 12222, true, true, "" },
    { "    s_cmp_eq_i32  xx, s69; xx=12222", 0xbf0045ffU, 12222, true, true, "" },
    { "    s_cmp_eq_i32  s29, 32545", 0xbf00ff1dU, 32545, true, true, "" },
    { "    s_cmp_eq_i32  s29, xx; xx=32545", 0xbf00ff1dU, 32545, true, true, "" },
    { "    s_cmp_lg_i32  s29, s69", 0xbf01451dU, 0, false, true, "" },
    { "    s_cmp_gt_i32  s29, s69", 0xbf02451dU, 0, false, true, "" },
    { "    s_cmp_ge_i32  s29, s69", 0xbf03451dU, 0, false, true, "" },
    { "    s_cmp_lt_i32  s29, s69", 0xbf04451dU, 0, false, true, "" },
    { "    s_cmp_le_i32  s29, s69", 0xbf05451dU, 0, false, true, "" },
    { "    s_cmp_eq_u32  s29, s69", 0xbf06451dU, 0, false, true, "" },
    { "    s_cmp_lg_u32  s29, s69", 0xbf07451dU, 0, false, true, "" },
    { "    s_cmp_gt_u32  s29, s69", 0xbf08451dU, 0, false, true, "" },
    { "    s_cmp_ge_u32  s29, s69", 0xbf09451dU, 0, false, true, "" },
    { "    s_cmp_lt_u32  s29, s69", 0xbf0a451dU, 0, false, true, "" },
    { "    s_cmp_le_u32  s29, s69", 0xbf0b451dU, 0, false, true, "" },
    { "    s_bitcmp0_b32  s29, s69", 0xbf0c451dU, 0, false, true, "" },
    { "    s_bitcmp1_b32  s29, s69", 0xbf0d451dU, 0, false, true, "" },
    { "    s_bitcmp0_b64  s[28:29], s69", 0xbf0e451cU, 0, false, true, "" },
    { "    s_bitcmp1_b64  s[28:29], s69", 0xbf0f451cU, 0, false, true, "" },
    { "    s_setvskip  s29, s69", 0xbf10451dU, 0, false, true, "" },
    /* SOPK */
    { "    s_movk_i32  s43, 0xd3b9", 0xb02bd3b9U, 0, false, true, "" },
    { "xc = 0xd4ba\n    s_movk_i32  s43, xc", 0xb02bd4baU, 0, false, true, "" },
    { "    s_movk_i32  s43, xc; xc = 0xd4ba", 0xb02bd4baU, 0, false, true, "" },
    { "xc = 0x11d4ba\n    s_movk_i32  s43, xc", 0xb02bd4baU, 0, false, true,
        "test.s:2:22: Warning: Value 0x11d4ba truncated to 0xd4ba\n" },
    { "    s_movk_i32  s43, xc; xc = 0x22d4ba", 0xb02bd4baU, 0, false, true,
        "test.s:1:22: Warning: Value 0x22d4ba truncated to 0xd4ba\n" },
    { "    s_cmovk_i32  s43, 0xd3b9", 0xb12bd3b9U, 0, false, true, "" },
    { "    s_cmpk_eq_i32  s43, 0xd3b9", 0xb1abd3b9U, 0, false, true, "" },
    { "    s_cmpk_lg_i32  s43, 0xd3b9", 0xb22bd3b9U, 0, false, true, "" },
    { "    s_cmpk_gt_i32  s43, 0xd3b9", 0xb2abd3b9U, 0, false, true, "" },
    { "    s_cmpk_ge_i32  s43, 0xd3b9", 0xb32bd3b9U, 0, false, true, "" },
    { "    s_cmpk_lt_i32  s43, 0xd3b9", 0xb3abd3b9U, 0, false, true, "" },
    { "    s_cmpk_le_i32  s43, 0xd3b9", 0xb42bd3b9U, 0, false, true, "" },
    { "    s_cmpk_eq_u32  s43, 0xd3b9", 0xb4abd3b9U, 0, false, true, "" },
    { "    s_cmpk_lg_u32  s43, 0xd3b9", 0xb52bd3b9U, 0, false, true, "" },
    { "    s_cmpk_gt_u32  s43, 0xd3b9", 0xb5abd3b9U, 0, false, true, "" },
    { "    s_cmpk_ge_u32  s43, 0xd3b9", 0xb62bd3b9U, 0, false, true, "" },
    { "    s_cmpk_lt_u32  s43, 0xd3b9", 0xb6abd3b9U, 0, false, true, "" },
    { "    s_cmpk_le_u32  s43, 0xd3b9", 0xb72bd3b9U, 0, false, true, "" },
    { "    s_addk_i32  s43, 0xd3b9", 0xb7abd3b9U, 0, false, true, "" },
    { "    s_mulk_i32  s43, 0xd3b9", 0xb82bd3b9U, 0, false, true, "" },
    { "    s_cbranch_i_fork s[44:45], xxxx-8\nxxxx:\n", 0xb8acfffeU, 0, false, true, "" },
    { "xxxx:    s_cbranch_i_fork s[44:45], xxxx-8\n", 0xb8acfffdU, 0, false, true, "" },
    { "    s_cbranch_i_fork s[44:45], xxxx+8\nxxxx:\n", 0xb8ac0002U, 0, false, true, "" },
    { "xxxx:    s_cbranch_i_fork s[44:45], xxxx+16\n", 0xb8ac0003U, 0, false, true, "" },
    { "    s_cbranch_i_fork s[44:45], xxxx+9\nxxxx:\n", 0, 0, false, false,
        "test.s:1:32: Error: Jump is not aligned to word!\n" },
    { "    s_cbranch_i_fork s[44:45], xxxx+10\nxxxx:\n", 0, 0, false, false,
        "test.s:1:32: Error: Jump is not aligned to word!\n" },
    { "xxxx:    s_cbranch_i_fork s[44:45], xxxx+17\n", 0, 0, false, false,
        "test.s:1:44: Error: Jump is not aligned to word!\n" },
    { "    s_cbranch_i_fork s[44:45], xxxx+8\n.rodata\nxxxx:\n", 0, 0, false, false,
        "test.s:1:32: Error: Jump over current section!\n" },
    { ".rodata\nxxxx:.text\n    s_cbranch_i_fork s[44:45], xxxx+8", 0, 0, false, false,
        "test.s:3:32: Error: Jump over current section!\n" },
    /* hwregs */
    { "    s_getreg_b32    s43, hwreg(mode, 0, 1)", 0xb92b0001U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg  (mode, 0, 1)", 0xb92b0001U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg  (mode  ,   0  , 1  )",
                    0xb92b0001U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_MODE, 0, 1)", 0xb92b0001U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(status, 0, 1)", 0xb92b0002U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_STATUS, 0, 1)",
        0xb92b0002U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(trapsts, 0, 1)", 0xb92b0003U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_TRAPSTS, 0, 1)",
                    0xb92b0003U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(hw_id, 0, 1)", 0xb92b0004U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_HW_ID, 0, 1)",
                    0xb92b0004U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(gpr_alloc, 0, 1)", 0xb92b0005U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_GPR_ALLOC, 0, 1)",
                    0xb92b0005U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(lds_alloc, 0, 1)", 0xb92b0006U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_LDS_ALLOC, 0, 1)",
                    0xb92b0006U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(ib_sts, 0, 1)", 0xb92b0007U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_IB_STS, 0, 1)",
                    0xb92b0007U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(pc_lo, 0, 1)", 0xb92b0008U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_PC_LO, 0, 1)",
                    0xb92b0008U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(pc_hi, 0, 1)", 0xb92b0009U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_PC_HI, 0, 1)",
                    0xb92b0009U, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(inst_dw0, 0, 1)", 0xb92b000aU, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_INST_DW0, 0, 1)",
                    0xb92b000aU, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(inst_dw1, 0, 1)", 0xb92b000bU, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_INST_DW1, 0, 1)",
                    0xb92b000bU, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(ib_dbg0, 0, 1)", 0xb92b000cU, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(HWREG_IB_DBG0, 0, 1)",
                    0xb92b000cU, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(trapsts, 10, 1)", 0xb92b0283u, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(trapsts, 3, 10)", 0xb92b48c3u, 0, false, true, "" },
    { "    s_getreg_b32    s43, hwreg(trapsts, 3, 32)", 0xb92bf8c3u, 0, false, true, "" },
    { "    s_setreg_imm32_b32 hwreg(trapsts, 3, 10), 0x24da4f",
                    0xba8048c3u, 0x24da4fU, true, true, "" },
    { "    s_setreg_imm32_b32 hwreg(trapsts, 3, 10), xx; xx=0x24da4f",
                    0xba8048c3u, 0x24da4fU, true, true, "" },
    { "xx=0x24da4e;    s_setreg_imm32_b32 hwreg(trapsts, 3, 10), xx",
                    0xba8048c3u, 0x24da4eU, true, true, "" },
    { "     s_setreg_b32  hwreg(trapsts, 3, 10), s43", 0xb9ab48c3u, 0, false, true, "" },
    /* hw regs errors */
    { "    s_getreg_b32  s43, hwreg   (HWREG_MODE, 0, 2", 0, 0, false, false,
        "test.s:1:49: Error: Unterminated hwreg function\n" },
    { "    s_getreg_b32  s43, hwrX(mode, 0, 1)", 0, 0, false, false,
        "test.s:1:24: Error: Expected hwreg function\n" },
    { "    s_getreg_b32  s43, hwreg(mode, , )", 0, 0, false, false,
        "test.s:1:36: Error: Expected expression\n"
        "test.s:1:38: Error: Expected operator or value or symbol\n"
        "test.s:1:39: Error: Unterminated hwreg function\n" },
    { "    s_getreg_b32  s43, hwreg(XXX,6,8)", 0, 0, false, false,
        "test.s:1:30: Error: Unknown HWRegister\n" },
    /// s_getreg_b32  s43, hwreg(HW_REG_IB_DBG0, 0, 1)
    /* SOPP */
    { "s_nop  0x0", 0xbf800000U, 0, false, true, "" },
    { "s_nop  7", 0xbf800007U, 0, false, true, "" },
    { "s_nop  644", 0xbf800284U, 0, false, true, "" },
    { "xv=6; xt=65; s_nop  xv+xt", 0xbf800047U, 0, false, true, "" },
    { "s_nop  xv+xt; xv=6; xt=65", 0xbf800047U, 0, false, true, "" },
    { "s_endpgm", 0xbf810000U, 0, false, true, "" },
    { "    s_branch xxxx-8\nxxxx:\n", 0xbf82fffeU, 0, false, true, "" },
    { "xxxx:    s_branch xxxx-8\n", 0xbf82fffdU, 0, false, true, "" },
    { "    s_branch xxxx+8\nxxxx:\n", 0xbf820002U, 0, false, true, "" },
    { "xxxx:    s_branch xxxx+16\n", 0xbf820003U, 0, false, true, "" },
    { "    s_branch xxxx+9\nxxxx:\n", 0, 0, false, false,
        "test.s:1:14: Error: Jump is not aligned to word!\n" },
    { "    s_branch xxxx+10\nxxxx:\n", 0, 0, false, false,
        "test.s:1:14: Error: Jump is not aligned to word!\n" },
    { "xxxx:    s_branch xxxx+17\n", 0, 0, false, false,
        "test.s:1:26: Error: Jump is not aligned to word!\n" },
    { "    s_branch xxxx+8\n.rodata\nxxxx:\n", 0, 0, false, false,
        "test.s:1:14: Error: Jump over current section!\n" },
    { ".rodata\nxxxx:.text\n    s_branch xxxx+8", 0, 0, false, false,
        "test.s:3:14: Error: Jump over current section!\n" },
    { "    s_cbranch_scc0  xxxx-8\nxxxx:\n", 0xbf84fffeU, 0, false, true, "" },
    { "    s_cbranch_scc1  xxxx-8\nxxxx:\n", 0xbf85fffeU, 0, false, true, "" },
    { "    s_cbranch_vccz  xxxx-8\nxxxx:\n", 0xbf86fffeU, 0, false, true, "" },
    { "    s_cbranch_vccnz  xxxx-8\nxxxx:\n", 0xbf87fffeU, 0, false, true, "" },
    { "    s_cbranch_execz  xxxx-8\nxxxx:\n", 0xbf88fffeU, 0, false, true, "" },
    { "    s_cbranch_execnz  xxxx-8\nxxxx:\n", 0xbf89fffeU, 0, false, true, "" },
    { "s_barrier", 0xbf8a0000U, 0, false, true, "" },
    { "s_waitcnt  vmcnt(14) & expcnt(2)", 0xbf8c0f2eU, 0, false, true, "" },
    { "vmcnt=14\nexpcnt=2\ns_waitcnt  vmcnt(vmcnt) & expcnt(expcnt)",
        0xbf8c0f2eU, 0, false, true, "" },
    { "a=14\nb=2\ns_waitcnt  vmcnt(a) & expcnt(b)", 0xbf8c0f2eU, 0, false, true, "" },
    { "s_waitcnt  vmcnt  (14  ) & expcnt(  2)", 0xbf8c0f2eU, 0, false, true, "" },
    { "s_waitcnt  vmcnt  (  14  )&expcnt(  2  )", 0xbf8c0f2eU, 0, false, true, "" },
    { "s_waitcnt  VmCnt  (  14  )&exPCnt(  2  )", 0xbf8c0f2eU, 0, false, true, "" },
    { "s_waitcnt  lgkmcnt(13) & expcnt(2)", 0xbf8c0d2fU, 0, false, true, "" },
    { "s_waitcnt  vmcnt(14) & lgkmcnt(13)", 0xbf8c0d7eU, 0, false, true, "" },
    { "s_waitcnt  vmcnt(14)", 0xbf8c0f7eU, 0, false, true, "" },
    { "s_waitcnt  lgkmcnt(9)", 0xbf8c097fU, 0, false, true, "" },
    { "s_waitcnt  expcnt(2)", 0xbf8c0f2fU, 0, false, true, "" },
    { "s_waitcnt  vmcnt(0) & expcnt(0) & lgkmcnt(0)", 0xbf8c0000U, 0, false, true, "" },
    { "s_waitcnt  vmcnt(11) & expcnt(4) & lgkmcnt(12)", 0xbf8c0c4bU, 0, false, true, "" },
    /* waitcnt warnings */
    { "s_waitcnt  vmcnt(33) & expcnt(10) & lgkmcnt(27)", 0xbf8c0b21U, 0, false, true,
        "test.s:1:18: Warning: Value out of range\n"
        "test.s:1:31: Warning: Value out of range\n"
        "test.s:1:45: Warning: Value out of range\n" },
    { "s_waitcnt  vmcnt(14) & expcnt(2)&expcnt(3)", 0xbf8c0f3eU, 0, false, true,
        "test.s:1:34: Warning: expcnt was already defined\n" },
    { "s_waitcnt  vmcnt(14) & expcnt(2)&vmcnt(12)", 0xbf8c0f2cU, 0, false, true,
        "test.s:1:34: Warning: vmcnt was already defined\n" },
    { "s_waitcnt  lgkmcnt(13) & expcnt(2) & lgkmcnt(11)", 0xbf8c0b2fU, 0, false, true,
        "test.s:1:38: Warning: lgkmcnt was already defined\n" },
    /* waitcnt errors */
    { "s_waitcnt  vmcnt(11) & expcnt() & lgkmcnt(12)", 0, 0, false, false,
        "test.s:1:31: Error: Expected operator or value or symbol\n"
        "test.s:1:33: Error: Expected primary expression before operator\n"
        "test.s:1:35: Error: Expression have unresolved symbol 'lgkmcnt'\n"
        "test.s:1:42: Error: Expected operator\n"
        "test.s:1:43: Error: Unterminated function\n" },
    { "s_waitcnt  vmcnt(11) & expcnt(3", 0, 0, false, false,
        "test.s:1:32: Error: Unterminated function\n" },
    { "s_waitcnt  vmcxxnt(11)", 0, 0, false, false,
        "test.s:1:12: Error: Expected vmcnt, lgkmcnt or expcnt\n" },
    { "s_waitcnt  vmcnt(11) _", 0, 0, false, false,
        "test.s:1:22: Error: Expected '&' before lock function\n" },
    { "s_waitcnt  vmcnt  (14  ) & expcnt(  2) & aaaa(6) & vmcnt(4)", 0, 0, false, false,
        "test.s:1:42: Error: Expected vmcnt, lgkmcnt or expcnt\n"
        "test.s:1:52: Warning: vmcnt was already defined\n" },
    /* other SOPP opcodes */
    { "    s_sethalt       0x32b", 0xbf8d032bU, 0, false, true, "" },
    { "    s_sleep         0x32b", 0xbf8e032bU, 0, false, true, "" },
    { "    s_setprio       0x32b", 0xbf8f032bU, 0, false, true, "" },
    /* sendmsg */
    { "    s_sendmsg  sendmsg(interrupt)", 0xbf900001U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_INTERRUPT)", 0xbf900001U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(system)", 0xbf90000fU, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_SYSTEM)", 0xbf90000fU, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_SYSMSG)", 0xbf90000fU, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(gs, nop)", 0xbf900002U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_GS, GS_OP_NOP)", 0xbf900002U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(gs, cut, 0)", 0xbf900012U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_GS, GS_OP_CUT, 0)", 0xbf900012U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(gs, cut, 1)", 0xbf900112U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_GS, GS_OP_CUT, 1)", 0xbf900112U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(gs, emit, 0)", 0xbf900022U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_GS, GS_OP_EMIT, 0)", 0xbf900022U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(gs, emit_cut, 3)", 0xbf900332U, 0,
        false, true, "" },
    { "    s_sendmsg  sendmsg ( gs    , emit_cut  , 3  )", 0xbf900332U, 0,
        false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_GS, GS_OP_EMIT_CUT, 3)", 0xbf900332U, 0,
        false, true, "" },
    { "    s_sendmsg  sendmsg(gs_done, nop)", 0xbf900003U, 0, false, true, "" },
    { "    s_sendmsg  sendmsg(MSG_GS_DONE, GS_OP_NOP)", 0xbf900003U, 0, false, true, "" },
    { "    s_sendmsghalt  sendmsg(interrupt)", 0xbf910001U, 0, false, true, "" },
    /* msg errors */
    { "    s_sendmsg  sendmsg(interrupxt)", 0, 0, false, false,
        "test.s:1:24: Error: Unknown message\n" },
    { "    s_sendmsg  sendmsg(gs, nop__x)", 0, 0, false, false,
        "test.s:1:28: Error: Unknown GSOP\n" },
    { "    s_sendmsg  sendmsg(gs, nop__x", 0, 0, false, false,
        "test.s:1:28: Error: Unknown GSOP\n"
        "test.s:1:34: Error: Unterminated sendmsg function\n" },
    { "    s_sendmsg  sendmsg(gs, cut, 122)", 0xbf900212U, 0, false, true,
        "test.s:1:33: Warning: StreamId (3rd argument) out of range\n" },
    /* other SOPP encoding */
    { "    s_trap  0x32b", 0xbf92032bU, 0, false, true, "" },
    { "    s_icache_inv", 0xbf930000U, 0, false, true, "" },
    { "    s_incperflevel  0x1234", 0xbf941234U, 0, false, true, "" },
    { "    s_decperflevel  0x1234", 0xbf951234U, 0, false, true, "" },
    { "    s_ttracedata", 0xbf960000U, 0, false, true, "" },
    /* SMRD encoding */
    { "    s_load_dword    s50, s[58:59], 0x5b", 0xc0193b5bU, 0, false, true, "" },
    { "    s_load_dword    s50, s[58:59], s91", 0xc0193a5bU, 0, false, true, "" },
    { "    s_load_dword    s51, s[58:59], s91", 0xc019ba5bU, 0, false, true, "" },
    { "s0=191;    s_load_dword    s50, s[58:59], @s0", 0xc0193bbfU, 0, false, true, "" },
    { "    s_load_dword    s50, s[58:59], @s0; s0=191", 0xc0193bbfU, 0, false, true, "" },
    { "x0=191;    s_load_dword    s50, s[58:59], x0", 0xc0193bbfU, 0, false, true, "" },
    { "    s_load_dword    s50, s[58:59], x0; x0=191", 0xc0193bbfU, 0, false, true, "" },
    { "    s_load_dwordx2  s[52:53], s[58:59], s83", 0xc05a3a53U, 0, false, true, "" },
    { "    s_load_dwordx4  s[52:55], s[58:59], s83", 0xc09a3a53U, 0, false, true, "" },
    { "    s_load_dwordx8  s[52:59], s[58:59], s83", 0xc0da3a53U, 0, false, true, "" },
    { "    s_load_dwordx16  s[52:67], s[58:59], s83", 0xc11a3a53U, 0, false, true, "" },
    { "    s_buffer_load_dword    s50, s[60:63], 191", 0xc2193dbfU, 0, false, true, "" },
    { "    s_buffer_load_dwordx2  s[52:53], s[60:63], s83",
        0xc25a3c53U, 0, false, true, "" },
    { "    s_buffer_load_dwordx4  s[52:55], s[60:63], s83",
        0xc29a3c53U, 0, false, true, "" },
    { "    s_buffer_load_dwordx8  s[52:59], s[60:63], s83",
        0xc2da3c53U, 0, false, true, "" },
    { "    s_buffer_load_dwordx16  s[52:67], s[60:63], s83",
        0xc31a3c53U, 0, false, true, "" },
    /* vccz as variable */
    { "    s_load_dword    s50, s[58:59], vccz; vccz=33",
        0xc0193b21U, 0, false, true, "" },
    /* other with only sdst */
    { "    s_memtime  s[50:51]", 0xc7990000U, 0, false, true, "" },
    { "    s_dcache_inv", 0xc7c00000U, 0, false, true, "" },
    /* SMRD warnings */
    { "s0=3191; s_load_dword    s50, s[58:59], @s0", 0xc0193b77U, 0, false, true,
        "test.s:1:42: Warning: Value 0xc77 truncated to 0x77\n" },
    { "s_load_dword    s50, s[58:59], @s0; s0=3191", 0xc0193b77U, 0, false, true,
        "test.s:1:33: Warning: Value 0xc77 truncated to 0x77\n" },
    /* SMRD errors */
    { "    s_load_dword    vccz, s[58:59], 0x5b", 0, 0, false, false,
        "test.s:1:21: Error: Expected 1 scalar register\n" },
    { "    s_load_dword    s50, s[59:60], 0x5b", 0, 0, false, false,
        "test.s:1:26: Error: Unaligned scalar register range\n" },
    /* VOP2/(VOP2 in VOP3) encoding */
    { "    v_cndmask_b32   v154, v21, v107, vcc", 0x0134d715U, 0, false, true, "" },
    { "    v_cndmask_b32   v154, v21, v107, vcc     ", 0x0134d715U, 0, false, true, "" },
    { "    v_cndmask_b32   v[154:154], v[21:21], v[107], vcc",
        0x0134d715U, 0, false, true, "" },
    { "    v_cndmask_b32   v[  154 : 154], v[ 21:21 ], v[ 107  ], vcc",
        0x0134d715U, 0, false, true, "" },
    { "    v_cndmask_b32   v154  , v21 , v107  , vcc", 0x0134d715U, 0, false, true, "" },
    { "    v_cndmask_b32   v154, lds, v107, vcc", 0x0134d6feU, 0, false, true, "" },
    { "    v_cndmask_b32   v154, lds_direct, v107, vcc", 0x0134d6feU, 0, false, true, "" },
    { "    v_cndmask_b32   v154, src_lds_direct, v107, vcc",
        0x0134d6feU, 0, false, true, "" },
    { "    v_add_f32   v154, s21, v107", 0x0734d615U, 0, false, true, "" },
    { "    v_cndmask_b32   v154, 2, v107, vcc", 0x0134d682U, 0, false, true, "" },
    { "    v_cndmask_b32   v154, -9, v107, vcc", 0x0134d6c9U, 0, false, true, "" },
    { "    v_cndmask_b32   v154, 20e-1, v107, vcc", 0x0134d6f4U, 0, false, true, "" },
    { "    v_cndmask_b32   v154, 1323, v107, vcc", 0x0134d6ffU, 1323, true, true, "" },
    { "xx=1323;v_cndmask_b32   v154, xx, v107, vcc", 0x0134d6ffU, 1323, true, true, "" },
    { "v_cndmask_b32   v154, xx, v107, vcc; xx=1323", 0x0134d6ffU, 1323, true, true, "" },
    { "v_cndmask_b32   v154, xx, v107, vcc; xx=2", 0x0134d6ffU, 2, true, true, "" },
    { "v0=1323;v_cndmask_b32   v154, @v0, v107, vcc", 0x0134d6ffU, 1323, true, true, "" },
    { "v_cndmask_b32   v154, @v0, v107, vcc; v0=1323", 0x0134d6ffU, 1323, true, true, "" },
    { "s0=1323;v_cndmask_b32   v154, @s0, v107, vcc", 0x0134d6ffU, 1323, true, true, "" },
    { "v_cndmask_b32   v154, @s0, v107, vcc; s0=1323", 0x0134d6ffU, 1323, true, true, "" },
    { "    v_readlane_b32  s26, v21, s94", 0x0234bd15U, 0, false, true, "" },
    { "    v_writelane_b32  v26, v21, s94", 0x0434bd15U, 0, false, true, "" },
    { "    v_add_f32  v154, v21, v107", 0x0734d715U, 0, false, true, "" },
    /* register names */
    { "tx=%v21; ty=%v107; v_add_f32  v154, tx, ty", 0x0734d715U, 0, false, true, "" },
    { "tx=%v[16:23]; ty=%v107; v_add_f32  v154, tx[5], ty",
        0x0734d715U, 0, false, true, "" },
    { "tx=%s21; v_add_f32  v154, tx, v107", 0x0734d615U, 0, false, true, "" },
    { "    v_add_f32  v154, s21, v107", 0x0734d615U, 0, false, true, "" },
    { "    v_add_f32  v154, v21, v107 vop3", 0xd206009aU, 0x0002d715U, true, true, "" },
    { "    v_add_f32  v154, v21, s98", 0xd206009aU, 0x0000c515U, true, true, "" },
    { "    v_add_f32  v154, abs(v21), v107", 0xd206019aU, 0x0002d715U, true, true, "" },
    { "    v_add_f32  v154, abs(v21), abs(v107)",
        0xd206039aU, 0x0002d715U, true, true, "" },
    { "    v_add_f32  v154, v21, abs(v107)", 0xd206029aU, 0x0002d715U, true, true, "" },
    { "    v_add_f32  v154, -v21, v107", 0xd206009aU, 0x2002d715U, true, true, "" },
    { "    v_add_f32  v154, v21, -v107", 0xd206009aU, 0x4002d715U, true, true, "" },
    { "    v_add_f32  v154, -v21, -v107", 0xd206009aU, 0x6002d715U, true, true, "" },
    { "    v_add_f32  v154, -abs(v21), -abs(v107)",
        0xd206039aU, 0x6002d715U, true, true, "" },
    { "    v_add_f32  v154, v21, v107 mul:2", 0xd206009aU, 0x0802d715U, true, true, "" },
    { "    v_add_f32  v154, v21, v107 mul:002", 0xd206009aU, 0x0802d715U, true, true, "" },
    { "    v_add_f32  v154, v21, v107 mul:4", 0xd206009aU, 0x1002d715U, true, true, "" },
    { "    v_add_f32  v154, v21, v107 div:2", 0xd206009aU, 0x1802d715U, true, true, "" },
    { "    v_add_f32  v154, v21, v107 div:002", 0xd206009aU, 0x1802d715U, true, true, "" },
    { "    v_add_f32  v154, v21, v107 mul:2 clamp",
        0xd206089aU, 0x0802d715U, true, true, "" },
    { "    v_add_f32  v154, v21, v107 clamp", 0xd206089aU, 0x0002d715U, true, true, "" },
    { "    v_cndmask_b32   v154, v21, v107, s[6:7]",
        0xd200009aU, 0x001ad715U, true, true, "" },
    { "    v_cndmask_b32   v154, abs(v21), abs(v107), s[6:7] clamp",
        0xd2000b9aU, 0x001ad715U, true, true, "" },
    /* negated expressions */
    { "vx=7;    v_add_f32  v154, -vx, v107", 0x0734d6c7U, 0, false, true, "" },
    { "v12ZZ=7;    v_add_f32  v154, -v12ZZ, v107", 0x0734d6c7U, 0, false, true, "" },
    { "v_add_f32  v154, -vx, v107; vx=7", 0x0734d6ffU, uint32_t(-7), true, true, "" },
    { "vx=7737;    v_add_f32  v154, -vx, v107",
        0x0734d6ffU, uint32_t(-7737), true, true, "" },
    { "    v_add_f32  v154, s2, s2", 0xd206009aU, 0x0402U, true, true, "" },
    /* var long names */
    { "xtmp1111111111111111111222222222=67777\n"
        "v_add_f32  v154, xtmp1111111111111111111222222222, v107",
        0x0734d6ffU, 67777, true, true, "" },
    /* VOP2 errors */
    { "    v_cndmask_b32   v[154:155], v21, v107, vcc", 0, 0, false, false,
        "test.s:1:21: Error: Required 1 vector register\n" },
    { "    v_cndmask_b32   v154, v[21:22], v107, vcc", 0, 0, false, false,
        "test.s:1:27: Error: Required 1 vector register\n" },
    { "    v_cndmask_b32   v154, v21, v[107:108], vcc", 0, 0, false, false,
        "test.s:1:32: Error: Required 1 vector register\n" },
    { "    v_cndmask_b32   v[154:, v21, v107, vcc", 0, 0, false, false,
        "test.s:1:27: Error: Expected expression\n" },
    { "    v_cndmask_b32   v[14, v21, v107, vcc", 0, 0, false, false,
        "test.s:1:21: Error: Unterminated vector register range\n" },
    { "    v_cndmask_b32   v[14:15, v21, v107, vcc", 0, 0, false, false,
        "test.s:1:21: Error: Unterminated vector register range\n" },
    { "    v_cndmask_b32   v256, v21, v107, vcc", 0, 0, false, false,
        "test.s:1:24: Error: Number is too big\n"
        "test.s:1:24: Error: Expected ',' before argument\n" },
    { "    v_cndmask_b32   v[255:256], v21, v107, vcc", 0, 0, false, false,
        "test.s:1:21: Error: Some vector register number out of range\n" },
    { "    v_add_f32  v154, v21, v107 mul,", 0, 0, false, false,
        "test.s:1:35: Error: Expected ':' before multiplier number\n"
        "test.s:1:35: Error: Some garbages at VOP modifier place\n" },
    { "    v_add_f32  v154, v21, v107 div,", 0, 0, false, false,
        "test.s:1:35: Error: Expected ':' before divider number\n"
        "test.s:1:35: Error: Some garbages at VOP modifier place\n" },
    /* VOP2 in VOP3 errors */
    { "    v_add_f32  v154, v21, v107 mul:3", 0, 0, false, false,
        "test.s:1:32: Error: Unknown VOP3 mul:X modifier\n" },
    { "    v_add_f32  v154, v21, v107 mul:", 0, 0, false, false,
        "test.s:1:36: Error: Missing number\n" },
    { "    v_add_f32  v154, v21, v107 mul", 0, 0, false, false,
        "test.s:1:35: Error: Expected ':' before multiplier number\n" },
    { "    v_add_f32  v154, v21, v107 mxl", 0, 0, false, false,
        "test.s:1:32: Error: Unknown VOP modifier\n" },
    { "    v_add_f32  v154, abs(v21), 12343", 0, 0, false, false,
        "test.s:1:5: Error: Literal in VOP3 encoding is illegal\n" },
    { "    v_add_f32  v154, 13243, abs(v21)", 0, 0, false, false,
        "test.s:1:5: Error: Literal in VOP3 encoding is illegal\n" },
    { "    v_add_f32  v154, 3123, s3", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_add_f32  v154, 3123, m0", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_add_f32  v154, s3, 3123", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_add_f32  v154, m0, 3123", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_add_f32  v154, s2, s4", 0, 0, false, false,
        "test.s:1:5: Error: More than one SGPR to read in instruction\n" },
    /* out holds scalar register (not vector) */
    { "out=%s64;v_add_f32  out, s21, v107", 0, 0, false, false,
        "test.s:1:21: Error: Expected 1 vector register\n" },
    { "out=%v[64:65];v_add_f32  out, s21, v107", 0, 0, false, false,
        "test.s:1:26: Error: Required 1 vector register\n" },
    /* OMOD already defined */
    { "    v_add_f32  v154, v21, v107 mul:4 div:2", 0xd206009aU, 0x1802d715U, true, true, 
        "test.s:1:38: Warning: OMOD is already defined\n" },
    /* rest of the VOP2 pcodes */
    { "    v_sub_f32  v154, v21, v107", 0x0934d715U, 0, false, true, "" },
    { "    v_sub_f32  v55, s27, -v90", 0xd2080037U, 0x4002b41bU, true, true, "" },
    { "    v_subrev_f32  v154, v21, v107", 0x0b34d715U, 0, false, true, "" },
    { "    v_subrev_f32  v55, s27, -v90", 0xd20a0037U, 0x4002b41bU, true, true, "" },
    { "    v_mac_legacy_f32  v154, v21, v107", 0x0d34d715U, 0, false, true, "" },
    { "    v_mac_legacy_f32  v55, s27, -v90", 0xd20c0037U, 0x4002b41bU, true, true, "" },
    { "    v_mul_legacy_f32  v154, v21, v107", 0x0f34d715U, 0, false, true, "" },
    { "    v_mul_legacy_f32  v55, s27, -v90", 0xd20e0037U, 0x4002b41bU, true, true, "" },
    { "    v_mul_f32  v154, v21, v107", 0x1134d715U, 0, false, true, "" },
    { "    v_mul_f32  v55, s27, -v90", 0xd2100037U, 0x4002b41bU, true, true, "" },
    { "    v_mul_i32_i24  v154, v21, v107", 0x1334d715U, 0, false, true, "" },
    { "    v_mul_i32_i24  v55, s27, -v90", 0xd2120037U, 0x4002b41bU, true, true, "" },
    { "    v_mul_hi_i32_i24  v154, v21, v107", 0x1534d715U, 0, false, true, "" },
    { "    v_mul_hi_i32_i24  v55, s27, -v90", 0xd2140037U, 0x4002b41bU, true, true, "" },
    { "    v_mul_u32_u24  v154, v21, v107", 0x1734d715U, 0, false, true, "" },
    { "    v_mul_u32_u24  v55, s27, -v90", 0xd2160037U, 0x4002b41bU, true, true, "" },
    { "    v_mul_hi_u32_u24  v154, v21, v107", 0x1934d715U, 0, false, true, "" },
    { "    v_mul_hi_u32_u24  v55, s27, -v90", 0xd2180037U, 0x4002b41bU, true, true, "" },
    { "    v_min_legacy_f32  v154, v21, v107", 0x1b34d715U, 0, false, true, "" },
    { "    v_min_legacy_f32  v55, s27, -v90", 0xd21a0037U, 0x4002b41bU, true, true, "" },
    { "    v_max_legacy_f32  v154, v21, v107", 0x1d34d715U, 0, false, true, "" },
    { "    v_max_legacy_f32  v55, s27, -v90", 0xd21c0037U, 0x4002b41bU, true, true, "" },
    { "    v_min_f32  v154, v21, v107", 0x1f34d715U, 0, false, true, "" },
    { "    v_min_f32  v55, s27, -v90", 0xd21e0037U, 0x4002b41bU, true, true, "" },
    { "    v_max_f32  v154, v21, v107", 0x2134d715U, 0, false, true, "" },
    { "    v_max_f32  v55, s27, -v90", 0xd2200037U, 0x4002b41bU, true, true, "" },
    { "    v_min_i32  v154, v21, v107", 0x2334d715U, 0, false, true, "" },
    { "    v_min_i32  v55, s27, -v90", 0xd2220037U, 0x4002b41bU, true, true, "" },
    { "    v_max_i32  v154, v21, v107", 0x2534d715U, 0, false, true, "" },
    { "    v_max_i32  v55, s27, -v90", 0xd2240037U, 0x4002b41bU, true, true, "" },
    { "    v_min_u32  v154, v21, v107", 0x2734d715U, 0, false, true, "" },
    { "    v_min_u32  v55, s27, -v90", 0xd2260037U, 0x4002b41bU, true, true, "" },
    { "    v_max_u32  v154, v21, v107", 0x2934d715U, 0, false, true, "" },
    { "    v_max_u32  v55, s27, -v90", 0xd2280037U, 0x4002b41bU, true, true, "" },
    { "    v_lshr_b32  v154, v21, v107", 0x2b34d715U, 0, false, true, "" },
    { "    v_lshr_b32  v55, s27, -v90", 0xd22a0037U, 0x4002b41bU, true, true, "" },
    { "    v_lshrrev_b32  v154, v21, v107", 0x2d34d715U, 0, false, true, "" },
    { "    v_lshrrev_b32  v55, s27, -v90", 0xd22c0037U, 0x4002b41bU, true, true, "" },
    { "    v_ashr_i32  v154, v21, v107", 0x2f34d715U, 0, false, true, "" },
    { "    v_ashr_i32  v55, s27, -v90", 0xd22e0037U, 0x4002b41bU, true, true, "" },
    { "    v_ashrrev_i32  v154, v21, v107", 0x3134d715U, 0, false, true, "" },
    { "    v_ashrrev_i32  v55, s27, -v90", 0xd2300037U, 0x4002b41bU, true, true, "" },
    { "    v_lshl_b32  v154, v21, v107", 0x3334d715U, 0, false, true, "" },
    { "    v_lshl_b32  v55, s27, -v90", 0xd2320037U, 0x4002b41bU, true, true, "" },
    { "    v_lshlrev_b32  v154, v21, v107", 0x3534d715U, 0, false, true, "" },
    { "    v_lshlrev_b32  v55, s27, -v90", 0xd2340037U, 0x4002b41bU, true, true, "" },
    { "    v_and_b32  v154, v21, v107", 0x3734d715U, 0, false, true, "" },
    { "    v_and_b32  v55, s27, -v90", 0xd2360037U, 0x4002b41bU, true, true, "" },
    { "    v_or_b32  v154, v21, v107", 0x3934d715U, 0, false, true, "" },
    { "    v_or_b32  v55, s27, -v90", 0xd2380037U, 0x4002b41bU, true, true, "" },
    { "    v_xor_b32  v154, v21, v107", 0x3b34d715U, 0, false, true, "" },
    { "    v_xor_b32  v55, s27, -v90", 0xd23a0037U, 0x4002b41bU, true, true, "" },
    { "    v_bfm_b32  v154, v21, v107", 0x3d34d715U, 0, false, true, "" },
    { "    v_bfm_b32  v55, s27, -v90", 0xd23c0037U, 0x4002b41bU, true, true, "" },
    { "    v_mac_f32  v154, v21, v107", 0x3f34d715U, 0, false, true, "" },
    { "    v_mac_f32  v55, s27, -v90", 0xd23e0037U, 0x4002b41bU, true, true, "" },
    { "    v_madmk_f32 v154, v21, 6.9551627e+13, v107",
        0x4134d715U, 0x567d0700U, true, true, "" },
    { "    v_madmk_f32 v154, v21, 45543, v107", 0x4134d715U, 45543, true, true, "" },
    { "cxx=77111;v_madmk_f32 v154, v21, cxx, v107", 0x4134d715U, 77111, true, true, "" },
    { "v_madmk_f32 v154, v21, cxx, v107; cxx=77111", 0x4134d715U, 77111, true, true, "" },
    { "    v_madak_f32 v154, v21, v107, 6.9551627e+13",
        0x4334d715U, 0x567d0700U, true, true, "" },
    { "    v_madak_f32 v154, v21, v107, 6.9551627e+13s",
        0x4334d715U, 0x567d0700U, true, true, "" },
    { "    v_madak_f32 v154, v21, v107, 6.9551627h", 0x4334d715U, 0x46f5U, true, true, "" },
    { "    v_madak_f32 v154, v21, v107, 45543", 0x4334d715U, 45543, true, true, "" },
    { "cxx=77111;v_madak_f32 v154, v21, v107, cxx", 0x4334d715U, 77111, true, true, "" },
    { "v_madak_f32 v154, v21, v107, cxx; cxx=77111", 0x4334d715U, 77111, true, true, "" },
    { "    v_bcnt_u32_b32  v154, v21, v107", 0x4534d715U, 0, false, true, "" },
    { "    v_bcnt_u32_b32  v55, s27, -v90", 0xd2440037U, 0x4002b41bU, true, true, "" },
    { "    v_mbcnt_lo_u32_b32  v154, v21, v107", 0x4734d715U, 0, false, true, "" },
    { "    v_mbcnt_lo_u32_b32  v55, s27, -v90", 0xd2460037U, 0x4002b41bU, true, true, "" },
    { "    v_mbcnt_hi_u32_b32  v154, v21, v107", 0x4934d715U, 0, false, true, "" },
    { "    v_mbcnt_hi_u32_b32  v55, s27, -v90", 0xd2480037U, 0x4002b41bU, true, true, "" },
    { "    v_add_i32  v154, vcc, v21, v107", 0x4b34d715U, 0, false, true, "" },
    { "    v_add_i32  v55, s[10:11], s27, -v90",
        0xd24a0a37U, 0x4002b41bU, true, true, "" },
    { "    v_sub_i32  v154, vcc, v21, v107", 0x4d34d715U, 0, false, true, "" },
    { "    v_sub_i32  v55, s[10:11], s27, -v90",
        0xd24c0a37U, 0x4002b41bU, true, true, "" },
    { "    v_sub_i32  v55, s[11:12], s27, -v90",
        0xd24c0b37U, 0x4002b41bU, true, true, "" },
    { "    v_subrev_i32  v154, vcc, v21, v107", 0x4f34d715U, 0, false, true, "" },
    { "    v_subrev_i32  v55, s[10:11], s27, -v90",
        0xd24e0a37U, 0x4002b41bU, true, true, "" },
    { "    v_addc_u32  v154, vcc, v21, v107, vcc", 0x5134d715U, 0, false, true, "" },
    { "    v_subb_u32  v154, vcc, v21, v107, vcc", 0x5334d715U, 0, false, true, "" },
    { "    v_subbrev_u32  v154, vcc, v21, v107, vcc", 0x5534d715U, 0, false, true, "" },
    /* VOP3B errors */
    { "    v_add_i32  v55, s[10:11], s27, abs(v90)", 0, 0, false, false,
        "test.s:1:39: Error: Expected operator\n"
        "test.s:1:40: Error: Unknown VOP modifier\n"
        "test.s:1:43: Error: Some garbages at VOP modifier place\n" },
    { "    v_add_i32  v55, s[10:11], abs(s27), v90", 0, 0, false, false,
        "test.s:1:34: Error: Expected operator\n"
        "test.s:1:35: Error: Expected ',' before argument\n" },
    { "    v_add_i32  v55, s[10:11], s27, v90 clamp", 0, 0, false, false,
        "test.s:1:40: Error: Modifier CLAMP in VOP3B is illegal\n" },
    /* rest of the VOP2 */
    { "    v_ldexp_f32  v154, v21, v107", 0x5734d715U, 0, false, true, "" },
    { "    v_ldexp_f32  v55, s27, -v90", 0xd2560037U, 0x4002b41bU, true, true, "" },
    { "    v_cvt_pkaccum_u8_f32 v154, v21, v107", 0x5934d715U, 0, false, true, "" },
    { "    v_cvt_pkaccum_u8_f32 v55, s27, -v90",
        0xd2580037U, 0x4002b41bU, true, true, "" },
    { "    v_cvt_pknorm_i16_f32 v154, v21, v107", 0x5b34d715U, 0, false, true, "" },
    { "    v_cvt_pknorm_i16_f32 v55, s27, -v90",
        0xd25a0037U, 0x4002b41bU, true, true, "" },
    { "    v_cvt_pknorm_u16_f32 v154, v21, v107", 0x5d34d715U, 0, false, true, "" },
    { "    v_cvt_pknorm_u16_f32 v55, s27, -v90",
        0xd25c0037U, 0x4002b41bU, true, true, "" },
    { "    v_cvt_pkrtz_f16_f32 v154, v21, v107", 0x5f34d715U, 0, false, true, "" },
    { "    v_cvt_pkrtz_f16_f32 v55, s27, -v90", 0xd25e0037U, 0x4002b41bU, true, true, "" },
    { "    v_cvt_pk_u16_u32  v154, v21, v107", 0x6134d715U, 0, false, true, "" },
    { "    v_cvt_pk_u16_u32  v55, s27, -v90", 0xd2600037U, 0x4002b41bU, true, true, "" },
    { "    v_cvt_pk_i16_i32  v154, v21, v107", 0x6334d715U, 0, false, true, "" },
    { "    v_cvt_pk_i16_i32  v55, s27, -v90", 0xd2620037U, 0x4002b41bU, true, true, "" },
    /* VOP1 and VOP1 in VOP3 */
    { "    v_nop", 0x7e000000U, 0, false, true, "" },
    { "    v_nop vop3", 0xd3000000U, 0, true, true, "" },
    { "    v_mov_b32  v158, s79", 0x7f3c024fU, 0, false, true, "" },
    { "    v_mov_b32  v15, s79", 0x7e1e024fU, 0, false, true, "" },
    { "    v_mov_b32  v15, v79", 0x7e1e034fU, 0, false, true, "" },
    { "vxx=9; v_mov_b32  v15, vxx", 0x7e1e0289U, 0, false, true, "" },
    { "vxx=9111; v_mov_b32  v15, vxx", 0x7e1e02ffU, 9111, true, true, "" },
    { "v_mov_b32  v15, vxx; vxx=9", 0x7e1e02ffU, 9, true, true, "" },
    { "v_mov_b32  v15, vxx; vxx=9111", 0x7e1e02ffU, 9111, true, true, "" },
    { "v0=9111; v_mov_b32  v15, @v0", 0x7e1e02ffU, 9111, true, true, "" },
    { "v_mov_b32  v15, @v0; v0=9", 0x7e1e02ffU, 9, true, true, "" },
    { "    v_mov_b32  v15, lds", 0x7e1e02feU, 0, false, true, "" },
    { "    v_mov_b32  v15, lds_direct", 0x7e1e02feU, 0, false, true, "" },
    { "    v_mov_b32  v55, s27 vop3", 0xd3020037U, 0x0000001bU, true, true, "" },
    { "    v_mov_b32  v55, v27 vop3", 0xd3020037U, 0x0000011bU, true, true, "" },
    { "    v_mov_b32  v55, v27 clamp", 0xd3020837U, 0x0000011bU, true, true, "" },
    { "    v_mov_b32  v55, abs(s27)", 0xd3020137U, 0x0000001bU, true, true, "" },
    { "    v_mov_b32  v55, -s27", 0xd3020037U, 0x2000001bU, true, true, "" },
    { "    v_mov_b32  v55, -abs(s27)", 0xd3020137U, 0x2000001bU, true, true, "" },
    { "    v_readfirstlane_b32 s30, s79", 0x7e3c044fU, 0, false, true, "" },
    { "    v_readfirstlane_b32 s30, v79", 0x7e3c054fU, 0, false, true, "" },
    { "    v_cvt_i32_f64  v158, v[79:80]", 0x7f3c074fU, 0, false, true, "" },
    { "    v_cvt_i32_f64  v55, abs(v[27:28])", 0xd3060137U, 0x0000011bU, true, true, "" },
    { "    v_cvt_i32_f64  v55, abs(s[26:27])", 0xd3060137U, 0x0000001aU, true, true, "" },
    { "    v_cvt_i32_f64  v55, -s[26:27]", 0xd3060037U, 0x2000001aU, true, true, "" },
    { "    v_cvt_f32_i32  v158, v79", 0x7f3c0b4fU, 0, false, true, "" },
    { "    v_cvt_f32_i32  v55, v27 vop3", 0xd30a0037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_u32  v158, v79", 0x7f3c0d4fU, 0, false, true, "" },
    { "    v_cvt_f32_u32  v55, v27 vop3", 0xd30c0037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_u32_f32  v158, v79", 0x7f3c0f4fU, 0, false, true, "" },
    { "    v_cvt_u32_f32  v55, v27 vop3", 0xd30e0037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_i32_f32  v158, v79", 0x7f3c114fU, 0, false, true, "" },
    { "    v_cvt_i32_f32  v55, v27 vop3", 0xd3100037U, 0x0000011bU, true, true, "" },
    { "    v_mov_fed_b32  v158, v79", 0x7f3c134fU, 0, false, true, "" },
    { "    v_mov_fed_b32  v55, v27 vop3", 0xd3120037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f16_f32  v158, v79", 0x7f3c154fU, 0, false, true, "" },
    { "    v_cvt_f16_f32  v55, v27 vop3", 0xd3140037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_f16  v158, v79", 0x7f3c174fU, 0, false, true, "" },
    { "    v_cvt_f32_f16  v55, v27 vop3", 0xd3160037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_f16  v158, -23.56", 0x7f3c16ffU, 0xcde4U, true, true, "" },
    /* constant immediates */
    { "    v_cvt_f32_f16  v158, 0.0", 0x7f3c1680U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, 0.5s", 0x7f3c16f0U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, -0.5s", 0x7f3c16f1U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, 1.s", 0x7f3c16f2U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, -1.s", 0x7f3c16f3U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, 2.s", 0x7f3c16f4U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, -2.s", 0x7f3c16f5U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, 4.s", 0x7f3c16f6U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, -4.s", 0x7f3c16f7U, 0, false, true, "" },
    { "    v_cvt_f32_f16  v158, 0.5", 0x7f3c16ffU, 0x3800, true, true, "" },
    { "    v_cvt_f32_f16  v158, 0.5h", 0x7f3c16ffU, 0x3800, true, true, "" },
    /* rest of VOP1 instructions */
    { "    v_cvt_rpi_i32_f32  v158, v79", 0x7f3c194fU, 0, false, true, "" },
    { "    v_cvt_rpi_i32_f32  v55, v27 vop3", 0xd3180037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_flr_i32_f32  v158, v79", 0x7f3c1b4fU, 0, false, true, "" },
    { "    v_cvt_flr_i32_f32  v55, v27 vop3", 0xd31a0037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_off_f32_i4  v158, v79", 0x7f3c1d4fU, 0, false, true, "" },
    { "    v_cvt_off_f32_i4  v55, v27 vop3", 0xd31c0037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_f64  v158, v[79:80]", 0x7f3c1f4fU, 0, false, true, "" },
    { "    v_cvt_f32_f64  v55, v[27:28] vop3", 0xd31e0037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f64_f32  v[158:159], v79", 0x7f3c214fU, 0, false, true, "" },
    { "    v_cvt_f64_f32  v[55:56], v27 vop3", 0xd3200037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_ubyte0  v158, v79", 0x7f3c234fU, 0, false, true, "" },
    { "    v_cvt_f32_ubyte0  v55, v27 vop3", 0xd3220037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_ubyte1  v158, v79", 0x7f3c254fU, 0, false, true, "" },
    { "    v_cvt_f32_ubyte1  v55, v27 vop3", 0xd3240037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_ubyte2  v158, v79", 0x7f3c274fU, 0, false, true, "" },
    { "    v_cvt_f32_ubyte2  v55, v27 vop3", 0xd3260037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f32_ubyte3  v158, v79", 0x7f3c294fU, 0, false, true, "" },
    { "    v_cvt_f32_ubyte3  v55, v27 vop3", 0xd3280037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_u32_f64  v158, v[79:80]", 0x7f3c2b4fU, 0, false, true, "" },
    { "    v_cvt_u32_f64  v55, v[27:28] vop3", 0xd32a0037U, 0x0000011bU, true, true, "" },
    { "    v_cvt_f64_u32  v[158:159], v79", 0x7f3c2d4fU, 0, false, true, "" },
    { "    v_cvt_f64_u32  v[55:56], v27 vop3", 0xd32c0037U, 0x0000011bU, true, true, "" },
    { "    v_fract_f32  v158, v79", 0x7f3c414fU, 0, false, true, "" },
    { "    v_fract_f32  v55, v27 vop3", 0xd3400037U, 0x0000011bU, true, true, "" },
    { "    v_trunc_f32  v158, v79", 0x7f3c434fU, 0, false, true, "" },
    { "    v_trunc_f32  v55, v27 vop3", 0xd3420037U, 0x0000011bU, true, true, "" },
    { "    v_ceil_f32  v158, v79", 0x7f3c454fU, 0, false, true, "" },
    { "    v_ceil_f32  v55, v27 vop3", 0xd3440037U, 0x0000011bU, true, true, "" },
    { "    v_rndne_f32  v158, v79", 0x7f3c474fU, 0, false, true, "" },
    { "    v_rndne_f32  v55, v27 vop3", 0xd3460037U, 0x0000011bU, true, true, "" },
    { "    v_floor_f32  v158, v79", 0x7f3c494fU, 0, false, true, "" },
    { "    v_floor_f32  v55, v27 vop3", 0xd3480037U, 0x0000011bU, true, true, "" },
    { "    v_exp_f32  v158, v79", 0x7f3c4b4fU, 0, false, true, "" },
    { "    v_exp_f32  v55, v27 vop3", 0xd34a0037U, 0x0000011bU, true, true, "" },
    { "    v_log_clamp_f32  v158, v79", 0x7f3c4d4fU, 0, false, true, "" },
    { "    v_log_clamp_f32  v55, v27 vop3", 0xd34c0037U, 0x0000011bU, true, true, "" },
    { "    v_log_f32  v158, v79", 0x7f3c4f4fU, 0, false, true, "" },
    { "    v_log_f32  v55, v27 vop3", 0xd34e0037U, 0x0000011bU, true, true, "" },
    { "    v_rcp_clamp_f32  v158, v79", 0x7f3c514fU, 0, false, true, "" },
    { "    v_rcp_clamp_f32  v55, v27 vop3", 0xd3500037U, 0x0000011bU, true, true, "" },
    { "    v_rcp_legacy_f32  v158, v79", 0x7f3c534fU, 0, false, true, "" },
    { "    v_rcp_legacy_f32  v55, v27 vop3", 0xd3520037U, 0x0000011bU, true, true, "" },
    { "    v_rcp_f32  v158, v79", 0x7f3c554fU, 0, false, true, "" },
    { "    v_rcp_f32  v55, v27 vop3", 0xd3540037U, 0x0000011bU, true, true, "" },
    { "    v_rcp_iflag_f32  v158, v79", 0x7f3c574fU, 0, false, true, "" },
    { "    v_rcp_iflag_f32  v55, v27 vop3", 0xd3560037U, 0x0000011bU, true, true, "" },
    { "    v_rsq_clamp_f32  v158, v79", 0x7f3c594fU, 0, false, true, "" },
    { "    v_rsq_clamp_f32  v55, v27 vop3", 0xd3580037U, 0x0000011bU, true, true, "" },
    { "    v_rsq_legacy_f32  v158, v79", 0x7f3c5b4fU, 0, false, true, "" },
    { "    v_rsq_legacy_f32  v55, v27 vop3", 0xd35a0037U, 0x0000011bU, true, true, "" },
    { "    v_rsq_f32  v158, v79", 0x7f3c5d4fU, 0, false, true, "" },
    { "    v_rsq_f32  v55, v27 vop3", 0xd35c0037U, 0x0000011bU, true, true, "" },
    { "    v_rcp_f64  v[158:159], v[79:80]", 0x7f3c5f4fU, 0, false, true, "" },
    { "    v_rcp_f64  v[158:159], s[79:80]", 0x7f3c5e4fU, 0, false, true, "" },
    { "    v_rcp_f64  v[55:56], v[27:28] vop3", 0xd35e0037U, 0x0000011bU, true, true, "" },
    { "    v_rcp_clamp_f64  v[158:159], v[79:80]", 0x7f3c614fU, 0, false, true, "" },
    { "    v_rcp_clamp_f64  v[55:56], v[27:28] vop3",
        0xd3600037U, 0x0000011bU, true, true, "" },
    { "    v_rsq_f64  v[158:159], v[79:80]", 0x7f3c634fU, 0, false, true, "" },
    { "    v_rsq_f64  v[55:56], v[27:28] vop3", 0xd3620037U, 0x0000011bU, true, true, "" },
    { "    v_rsq_clamp_f64  v[158:159], v[79:80]", 0x7f3c654fU, 0, false, true, "" },
    { "    v_rsq_clamp_f64  v[55:56], v[27:28] vop3",
        0xd3640037U, 0x0000011bU, true, true, "" },
    { "    v_sqrt_f32  v158, v79", 0x7f3c674fU, 0, false, true, "" },
    { "    v_sqrt_f32  v55, v27 vop3", 0xd3660037U, 0x0000011bU, true, true, "" },
    { "    v_sqrt_f64  v[158:159], v[79:80]", 0x7f3c694fU, 0, false, true, "" },
    { "    v_sqrt_f64  v[55:56], v[27:28] vop3", 0xd3680037U, 0x0000011bU, true, true, "" },
    { "    v_sin_f32  v158, v79", 0x7f3c6b4fU, 0, false, true, "" },
    { "    v_sin_f32  v55, v27 vop3", 0xd36a0037U, 0x0000011bU, true, true, "" },
    { "    v_cos_f32  v158, v79", 0x7f3c6d4fU, 0, false, true, "" },
    { "    v_cos_f32  v55, v27 vop3", 0xd36c0037U, 0x0000011bU, true, true, "" },
    { "    v_not_b32  v158, v79", 0x7f3c6f4fU, 0, false, true, "" },
    { "    v_not_b32  v55, v27 vop3", 0xd36e0037U, 0x0000011bU, true, true, "" },
    { "    v_bfrev_b32  v158, v79", 0x7f3c714fU, 0, false, true, "" },
    { "    v_bfrev_b32  v55, v27 vop3", 0xd3700037U, 0x0000011bU, true, true, "" },
    { "    v_ffbh_u32  v158, v79", 0x7f3c734fU, 0, false, true, "" },
    { "    v_ffbh_u32  v55, v27 vop3", 0xd3720037U, 0x0000011bU, true, true, "" },
    { "    v_ffbl_b32  v158, v79", 0x7f3c754fU, 0, false, true, "" },
    { "    v_ffbl_b32  v55, v27 vop3", 0xd3740037U, 0x0000011bU, true, true, "" },
    { "    v_ffbh_i32  v158, v79", 0x7f3c774fU, 0, false, true, "" },
    { "    v_ffbh_i32  v55, v27 vop3", 0xd3760037U, 0x0000011bU, true, true, "" },
    { "    v_frexp_exp_i32_f64 v158, v[79:80]", 0x7f3c794fU, 0, false, true, "" },
    { "    v_frexp_exp_i32_f64 v55, v[27:28] vop3",
        0xd3780037U, 0x0000011bU, true, true, "" },
    { "    v_frexp_mant_f64 v[158:159], v[79:80]", 0x7f3c7b4fU, 0, false, true, "" },
    { "    v_frexp_mant_f64 v[55:56], v[27:28] vop3",
        0xd37a0037U, 0x0000011bU, true, true, "" },
    { "    v_fract_f64  v[158:159], v[79:80]", 0x7f3c7d4fU, 0, false, true, "" },
    { "    v_fract_f64  v[55:56], v[27:28] vop3",
        0xd37c0037U, 0x0000011bU, true, true, "" },
    { "    v_frexp_exp_i32_f32  v158, v79", 0x7f3c7f4fU, 0, false, true, "" },
    { "    v_frexp_exp_i32_f32  v55, v27 vop3", 0xd37e0037U, 0x0000011bU, true, true, "" },
    { "    v_frexp_mant_f32  v158, v79", 0x7f3c814fU, 0, false, true, "" },
    { "    v_frexp_mant_f32  v55, v27 vop3", 0xd3800037U, 0x0000011bU, true, true, "" },
    { "    v_clrexcp", 0x7e008200U, 0, false, true, "" },
    { "    v_clrexcp vop3", 0xd3820000U, 0, true, true, "" },
    { "    v_movreld_b32  v158, v79", 0x7f3c854fU, 0, false, true, "" },
    { "    v_movreld_b32  v55, v27 vop3", 0xd3840037U, 0x0000011bU, true, true, "" },
    { "    v_movrels_b32  v158, v79", 0x7f3c874fU, 0, false, true, "" },
    { "    v_movrels_b32  v55, v27 vop3", 0xd3860037U, 0x0000011bU, true, true, "" },
    { "    v_movrelsd_b32  v158, v79", 0x7f3c894fU, 0, false, true, "" },
    { "    v_movrelsd_b32  v55, v27 vop3", 0xd3880037U, 0x0000011bU, true, true, "" },
    /* VOP1 errors */
    { "    v_mov_b32  v15, 133 vop3", 0, 0, false, false,
        "test.s:1:5: Error: Literal in VOP3 encoding is illegal\n" },
    { "    v_rcp_f64  v[158:], v[79:80]", 0, 0, false, false,
        "test.s:1:22: Error: Expected expression\n"
        "test.s:1:22: Error: Expected ',' before argument\n" },
    { "    v_rcp_f64  v[158:159, v[79:80]", 0, 0, false, false,
        "test.s:1:16: Error: Unterminated vector register range\n" },
    { "    v_rcp_f64  v158, s79", 0, 0, false, false,
        "test.s:1:16: Error: Required 2 vector registers\n"
        "test.s:1:25: Error: Required 2 scalar registers\n" },
    { "    v_mov_b32  v[48:59], s[80:81]", 0, 0, false, false,
        "test.s:1:16: Error: Required 1 vector register\n"
        "test.s:1:26: Error: Required 1 scalar register\n" },
    /* VOPC encoding */
    { "    v_cmp_f_f32  vcc, v79, v201", 0x7c01934fU, 0, false, true, "" },
    { "    v_cmp_f_f32  vcc, v79, v93", 0x7c00bb4fU, 0, false, true, "" },
    { "    v_cmp_f_f32  vcc, lds, v201", 0x7c0192feU, 0, false, true, "" },
    { "    v_cmp_f_f32  vcc, lds_direct, v201", 0x7c0192feU, 0, false, true, "" },
    { "    v_cmp_f_f32  vcc, s79, v201", 0x7c01924fU, 0, false, true, "" },
    { "    v_cmp_f_f32  vcc, -9, v201", 0x7c0192c9U, 0, false, true, "" },
    { "    v_cmp_f_f32  vcc, 44517, v201", 0x7c0192ffU, 44517, true, true, "" },
    { "txy=54244;v_cmp_f_f32  vcc, txy, v201", 0x7c0192ffU, 54244, true, true, "" },
    { "v_cmp_f_f32  vcc, txy, v201;txy=54244", 0x7c0192ffU, 54244, true, true, "" },
    { "    v_cmp_f_f32  vcc, v79, v201 vop3", 0xd000006aU, 0x3934fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, v79, v201 clamp", 0xd000086aU, 0x3934fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, v79, abs(v201)", 0xd000026aU, 0x3934fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, -v79, v201", 0xd000006aU, 0x2003934fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, v79, -v201", 0xd000006aU, 0x4003934fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, v79, v201 mul:2", 0xd000006aU, 0x0803934fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, v79, s101", 0xd000006aU, 0x0cb4fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, v79, 24", 0xd000006aU, 0x1314fU, true, true, "" },
    { "    v_cmp_f_f32  s[28:29], v79, v201", 0xd000001cU, 0x3934fU, true, true, "" },
    { "    v_cmp_f_f32  s[29:30], v79, v201", 0xd000001dU, 0x3934fU, true, true, "" },
    { "    v_cmp_f_f32  vcc, 10.5332, v201", 0x7c0192ffU, 0x412887fd, true, true, "" },
    { "    v_cmp_f_f32  vcc, s8, s8", 0xd000006aU, 0x1008U, true, true, "" },
    /* VOPC errors */
    { "    v_cmp_f_f32  vcc_lo, v79, v201", 0, 0, false, false,
        "test.s:1:18: Error: Required 2 scalar registers\n" },
    { "    v_cmp_f_f32  vcc, 42321, s37", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_cmp_f_f32  vcc, 42321, m0", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_cmp_f_f32  vcc, s68, 42321", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_cmp_f_f32  vcc, m0, 42321", 0, 0, false, false,
        "test.s:1:5: Error: Literal with SGPR or M0 is illegal\n" },
    { "    v_cmp_f_f32  vcc, s8, s9", 0, 0, false, false,
        "test.s:1:5: Error: More than one SGPR to read in instruction\n" },
    { "    v_cmp_f_f32  vcc, 44517, v201 vop3", 0, 0, false, false,
        "test.s:1:5: Error: Literal in VOP3 encoding is illegal\n" },
    { "    v_cmp_f_f32  vcc, 44517, 63111", 0, 0, false, false,
        "test.s:1:30: Error: Only one literal can be used in instruction\n" },
    { "    v_cmp_f_f32  vcc, sss, dddd", 0, 0, false, false,
        "test.s:1:28: Error: Only one literal can be used in instruction\n" },
    /* VOPC instructions */
    { "    v_cmp_f_f32  vcc, v79, v201", 0x7c01934fU, 0, false, true, "" },
    { "    v_cmp_f_f32  s[42:43], v50, v107", 0xd000002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lt_f32  vcc, v79, v201", 0x7c03934fU, 0, false, true, "" },
    { "    v_cmp_lt_f32  s[42:43], v50, v107", 0xd002002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_eq_f32  vcc, v79, v201", 0x7c05934fU, 0, false, true, "" },
    { "    v_cmp_eq_f32  s[42:43], v50, v107", 0xd004002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_le_f32  vcc, v79, v201", 0x7c07934fU, 0, false, true, "" },
    { "    v_cmp_le_f32  s[42:43], v50, v107", 0xd006002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_gt_f32  vcc, v79, v201", 0x7c09934fU, 0, false, true, "" },
    { "    v_cmp_gt_f32  s[42:43], v50, v107", 0xd008002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lg_f32  vcc, v79, v201", 0x7c0b934fU, 0, false, true, "" },
    { "    v_cmp_lg_f32  s[42:43], v50, v107", 0xd00a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ge_f32  vcc, v79, v201", 0x7c0d934fU, 0, false, true, "" },
    { "    v_cmp_ge_f32  s[42:43], v50, v107", 0xd00c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_o_f32  vcc, v79, v201", 0x7c0f934fU, 0, false, true, "" },
    { "    v_cmp_o_f32  s[42:43], v50, v107", 0xd00e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_u_f32  vcc, v79, v201", 0x7c11934fU, 0, false, true, "" },
    { "    v_cmp_u_f32  s[42:43], v50, v107", 0xd010002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nge_f32  vcc, v79, v201", 0x7c13934fU, 0, false, true, "" },
    { "    v_cmp_nge_f32  s[42:43], v50, v107", 0xd012002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nlg_f32  vcc, v79, v201", 0x7c15934fU, 0, false, true, "" },
    { "    v_cmp_nlg_f32  s[42:43], v50, v107", 0xd014002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ngt_f32  vcc, v79, v201", 0x7c17934fU, 0, false, true, "" },
    { "    v_cmp_ngt_f32  s[42:43], v50, v107", 0xd016002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nle_f32  vcc, v79, v201", 0x7c19934fU, 0, false, true, "" },
    { "    v_cmp_nle_f32  s[42:43], v50, v107", 0xd018002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_neq_f32  vcc, v79, v201", 0x7c1b934fU, 0, false, true, "" },
    { "    v_cmp_neq_f32  s[42:43], v50, v107", 0xd01a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nlt_f32  vcc, v79, v201", 0x7c1d934fU, 0, false, true, "" },
    { "    v_cmp_nlt_f32  s[42:43], v50, v107", 0xd01c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_tru_f32  vcc, v79, v201", 0x7c1f934fU, 0, false, true, "" },
    { "    v_cmp_tru_f32  s[42:43], v50, v107", 0xd01e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_t_f32  vcc, v79, v201", 0x7c1f934fU, 0, false, true, "" },
    { "    v_cmp_t_f32  s[42:43], v50, v107", 0xd01e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpx_f_f32  vcc, v79, v201", 0x7c21934fU, 0, false, true, "" },
    { "    v_cmpx_f_f32  s[42:43], v50, v107", 0xd020002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lt_f32  vcc, v79, v201", 0x7c23934fU, 0, false, true, "" },
    { "    v_cmpx_lt_f32  s[42:43], v50, v107", 0xd022002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_eq_f32  vcc, v79, v201", 0x7c25934fU, 0, false, true, "" },
    { "    v_cmpx_eq_f32  s[42:43], v50, v107", 0xd024002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_le_f32  vcc, v79, v201", 0x7c27934fU, 0, false, true, "" },
    { "    v_cmpx_le_f32  s[42:43], v50, v107", 0xd026002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_gt_f32  vcc, v79, v201", 0x7c29934fU, 0, false, true, "" },
    { "    v_cmpx_gt_f32  s[42:43], v50, v107", 0xd028002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lg_f32  vcc, v79, v201", 0x7c2b934fU, 0, false, true, "" },
    { "    v_cmpx_lg_f32  s[42:43], v50, v107", 0xd02a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ge_f32  vcc, v79, v201", 0x7c2d934fU, 0, false, true, "" },
    { "    v_cmpx_ge_f32  s[42:43], v50, v107", 0xd02c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_o_f32  vcc, v79, v201", 0x7c2f934fU, 0, false, true, "" },
    { "    v_cmpx_o_f32  s[42:43], v50, v107", 0xd02e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_u_f32  vcc, v79, v201", 0x7c31934fU, 0, false, true, "" },
    { "    v_cmpx_u_f32  s[42:43], v50, v107", 0xd030002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nge_f32  vcc, v79, v201", 0x7c33934fU, 0, false, true, "" },
    { "    v_cmpx_nge_f32  s[42:43], v50, v107", 0xd032002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nlg_f32  vcc, v79, v201", 0x7c35934fU, 0, false, true, "" },
    { "    v_cmpx_nlg_f32  s[42:43], v50, v107", 0xd034002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ngt_f32  vcc, v79, v201", 0x7c37934fU, 0, false, true, "" },
    { "    v_cmpx_ngt_f32  s[42:43], v50, v107", 0xd036002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nle_f32  vcc, v79, v201", 0x7c39934fU, 0, false, true, "" },
    { "    v_cmpx_nle_f32  s[42:43], v50, v107", 0xd038002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_neq_f32  vcc, v79, v201", 0x7c3b934fU, 0, false, true, "" },
    { "    v_cmpx_neq_f32  s[42:43], v50, v107", 0xd03a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nlt_f32  vcc, v79, v201", 0x7c3d934fU, 0, false, true, "" },
    { "    v_cmpx_nlt_f32  s[42:43], v50, v107", 0xd03c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_tru_f32  vcc, v79, v201", 0x7c3f934fU, 0, false, true, "" },
    { "    v_cmpx_tru_f32  s[42:43], v50, v107", 0xd03e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_t_f32  vcc, v79, v201", 0x7c3f934fU, 0, false, true, "" },
    { "    v_cmpx_t_f32  s[42:43], v50, v107", 0xd03e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmp_f_f64  vcc, v[79:80], v[201:202]", 0x7c41934fU, 0, false, true, "" },
    { "    v_cmp_f_f64  s[42:43], v[50:51], v[107:108]",
        0xd040002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lt_f64  vcc, v[79:80], v[201:202]", 0x7c43934fU, 0, false, true, "" },
    { "    v_cmp_lt_f64  s[42:43], v[50:51], v[107:108]",
        0xd042002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_eq_f64  vcc, v[79:80], v[201:202]", 0x7c45934fU, 0, false, true, "" },
    { "    v_cmp_eq_f64  s[42:43], v[50:51], v[107:108]",
        0xd044002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_le_f64  vcc, v[79:80], v[201:202]", 0x7c47934fU, 0, false, true, "" },
    { "    v_cmp_le_f64  s[42:43], v[50:51], v[107:108]",
        0xd046002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_gt_f64  vcc, v[79:80], v[201:202]", 0x7c49934fU, 0, false, true, "" },
    { "    v_cmp_gt_f64  s[42:43], v[50:51], v[107:108]",
        0xd048002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lg_f64  vcc, v[79:80], v[201:202]", 0x7c4b934fU, 0, false, true, "" },
    { "    v_cmp_lg_f64  s[42:43], v[50:51], v[107:108]",
        0xd04a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ge_f64  vcc, v[79:80], v[201:202]", 0x7c4d934fU, 0, false, true, "" },
    { "    v_cmp_ge_f64  s[42:43], v[50:51], v[107:108]",
        0xd04c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_o_f64  vcc, v[79:80], v[201:202]", 0x7c4f934fU, 0, false, true, "" },
    { "    v_cmp_o_f64  s[42:43], v[50:51], v[107:108]",
        0xd04e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_u_f64  vcc, v[79:80], v[201:202]", 0x7c51934fU, 0, false, true, "" },
    { "    v_cmp_u_f64  s[42:43], v[50:51], v[107:108]",
        0xd050002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nge_f64  vcc, v[79:80], v[201:202]", 0x7c53934fU, 0, false, true, "" },
    { "    v_cmp_nge_f64  s[42:43], v[50:51], v[107:108]",
        0xd052002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nlg_f64  vcc, v[79:80], v[201:202]", 0x7c55934fU, 0, false, true, "" },
    { "    v_cmp_nlg_f64  s[42:43], v[50:51], v[107:108]",
        0xd054002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ngt_f64  vcc, v[79:80], v[201:202]", 0x7c57934fU, 0, false, true, "" },
    { "    v_cmp_ngt_f64  s[42:43], v[50:51], v[107:108]",
        0xd056002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nle_f64  vcc, v[79:80], v[201:202]", 0x7c59934fU, 0, false, true, "" },
    { "    v_cmp_nle_f64  s[42:43], v[50:51], v[107:108]",
        0xd058002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_neq_f64  vcc, v[79:80], v[201:202]", 0x7c5b934fU, 0, false, true, "" },
    { "    v_cmp_neq_f64  s[42:43], v[50:51], v[107:108]",
        0xd05a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_nlt_f64  vcc, v[79:80], v[201:202]", 0x7c5d934fU, 0, false, true, "" },
    { "    v_cmp_nlt_f64  s[42:43], v[50:51], v[107:108]",
        0xd05c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_tru_f64  vcc, v[79:80], v[201:202]", 0x7c5f934fU, 0, false, true, "" },
    { "    v_cmp_tru_f64  s[42:43], v[50:51], v[107:108]",
        0xd05e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_t_f64  vcc, v[79:80], v[201:202]", 0x7c5f934fU, 0, false, true, "" },
    { "    v_cmp_t_f64  s[42:43], v[50:51], v[107:108]",
        0xd05e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpx_f_f64  vcc, v[79:80], v[201:202]", 0x7c61934fU, 0, false, true, "" },
    { "    v_cmpx_f_f64  s[42:43], v[50:51], v[107:108]",
        0xd060002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lt_f64  vcc, v[79:80], v[201:202]", 0x7c63934fU, 0, false, true, "" },
    { "    v_cmpx_lt_f64  s[42:43], v[50:51], v[107:108]",
        0xd062002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_eq_f64  vcc, v[79:80], v[201:202]", 0x7c65934fU, 0, false, true, "" },
    { "    v_cmpx_eq_f64  s[42:43], v[50:51], v[107:108]",
        0xd064002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_le_f64  vcc, v[79:80], v[201:202]", 0x7c67934fU, 0, false, true, "" },
    { "    v_cmpx_le_f64  s[42:43], v[50:51], v[107:108]",
        0xd066002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_gt_f64  vcc, v[79:80], v[201:202]", 0x7c69934fU, 0, false, true, "" },
    { "    v_cmpx_gt_f64  s[42:43], v[50:51], v[107:108]",
        0xd068002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lg_f64  vcc, v[79:80], v[201:202]", 0x7c6b934fU, 0, false, true, "" },
    { "    v_cmpx_lg_f64  s[42:43], v[50:51], v[107:108]",
        0xd06a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ge_f64  vcc, v[79:80], v[201:202]", 0x7c6d934fU, 0, false, true, "" },
    { "    v_cmpx_ge_f64  s[42:43], v[50:51], v[107:108]",
        0xd06c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_o_f64  vcc, v[79:80], v[201:202]", 0x7c6f934fU, 0, false, true, "" },
    { "    v_cmpx_o_f64  s[42:43], v[50:51], v[107:108]",
        0xd06e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_u_f64  vcc, v[79:80], v[201:202]", 0x7c71934fU, 0, false, true, "" },
    { "    v_cmpx_u_f64  s[42:43], v[50:51], v[107:108]",
        0xd070002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nge_f64  vcc, v[79:80], v[201:202]", 0x7c73934fU, 0, false, true, "" },
    { "    v_cmpx_nge_f64  s[42:43], v[50:51], v[107:108]",
        0xd072002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nlg_f64  vcc, v[79:80], v[201:202]", 0x7c75934fU, 0, false, true, "" },
    { "    v_cmpx_nlg_f64  s[42:43], v[50:51], v[107:108]",
        0xd074002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ngt_f64  vcc, v[79:80], v[201:202]", 0x7c77934fU, 0, false, true, "" },
    { "    v_cmpx_ngt_f64  s[42:43], v[50:51], v[107:108]",
        0xd076002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nle_f64  vcc, v[79:80], v[201:202]", 0x7c79934fU, 0, false, true, "" },
    { "    v_cmpx_nle_f64  s[42:43], v[50:51], v[107:108]",
        0xd078002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_neq_f64  vcc, v[79:80], v[201:202]", 0x7c7b934fU, 0, false, true, "" },
    { "    v_cmpx_neq_f64  s[42:43], v[50:51], v[107:108]",
        0xd07a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_nlt_f64  vcc, v[79:80], v[201:202]", 0x7c7d934fU, 0, false, true, "" },
    { "    v_cmpx_nlt_f64  s[42:43], v[50:51], v[107:108]",
        0xd07c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_tru_f64  vcc, v[79:80], v[201:202]", 0x7c7f934fU, 0, false, true, "" },
    { "    v_cmpx_tru_f64  s[42:43], v[50:51], v[107:108]",
        0xd07e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_t_f64  vcc, v[79:80], v[201:202]", 0x7c7f934fU, 0, false, true, "" },
    { "    v_cmpx_t_f64  s[42:43], v[50:51], v[107:108]",
        0xd07e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmps_f_f32  vcc, v79, v201", 0x7c81934fU, 0, false, true, "" },
    { "    v_cmps_f_f32  s[42:43], v50, v107", 0xd080002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_lt_f32  vcc, v79, v201", 0x7c83934fU, 0, false, true, "" },
    { "    v_cmps_lt_f32  s[42:43], v50, v107", 0xd082002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_eq_f32  vcc, v79, v201", 0x7c85934fU, 0, false, true, "" },
    { "    v_cmps_eq_f32  s[42:43], v50, v107", 0xd084002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_le_f32  vcc, v79, v201", 0x7c87934fU, 0, false, true, "" },
    { "    v_cmps_le_f32  s[42:43], v50, v107", 0xd086002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_gt_f32  vcc, v79, v201", 0x7c89934fU, 0, false, true, "" },
    { "    v_cmps_gt_f32  s[42:43], v50, v107", 0xd088002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_lg_f32  vcc, v79, v201", 0x7c8b934fU, 0, false, true, "" },
    { "    v_cmps_lg_f32  s[42:43], v50, v107", 0xd08a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_ge_f32  vcc, v79, v201", 0x7c8d934fU, 0, false, true, "" },
    { "    v_cmps_ge_f32  s[42:43], v50, v107", 0xd08c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_o_f32  vcc, v79, v201", 0x7c8f934fU, 0, false, true, "" },
    { "    v_cmps_o_f32  s[42:43], v50, v107", 0xd08e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_u_f32  vcc, v79, v201", 0x7c91934fU, 0, false, true, "" },
    { "    v_cmps_u_f32  s[42:43], v50, v107", 0xd090002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nge_f32  vcc, v79, v201", 0x7c93934fU, 0, false, true, "" },
    { "    v_cmps_nge_f32  s[42:43], v50, v107", 0xd092002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nlg_f32  vcc, v79, v201", 0x7c95934fU, 0, false, true, "" },
    { "    v_cmps_nlg_f32  s[42:43], v50, v107", 0xd094002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_ngt_f32  vcc, v79, v201", 0x7c97934fU, 0, false, true, "" },
    { "    v_cmps_ngt_f32  s[42:43], v50, v107", 0xd096002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nle_f32  vcc, v79, v201", 0x7c99934fU, 0, false, true, "" },
    { "    v_cmps_nle_f32  s[42:43], v50, v107", 0xd098002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_neq_f32  vcc, v79, v201", 0x7c9b934fU, 0, false, true, "" },
    { "    v_cmps_neq_f32  s[42:43], v50, v107", 0xd09a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nlt_f32  vcc, v79, v201", 0x7c9d934fU, 0, false, true, "" },
    { "    v_cmps_nlt_f32  s[42:43], v50, v107", 0xd09c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_tru_f32  vcc, v79, v201", 0x7c9f934fU, 0, false, true, "" },
    { "    v_cmps_tru_f32  s[42:43], v50, v107", 0xd09e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_t_f32  vcc, v79, v201", 0x7c9f934fU, 0, false, true, "" },
    { "    v_cmps_t_f32  s[42:43], v50, v107", 0xd09e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpsx_f_f32  vcc, v79, v201", 0x7ca1934fU, 0, false, true, "" },
    { "    v_cmpsx_f_f32  s[42:43], v50, v107", 0xd0a0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_lt_f32  vcc, v79, v201", 0x7ca3934fU, 0, false, true, "" },
    { "    v_cmpsx_lt_f32  s[42:43], v50, v107", 0xd0a2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_eq_f32  vcc, v79, v201", 0x7ca5934fU, 0, false, true, "" },
    { "    v_cmpsx_eq_f32  s[42:43], v50, v107", 0xd0a4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_le_f32  vcc, v79, v201", 0x7ca7934fU, 0, false, true, "" },
    { "    v_cmpsx_le_f32  s[42:43], v50, v107", 0xd0a6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_gt_f32  vcc, v79, v201", 0x7ca9934fU, 0, false, true, "" },
    { "    v_cmpsx_gt_f32  s[42:43], v50, v107", 0xd0a8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_lg_f32  vcc, v79, v201", 0x7cab934fU, 0, false, true, "" },
    { "    v_cmpsx_lg_f32  s[42:43], v50, v107", 0xd0aa002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_ge_f32  vcc, v79, v201", 0x7cad934fU, 0, false, true, "" },
    { "    v_cmpsx_ge_f32  s[42:43], v50, v107", 0xd0ac002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_o_f32  vcc, v79, v201", 0x7caf934fU, 0, false, true, "" },
    { "    v_cmpsx_o_f32  s[42:43], v50, v107", 0xd0ae002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_u_f32  vcc, v79, v201", 0x7cb1934fU, 0, false, true, "" },
    { "    v_cmpsx_u_f32  s[42:43], v50, v107", 0xd0b0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nge_f32  vcc, v79, v201", 0x7cb3934fU, 0, false, true, "" },
    { "    v_cmpsx_nge_f32  s[42:43], v50, v107",
        0xd0b2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nlg_f32  vcc, v79, v201", 0x7cb5934fU, 0, false, true, "" },
    { "    v_cmpsx_nlg_f32  s[42:43], v50, v107",
        0xd0b4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_ngt_f32  vcc, v79, v201", 0x7cb7934fU, 0, false, true, "" },
    { "    v_cmpsx_ngt_f32  s[42:43], v50, v107",
        0xd0b6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nle_f32  vcc, v79, v201", 0x7cb9934fU, 0, false, true, "" },
    { "    v_cmpsx_nle_f32  s[42:43], v50, v107",
        0xd0b8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_neq_f32  vcc, v79, v201", 0x7cbb934fU, 0, false, true, "" },
    { "    v_cmpsx_neq_f32  s[42:43], v50, v107",
        0xd0ba002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nlt_f32  vcc, v79, v201", 0x7cbd934fU, 0, false, true, "" },
    { "    v_cmpsx_nlt_f32  s[42:43], v50, v107",
        0xd0bc002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_tru_f32  vcc, v79, v201", 0x7cbf934fU, 0, false, true, "" },
    { "    v_cmpsx_tru_f32  s[42:43], v50, v107",
        0xd0be002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_t_f32  vcc, v79, v201", 0x7cbf934fU, 0, false, true, "" },
    { "    v_cmpsx_t_f32  s[42:43], v50, v107",
        0xd0be002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmps_f_f64  vcc, v[79:80], v[201:202]", 0x7cc1934fU, 0, false, true, "" },
    { "    v_cmps_f_f64  s[42:43], v[50:51], v[107:108]",
        0xd0c0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_lt_f64  vcc, v[79:80], v[201:202]", 0x7cc3934fU, 0, false, true, "" },
    { "    v_cmps_lt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0c2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_eq_f64  vcc, v[79:80], v[201:202]", 0x7cc5934fU, 0, false, true, "" },
    { "    v_cmps_eq_f64  s[42:43], v[50:51], v[107:108]",
        0xd0c4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_le_f64  vcc, v[79:80], v[201:202]", 0x7cc7934fU, 0, false, true, "" },
    { "    v_cmps_le_f64  s[42:43], v[50:51], v[107:108]",
        0xd0c6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_gt_f64  vcc, v[79:80], v[201:202]", 0x7cc9934fU, 0, false, true, "" },
    { "    v_cmps_gt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0c8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_lg_f64  vcc, v[79:80], v[201:202]", 0x7ccb934fU, 0, false, true, "" },
    { "    v_cmps_lg_f64  s[42:43], v[50:51], v[107:108]",
        0xd0ca002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_ge_f64  vcc, v[79:80], v[201:202]", 0x7ccd934fU, 0, false, true, "" },
    { "    v_cmps_ge_f64  s[42:43], v[50:51], v[107:108]",
        0xd0cc002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_o_f64  vcc, v[79:80], v[201:202]", 0x7ccf934fU, 0, false, true, "" },
    { "    v_cmps_o_f64  s[42:43], v[50:51], v[107:108]",
        0xd0ce002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_u_f64  vcc, v[79:80], v[201:202]", 0x7cd1934fU, 0, false, true, "" },
    { "    v_cmps_u_f64  s[42:43], v[50:51], v[107:108]",
        0xd0d0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nge_f64  vcc, v[79:80], v[201:202]", 0x7cd3934fU, 0, false, true, "" },
    { "    v_cmps_nge_f64  s[42:43], v[50:51], v[107:108]",
        0xd0d2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nlg_f64  vcc, v[79:80], v[201:202]", 0x7cd5934fU, 0, false, true, "" },
    { "    v_cmps_nlg_f64  s[42:43], v[50:51], v[107:108]",
        0xd0d4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_ngt_f64  vcc, v[79:80], v[201:202]", 0x7cd7934fU, 0, false, true, "" },
    { "    v_cmps_ngt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0d6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nle_f64  vcc, v[79:80], v[201:202]", 0x7cd9934fU, 0, false, true, "" },
    { "    v_cmps_nle_f64  s[42:43], v[50:51], v[107:108]",
        0xd0d8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_neq_f64  vcc, v[79:80], v[201:202]", 0x7cdb934fU, 0, false, true, "" },
    { "    v_cmps_neq_f64  s[42:43], v[50:51], v[107:108]",
        0xd0da002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_nlt_f64  vcc, v[79:80], v[201:202]", 0x7cdd934fU, 0, false, true, "" },
    { "    v_cmps_nlt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0dc002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_tru_f64  vcc, v[79:80], v[201:202]", 0x7cdf934fU, 0, false, true, "" },
    { "    v_cmps_tru_f64  s[42:43], v[50:51], v[107:108]",
        0xd0de002aU, 0x0002d732U, true, true, "" },
    { "    v_cmps_t_f64  vcc, v[79:80], v[201:202]", 0x7cdf934fU, 0, false, true, "" },
    { "    v_cmps_t_f64  s[42:43], v[50:51], v[107:108]",
        0xd0de002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpsx_f_f64  vcc, v[79:80], v[201:202]", 0x7ce1934fU, 0, false, true, "" },
    { "    v_cmpsx_f_f64  s[42:43], v[50:51], v[107:108]",
        0xd0e0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_lt_f64  vcc, v[79:80], v[201:202]", 0x7ce3934fU, 0, false, true, "" },
    { "    v_cmpsx_lt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0e2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_eq_f64  vcc, v[79:80], v[201:202]", 0x7ce5934fU, 0, false, true, "" },
    { "    v_cmpsx_eq_f64  s[42:43], v[50:51], v[107:108]",
        0xd0e4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_le_f64  vcc, v[79:80], v[201:202]", 0x7ce7934fU, 0, false, true, "" },
    { "    v_cmpsx_le_f64  s[42:43], v[50:51], v[107:108]",
        0xd0e6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_gt_f64  vcc, v[79:80], v[201:202]", 0x7ce9934fU, 0, false, true, "" },
    { "    v_cmpsx_gt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0e8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_lg_f64  vcc, v[79:80], v[201:202]", 0x7ceb934fU, 0, false, true, "" },
    { "    v_cmpsx_lg_f64  s[42:43], v[50:51], v[107:108]",
        0xd0ea002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_ge_f64  vcc, v[79:80], v[201:202]", 0x7ced934fU, 0, false, true, "" },
    { "    v_cmpsx_ge_f64  s[42:43], v[50:51], v[107:108]",
        0xd0ec002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_o_f64  vcc, v[79:80], v[201:202]", 0x7cef934fU, 0, false, true, "" },
    { "    v_cmpsx_o_f64  s[42:43], v[50:51], v[107:108]",
        0xd0ee002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_u_f64  vcc, v[79:80], v[201:202]", 0x7cf1934fU, 0, false, true, "" },
    { "    v_cmpsx_u_f64  s[42:43], v[50:51], v[107:108]",
        0xd0f0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nge_f64  vcc, v[79:80], v[201:202]", 0x7cf3934fU, 0, false, true, "" },
    { "    v_cmpsx_nge_f64  s[42:43], v[50:51], v[107:108]",
        0xd0f2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nlg_f64  vcc, v[79:80], v[201:202]", 0x7cf5934fU, 0, false, true, "" },
    { "    v_cmpsx_nlg_f64  s[42:43], v[50:51], v[107:108]",
        0xd0f4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_ngt_f64  vcc, v[79:80], v[201:202]", 0x7cf7934fU, 0, false, true, "" },
    { "    v_cmpsx_ngt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0f6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nle_f64  vcc, v[79:80], v[201:202]", 0x7cf9934fU, 0, false, true, "" },
    { "    v_cmpsx_nle_f64  s[42:43], v[50:51], v[107:108]",
        0xd0f8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_neq_f64  vcc, v[79:80], v[201:202]", 0x7cfb934fU, 0, false, true, "" },
    { "    v_cmpsx_neq_f64  s[42:43], v[50:51], v[107:108]",
        0xd0fa002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_nlt_f64  vcc, v[79:80], v[201:202]", 0x7cfd934fU, 0, false, true, "" },
    { "    v_cmpsx_nlt_f64  s[42:43], v[50:51], v[107:108]",
        0xd0fc002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_tru_f64  vcc, v[79:80], v[201:202]", 0x7cff934fU, 0, false, true, "" },
    { "    v_cmpsx_tru_f64  s[42:43], v[50:51], v[107:108]",
        0xd0fe002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpsx_t_f64  vcc, v[79:80], v[201:202]", 0x7cff934fU, 0, false, true, "" },
    { "    v_cmpsx_t_f64  s[42:43], v[50:51], v[107:108]",
        0xd0fe002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmp_f_i32  vcc, v79, v201", 0x7d01934fU, 0, false, true, "" },
    { "    v_cmp_f_i32  s[42:43], v50, v107", 0xd100002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lt_i32  vcc, v79, v201", 0x7d03934fU, 0, false, true, "" },
    { "    v_cmp_lt_i32  s[42:43], v50, v107", 0xd102002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_eq_i32  vcc, v79, v201", 0x7d05934fU, 0, false, true, "" },
    { "    v_cmp_eq_i32  s[42:43], v50, v107", 0xd104002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_le_i32  vcc, v79, v201", 0x7d07934fU, 0, false, true, "" },
    { "    v_cmp_le_i32  s[42:43], v50, v107", 0xd106002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_gt_i32  vcc, v79, v201", 0x7d09934fU, 0, false, true, "" },
    { "    v_cmp_gt_i32  s[42:43], v50, v107", 0xd108002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lg_i32  vcc, v79, v201", 0x7d0b934fU, 0, false, true, "" },
    { "    v_cmp_lg_i32  s[42:43], v50, v107", 0xd10a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ne_i32  vcc, v79, v201", 0x7d0b934fU, 0, false, true, "" },
    { "    v_cmp_ne_i32  s[42:43], v50, v107", 0xd10a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ge_i32  vcc, v79, v201", 0x7d0d934fU, 0, false, true, "" },
    { "    v_cmp_ge_i32  s[42:43], v50, v107", 0xd10c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_tru_i32  vcc, v79, v201", 0x7d0f934fU, 0, false, true, "" },
    { "    v_cmp_tru_i32  s[42:43], v50, v107", 0xd10e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_t_i32  vcc, v79, v201", 0x7d0f934fU, 0, false, true, "" },
    { "    v_cmp_t_i32  s[42:43], v50, v107", 0xd10e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpx_f_i32  vcc, v79, v201", 0x7d21934fU, 0, false, true, "" },
    { "    v_cmpx_f_i32  s[42:43], v50, v107", 0xd120002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lt_i32  vcc, v79, v201", 0x7d23934fU, 0, false, true, "" },
    { "    v_cmpx_lt_i32  s[42:43], v50, v107", 0xd122002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_eq_i32  vcc, v79, v201", 0x7d25934fU, 0, false, true, "" },
    { "    v_cmpx_eq_i32  s[42:43], v50, v107", 0xd124002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_le_i32  vcc, v79, v201", 0x7d27934fU, 0, false, true, "" },
    { "    v_cmpx_le_i32  s[42:43], v50, v107", 0xd126002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_gt_i32  vcc, v79, v201", 0x7d29934fU, 0, false, true, "" },
    { "    v_cmpx_gt_i32  s[42:43], v50, v107", 0xd128002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lg_i32  vcc, v79, v201", 0x7d2b934fU, 0, false, true, "" },
    { "    v_cmpx_lg_i32  s[42:43], v50, v107", 0xd12a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ne_i32  vcc, v79, v201", 0x7d2b934fU, 0, false, true, "" },
    { "    v_cmpx_ne_i32  s[42:43], v50, v107", 0xd12a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ge_i32  vcc, v79, v201", 0x7d2d934fU, 0, false, true, "" },
    { "    v_cmpx_ge_i32  s[42:43], v50, v107", 0xd12c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_tru_i32  vcc, v79, v201", 0x7d2f934fU, 0, false, true, "" },
    { "    v_cmpx_tru_i32  s[42:43], v50, v107", 0xd12e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_t_i32  vcc, v79, v201", 0x7d2f934fU, 0, false, true, "" },
    { "    v_cmpx_t_i32  s[42:43], v50, v107", 0xd12e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmp_f_i64  vcc, v[79:80], v[201:202]", 0x7d41934fU, 0, false, true, "" },
    { "    v_cmp_f_i64  s[42:43], v[50:51], v[107:108]",
        0xd140002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lt_i64  vcc, v[79:80], v[201:202]", 0x7d43934fU, 0, false, true, "" },
    { "    v_cmp_lt_i64  s[42:43], v[50:51], v[107:108]",
        0xd142002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_eq_i64  vcc, v[79:80], v[201:202]", 0x7d45934fU, 0, false, true, "" },
    { "    v_cmp_eq_i64  s[42:43], v[50:51], v[107:108]",
        0xd144002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_le_i64  vcc, v[79:80], v[201:202]", 0x7d47934fU, 0, false, true, "" },
    { "    v_cmp_le_i64  s[42:43], v[50:51], v[107:108]",
        0xd146002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_gt_i64  vcc, v[79:80], v[201:202]", 0x7d49934fU, 0, false, true, "" },
    { "    v_cmp_gt_i64  s[42:43], v[50:51], v[107:108]",
        0xd148002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lg_i64  vcc, v[79:80], v[201:202]", 0x7d4b934fU, 0, false, true, "" },
    { "    v_cmp_lg_i64  s[42:43], v[50:51], v[107:108]",
        0xd14a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ne_i64  vcc, v[79:80], v[201:202]", 0x7d4b934fU, 0, false, true, "" },
    { "    v_cmp_ne_i64  s[42:43], v[50:51], v[107:108]",
        0xd14a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ge_i64  vcc, v[79:80], v[201:202]", 0x7d4d934fU, 0, false, true, "" },
    { "    v_cmp_ge_i64  s[42:43], v[50:51], v[107:108]",
        0xd14c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_tru_i64  vcc, v[79:80], v[201:202]", 0x7d4f934fU, 0, false, true, "" },
    { "    v_cmp_tru_i64  s[42:43], v[50:51], v[107:108]",
        0xd14e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_t_i64  vcc, v[79:80], v[201:202]", 0x7d4f934fU, 0, false, true, "" },
    { "    v_cmp_t_i64  s[42:43], v[50:51], v[107:108]",
        0xd14e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpx_f_i64  vcc, v[79:80], v[201:202]", 0x7d61934fU, 0, false, true, "" },
    { "    v_cmpx_f_i64  s[42:43], v[50:51], v[107:108]",
        0xd160002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lt_i64  vcc, v[79:80], v[201:202]", 0x7d63934fU, 0, false, true, "" },
    { "    v_cmpx_lt_i64  s[42:43], v[50:51], v[107:108]",
        0xd162002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_eq_i64  vcc, v[79:80], v[201:202]", 0x7d65934fU, 0, false, true, "" },
    { "    v_cmpx_eq_i64  s[42:43], v[50:51], v[107:108]",
        0xd164002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_le_i64  vcc, v[79:80], v[201:202]", 0x7d67934fU, 0, false, true, "" },
    { "    v_cmpx_le_i64  s[42:43], v[50:51], v[107:108]",
        0xd166002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_gt_i64  vcc, v[79:80], v[201:202]", 0x7d69934fU, 0, false, true, "" },
    { "    v_cmpx_gt_i64  s[42:43], v[50:51], v[107:108]",
        0xd168002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lg_i64  vcc, v[79:80], v[201:202]", 0x7d6b934fU, 0, false, true, "" },
    { "    v_cmpx_lg_i64  s[42:43], v[50:51], v[107:108]",
        0xd16a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ne_i64  vcc, v[79:80], v[201:202]", 0x7d6b934fU, 0, false, true, "" },
    { "    v_cmpx_ne_i64  s[42:43], v[50:51], v[107:108]",
        0xd16a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ge_i64  vcc, v[79:80], v[201:202]", 0x7d6d934fU, 0, false, true, "" },
    { "    v_cmpx_ge_i64  s[42:43], v[50:51], v[107:108]",
        0xd16c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_tru_i64  vcc, v[79:80], v[201:202]", 0x7d6f934fU, 0, false, true, "" },
    { "    v_cmpx_tru_i64  s[42:43], v[50:51], v[107:108]",
        0xd16e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_t_i64  vcc, v[79:80], v[201:202]", 0x7d6f934fU, 0, false, true, "" },
    { "    v_cmpx_t_i64  s[42:43], v[50:51], v[107:108]",
        0xd16e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmp_f_u32  vcc, v79, v201", 0x7d81934fU, 0, false, true, "" },
    { "    v_cmp_f_u32  s[42:43], v50, v107", 0xd180002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lt_u32  vcc, v79, v201", 0x7d83934fU, 0, false, true, "" },
    { "    v_cmp_lt_u32  s[42:43], v50, v107", 0xd182002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_eq_u32  vcc, v79, v201", 0x7d85934fU, 0, false, true, "" },
    { "    v_cmp_eq_u32  s[42:43], v50, v107", 0xd184002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_le_u32  vcc, v79, v201", 0x7d87934fU, 0, false, true, "" },
    { "    v_cmp_le_u32  s[42:43], v50, v107", 0xd186002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_gt_u32  vcc, v79, v201", 0x7d89934fU, 0, false, true, "" },
    { "    v_cmp_gt_u32  s[42:43], v50, v107", 0xd188002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lg_u32  vcc, v79, v201", 0x7d8b934fU, 0, false, true, "" },
    { "    v_cmp_lg_u32  s[42:43], v50, v107", 0xd18a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ne_u32  vcc, v79, v201", 0x7d8b934fU, 0, false, true, "" },
    { "    v_cmp_ne_u32  s[42:43], v50, v107", 0xd18a002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ge_u32  vcc, v79, v201", 0x7d8d934fU, 0, false, true, "" },
    { "    v_cmp_ge_u32  s[42:43], v50, v107", 0xd18c002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_tru_u32  vcc, v79, v201", 0x7d8f934fU, 0, false, true, "" },
    { "    v_cmp_tru_u32  s[42:43], v50, v107", 0xd18e002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_t_u32  vcc, v79, v201", 0x7d8f934fU, 0, false, true, "" },
    { "    v_cmp_t_u32  s[42:43], v50, v107", 0xd18e002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpx_f_u32  vcc, v79, v201", 0x7da1934fU, 0, false, true, "" },
    { "    v_cmpx_f_u32  s[42:43], v50, v107", 0xd1a0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lt_u32  vcc, v79, v201", 0x7da3934fU, 0, false, true, "" },
    { "    v_cmpx_lt_u32  s[42:43], v50, v107", 0xd1a2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_eq_u32  vcc, v79, v201", 0x7da5934fU, 0, false, true, "" },
    { "    v_cmpx_eq_u32  s[42:43], v50, v107", 0xd1a4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_le_u32  vcc, v79, v201", 0x7da7934fU, 0, false, true, "" },
    { "    v_cmpx_le_u32  s[42:43], v50, v107", 0xd1a6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_gt_u32  vcc, v79, v201", 0x7da9934fU, 0, false, true, "" },
    { "    v_cmpx_gt_u32  s[42:43], v50, v107", 0xd1a8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lg_u32  vcc, v79, v201", 0x7dab934fU, 0, false, true, "" },
    { "    v_cmpx_lg_u32  s[42:43], v50, v107", 0xd1aa002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ne_u32  vcc, v79, v201", 0x7dab934fU, 0, false, true, "" },
    { "    v_cmpx_ne_u32  s[42:43], v50, v107", 0xd1aa002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ge_u32  vcc, v79, v201", 0x7dad934fU, 0, false, true, "" },
    { "    v_cmpx_ge_u32  s[42:43], v50, v107", 0xd1ac002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_tru_u32  vcc, v79, v201", 0x7daf934fU, 0, false, true, "" },
    { "    v_cmpx_tru_u32  s[42:43], v50, v107", 0xd1ae002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_t_u32  vcc, v79, v201", 0x7daf934fU, 0, false, true, "" },
    { "    v_cmpx_t_u32  s[42:43], v50, v107", 0xd1ae002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmp_f_u64  vcc, v[79:80], v[201:202]", 0x7dc1934fU, 0, false, true, "" },
    { "    v_cmp_f_u64  s[42:43], v[50:51], v[107:108]",
        0xd1c0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lt_u64  vcc, v[79:80], v[201:202]", 0x7dc3934fU, 0, false, true, "" },
    { "    v_cmp_lt_u64  s[42:43], v[50:51], v[107:108]",
        0xd1c2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_eq_u64  vcc, v[79:80], v[201:202]", 0x7dc5934fU, 0, false, true, "" },
    { "    v_cmp_eq_u64  s[42:43], v[50:51], v[107:108]",
        0xd1c4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_le_u64  vcc, v[79:80], v[201:202]", 0x7dc7934fU, 0, false, true, "" },
    { "    v_cmp_le_u64  s[42:43], v[50:51], v[107:108]",
        0xd1c6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_gt_u64  vcc, v[79:80], v[201:202]", 0x7dc9934fU, 0, false, true, "" },
    { "    v_cmp_gt_u64  s[42:43], v[50:51], v[107:108]",
        0xd1c8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_lg_u64  vcc, v[79:80], v[201:202]", 0x7dcb934fU, 0, false, true, "" },
    { "    v_cmp_lg_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ca002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ne_u64  vcc, v[79:80], v[201:202]", 0x7dcb934fU, 0, false, true, "" },
    { "    v_cmp_ne_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ca002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_ge_u64  vcc, v[79:80], v[201:202]", 0x7dcd934fU, 0, false, true, "" },
    { "    v_cmp_ge_u64  s[42:43], v[50:51], v[107:108]",
        0xd1cc002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_tru_u64  vcc, v[79:80], v[201:202]", 0x7dcf934fU, 0, false, true, "" },
    { "    v_cmp_tru_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ce002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_t_u64  vcc, v[79:80], v[201:202]", 0x7dcf934fU, 0, false, true, "" },
    { "    v_cmp_t_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ce002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmpx_f_u64  vcc, v[79:80], v[201:202]", 0x7de1934fU, 0, false, true, "" },
    { "    v_cmpx_f_u64  s[42:43], v[50:51], v[107:108]",
        0xd1e0002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lt_u64  vcc, v[79:80], v[201:202]", 0x7de3934fU, 0, false, true, "" },
    { "    v_cmpx_lt_u64  s[42:43], v[50:51], v[107:108]",
        0xd1e2002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_eq_u64  vcc, v[79:80], v[201:202]", 0x7de5934fU, 0, false, true, "" },
    { "    v_cmpx_eq_u64  s[42:43], v[50:51], v[107:108]",
        0xd1e4002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_le_u64  vcc, v[79:80], v[201:202]", 0x7de7934fU, 0, false, true, "" },
    { "    v_cmpx_le_u64  s[42:43], v[50:51], v[107:108]",
        0xd1e6002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_gt_u64  vcc, v[79:80], v[201:202]", 0x7de9934fU, 0, false, true, "" },
    { "    v_cmpx_gt_u64  s[42:43], v[50:51], v[107:108]",
        0xd1e8002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_lg_u64  vcc, v[79:80], v[201:202]", 0x7deb934fU, 0, false, true, "" },
    { "    v_cmpx_lg_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ea002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ne_u64  vcc, v[79:80], v[201:202]", 0x7deb934fU, 0, false, true, "" },
    { "    v_cmpx_ne_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ea002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_ge_u64  vcc, v[79:80], v[201:202]", 0x7ded934fU, 0, false, true, "" },
    { "    v_cmpx_ge_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ec002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_tru_u64  vcc, v[79:80], v[201:202]", 0x7def934fU, 0, false, true, "" },
    { "    v_cmpx_tru_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ee002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_t_u64  vcc, v[79:80], v[201:202]", 0x7def934fU, 0, false, true, "" },
    { "    v_cmpx_t_u64  s[42:43], v[50:51], v[107:108]",
        0xd1ee002aU, 0x0002d732U, true, true, "" },
    
    { "    v_cmp_class_f32  vcc, v79, v201", 0x7d11934fU, 0, false, true, "" },
    { "    v_cmp_class_f32  s[42:43], v50, v107",
        0xd110002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_class_f32  vcc, v79, v201", 0x7d31934fU, 0, false, true, "" },
    { "    v_cmpx_class_f32  s[42:43], v50, v107",
        0xd130002aU, 0x0002d732U, true, true, "" },
    { "    v_cmp_class_f64  vcc, v[79:80], v[201:202]", 0x7d51934fU, 0, false, true, "" },
    { "    v_cmp_class_f64  s[42:43], v[50:51], v[107:108]",
        0xd150002aU, 0x0002d732U, true, true, "" },
    { "    v_cmpx_class_f64  vcc, v[79:80], v[201:202]", 0x7d71934fU, 0, false, true, "" },
    { "    v_cmpx_class_f64  s[42:43], v[50:51], v[107:108]",
        0xd170002aU, 0x0002d732U, true, true, "" },
    
    /* VOP3 encoding */
    { "   v_mad_legacy_f32 v55, v79, v166, v229",
        0xd2800037U, 0x07974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, s101",
        0xd2800037U, 0x1974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, s63, v229",
        0xd2800037U, 0x07947f4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, s79, v166, v229",
        0xd2800037U, 0x07974c4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, abs(v79), v166, v229",
        0xd2800137U, 0x07974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, abs(v166), v229",
        0xd2800237U, 0x07974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, abs(v229)",
        0xd2800437U, 0x07974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, -v79, v166, v229",
        0xd2800037U, 0x27974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, -v166, v229",
        0xd2800037U, 0x47974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, -v229",
        0xd2800037U, 0x87974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, v229 clamp",
        0xd2800837U, 0x07974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, v229 mul:2",
        0xd2800037U, 0x0f974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, v229 mul:4",
        0xd2800037U, 0x17974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, v229 div:2",
        0xd2800037U, 0x1f974d4fU, true, true, "" },
    { "   v_mad_legacy_f32 v55, 56, v166, v229",
        0xd2800037U, 0x07974cb8U, true, true, "" },
    { "bx=51;v_mad_legacy_f32 v55, bx+5, v166, v229",
        0xd2800037U, 0x07974cb8U, true, true, "" },
    { "   v_mad_legacy_f32 v55, v56, -15, v229",
        0xd2800037U, 0x07959f38U, true, true, "" },
    { "vxt=15; v_mad_legacy_f32 v55, v56, -vxt, v229",
        0xd2800037U, 0x07959f38U, true, true, "" },
    { "   v_mad_legacy_f32 v55, v79, v166, 45",
        0xd2800037U, 0x02b74d4fU, true, true, "" },
    { "gg=22;v_mad_legacy_f32 v55, v79, v166, 2*gg+1",
        0xd2800037U, 0x02b74d4fU, true, true, "" },
    /* one SGPR */
    { "    v_mad_legacy_f32 v55, s79, v166, s79",
        0xd2800037U, 0x013f4c4fU, true, true, "" },
    { "    v_mad_legacy_f32 v55, s79, s79, s79",
        0xd2800037U, 0x013c9e4fU, true, true, "" },
    /* VOP3 errors */
    { "v_mad_legacy_f32 v55, 66, v166, v229", 0, 0, false, false,
        "test.s:1:23: Error: Literal in VOP3 is illegal\n" },
    { "v_mad_legacy_f32 v55, s79, 743, v229", 0, 0, false, false,
        "test.s:1:28: Error: Literal in VOP3 is illegal\n" },
    { "v_mad_legacy_f32 v55, s79, v44, 6434", 0, 0, false, false,
        "test.s:1:33: Error: Literal in VOP3 is illegal\n" },
    { "    v_mad_legacy_f32 v55, s79, s166, v229", 0, 0, false, false,
        "test.s:1:32: Error: Scalar register number out of range\n" },
    { "    v_mad_legacy_f32 v55, s79, s100, v229", 0, 0, false, false,
        "test.s:1:5: Error: More than one SGPR to read in instruction\n" },
    { "    v_mad_legacy_f32 v55, s79, v166, s91", 0, 0, false, false,
        "test.s:1:5: Error: More than one SGPR to read in instruction\n" },
    { "    v_mad_legacy_f32 v55, s79, v166, s52", 0, 0, false, false,
        "test.s:1:5: Error: More than one SGPR to read in instruction\n" },
    { "   v_fma_f64  v[55], v[79], v[166], v[229]", 0, 0, false, false,
        "test.s:1:15: Error: Required 2 vector registers\n"
        "test.s:1:22: Error: Required 2 vector registers\n"
        "test.s:1:29: Error: Required 2 vector registers\n"
        "test.s:1:37: Error: Required 2 vector registers\n" },
    { "   v_fma_f64  v[55:56], s[79:80], v[166:167], s[29:30]", 0, 0, false, false,
        "test.s:1:4: Error: More than one SGPR to read in instruction\n" },
    { "   v_div_scale_f32  v55, s[37:38], v79, v166, v79", 0, 0, false, false,
        "test.s:1:26: Error: Unaligned scalar register range\n" },
    { "   v_div_scale_f32  v55, s[36:37], v79, v166, v79 clamp", 0, 0, false, false,
        "test.s:1:51: Error: Modifier CLAMP in VOP3B is illegal\n" },
    { "   v_div_scale_f32  v55, s[36:37], abs(v79), v166, v79", 0, 0, false, false,
        "test.s:1:39: Error: Expected operator\n"
        "test.s:1:40: Error: Expected ',' before argument\n" },
    { "   v_mad_legacy_f32 v55, v79, v166, v229 clamp ,", 0, 0, false, false,
        "test.s:1:48: Error: Some garbages at VOP modifier place\n" },
    /* VOP3 instructions */
    { "   v_mad_f32  v55, v79, v166, v229", 0xd2820037U, 0x07974d4fU, true, true, "" },
    { "   v_mad_i32_i24  v55, v79, v166, v229", 0xd2840037U, 0x07974d4fU, true, true, "" },
    { "   v_mad_u32_u24  v55, v79, v166, v229", 0xd2860037U, 0x07974d4fU, true, true, "" },
    { "   v_cubeid_f32  v55, v79, v166, v229", 0xd2880037U, 0x07974d4fU, true, true, "" },
    { "   v_cubesc_f32  v55, v79, v166, v229", 0xd28a0037U, 0x07974d4fU, true, true, "" },
    { "   v_cubetc_f32  v55, v79, v166, v229", 0xd28c0037U, 0x07974d4fU, true, true, "" },
    { "   v_cubema_f32  v55, v79, v166, v229", 0xd28e0037U, 0x07974d4fU, true, true, "" },
    { "   v_bfe_u32  v55, v79, v166, v229", 0xd2900037U, 0x07974d4fU, true, true, "" },
    { "   v_bfe_i32  v55, v79, v166, v229", 0xd2920037U, 0x07974d4fU, true, true, "" },
    { "   v_bfi_b32  v55, v79, v166, v229", 0xd2940037U, 0x07974d4fU, true, true, "" },
    { "   v_fma_f32  v55, v79, v166, v229", 0xd2960037U, 0x07974d4fU, true, true, "" },
    { "   v_fma_f64  v[55:56], v[79:80], v[166:167], v[229:230]",
        0xd2980037U, 0x07974d4fU, true, true, "" },
    { "   v_fma_f64 v[5:6], v[1:2], v[3:4], s[1:2]",
        0xd2980005U, 0x00060701U, true, true, "" },
    { "   v_lerp_u8  v55, v79, v166, v229", 0xd29a0037U, 0x07974d4fU, true, true, "" },
    { "   v_alignbit_b32  v55, v79, v166, v229", 0xd29c0037U, 0x07974d4fU, true, true, "" },
    { "   v_alignbyte_b32  v55, v79, v166, v229",
        0xd29e0037U, 0x07974d4fU, true, true, "" },
    { "   v_mullit_f32    v55, v79, v166, s0", 0xd2a00037U, 0x00034d4fU, true, true, "" },
    { "   v_mullit_f32    v55, v79, v166, s0", 0xd2a00037U, 0x00034d4fU, true, true, "" },
    { "   v_mullit_f32    v55, s79, v166, v229", 0xd2a00037U, 0x07974c4fU, true, true, "" },
    { "   v_min3_f32  v55, v79, v166, v229", 0xd2a20037U, 0x07974d4fU, true, true, "" },
    { "   v_min3_i32  v55, v79, v166, v229", 0xd2a40037U, 0x07974d4fU, true, true, "" },
    { "   v_min3_u32  v55, v79, v166, v229", 0xd2a60037U, 0x07974d4fU, true, true, "" },
    { "   v_max3_f32  v55, v79, v166, v229", 0xd2a80037U, 0x07974d4fU, true, true, "" },
    { "   v_max3_i32  v55, v79, v166, v229", 0xd2aa0037U, 0x07974d4fU, true, true, "" },
    { "   v_max3_u32  v55, v79, v166, v229", 0xd2ac0037U, 0x07974d4fU, true, true, "" },
    { "   v_med3_f32  v55, v79, v166, v229", 0xd2ae0037U, 0x07974d4fU, true, true, "" },
    { "   v_med3_i32  v55, v79, v166, v229", 0xd2b00037U, 0x07974d4fU, true, true, "" },
    { "   v_med3_u32  v55, v79, v166, v229", 0xd2b20037U, 0x07974d4fU, true, true, "" },
    { "   v_sad_u8  v55, v79, v166, v229", 0xd2b40037U, 0x07974d4fU, true, true, "" },
    { "   v_sad_hi_u8  v55, v79, v166, v229", 0xd2b60037U, 0x07974d4fU, true, true, "" },
    { "   v_sad_u16  v55, v79, v166, v229", 0xd2b80037U, 0x07974d4fU, true, true, "" },
    { "   v_sad_u32  v55, v79, v166, v229", 0xd2ba0037U, 0x07974d4fU, true, true, "" },
    { "   v_cvt_pk_u8_f32 v55, v79, v166, v229", 0xd2bc0037U, 0x07974d4fU, true, true, "" },
    { "   v_div_fixup_f32 v55, v79, v166, v229", 0xd2be0037U, 0x07974d4fU, true, true, "" },
    { "   v_div_fixup_f64 v[55:56], v[79:80], v[166:167], v[229:230]",
        0xd2c00037U, 0x07974d4fU, true, true, "" },
    { "   v_lshl_b64  v[55:56], v[79:80], v166", 0xd2c20037U, 0x00034d4fU, true, true, "" },
    { "   v_lshr_b64  v[55:56], v[79:80], v166", 0xd2c40037U, 0x00034d4fU, true, true, "" },
    { "   v_ashr_i64  v[55:56], v[79:80], v166", 0xd2c60037U, 0x00034d4fU, true, true, "" },
    { "   v_add_f64  v[55:56], v[79:80], v[166:167]",
        0xd2c80037U, 0x00034d4fU, true, true, "" },
    { "   v_mul_f64  v[55:56], v[79:80], v[166:167]",
        0xd2ca0037U, 0x00034d4fU, true, true, "" },
    { "   v_min_f64  v[55:56], v[79:80], v[166:167]",
        0xd2cc0037U, 0x00034d4fU, true, true, "" },
    { "   v_max_f64  v[55:56], v[79:80], v[166:167]",
        0xd2ce0037U, 0x00034d4fU, true, true, "" },
    { "   v_ldexp_f64  v[55:56], v[79:80], v166",
        0xd2d00037U, 0x00034d4fU, true, true, "" },
    { "   v_mul_lo_u32  v55, v79, v166", 0xd2d20037U, 0x00034d4fU, true, true, "" },
    { "   v_mul_hi_u32  v55, v79, v166", 0xd2d40037U, 0x00034d4fU, true, true, "" },
    { "   v_mul_lo_i32  v55, v79, v166", 0xd2d60037U, 0x00034d4fU, true, true, "" },
    { "   v_mul_hi_i32  v55, v79, v166", 0xd2d80037U, 0x00034d4fU, true, true, "" },
    { "   v_div_scale_f32  v55, s[38:39], v79, v166, v79",
        0xd2da2637U, 0x053f4d4fU, true, true, "" },
    { "   v_div_scale_f32  v55, s[38:39], v79, v79, v229",
        0xd2da2637U, 0x07969f4fU, true, true, "" },
    { "   v_div_scale_f64 v[55:56], s[38:39], v[79:80], v[166:167], v[79:80]",
        0xd2dc2637U, 0x053f4d4fU, true, true, "" },
    { "   v_div_scale_f64 v[55:56], s[38:39], v[79:80], v[79:80], v[229:230]",
        0xd2dc2637U, 0x07969f4fU, true, true, "" },
    { "   v_div_fmas_f32  v55, v79, v166, v229", 0xd2de0037U, 0x07974d4fU, true, true, "" },
    { "   v_div_fmas_f64  v[55:56], v[79:80], v[166:167], v[229:230]",
        0xd2e00037U, 0x07974d4fU, true, true, "" },
    { "   v_msad_u8  v55, v79, v166, v229", 0xd2e20037U, 0x07974d4fU, true, true, "" },
    { "   v_qsad_u8  v[55:56], v[79:80], v166, v[229:230]",
        0xd2e40037U, 0x07974d4fU, true, true, "" },
    { "   v_mqsad_u8  v[55:56], v[79:80], v166, v[229:230]",
        0xd2e60037U, 0x07974d4fU, true, true, "" },
    { "   v_trig_preop_f64  v[55:56], v[79:80], v166",
        0xd2e80037U, 0x00034d4fU, true, true, "" },
    /* VINTRP encoding */
    { "   v_interp_p1_f32 v93, v211, attr26.w", 0xc9746bd3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93, v211, attr26 . w", 0xc9746bd3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93, v211, aTTR26 . W", 0xc9746bd3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93, v211, aTTR00026 . W", 0xc9746bd3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93, v211, attr26.x", 0xc97468d3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93, v211, attr26.y", 0xc97469d3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93, v211, attr26.z", 0xc9746ad3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93, v211, attr63.z", 0xc974fed3U, 0, false, true, "" },
    { "   v_interp_p1_f32 v93  , v211  , attr63.z", 0xc974fed3U, 0, false, true, "" },
    { "   v_interp_p2_f32 v93, v211, attr26.w", 0xc9756bd3U, 0, false, true, "" },
    { "   v_interp_mov_f32 v93, p10, attr26.w", 0xc9766b00U, 0, false, true, "" },
    { "   v_interp_mov_f32 v93, p20, attr26.w", 0xc9766b01U, 0, false, true, "" },
    { "   v_interp_mov_f32 v93, p0, attr26.w", 0xc9766b02U, 0, false, true, "" },
    /* VINTRP errors */
    { "   v_interp_p1_f32 v[93:94], v211, attr26.w", 0, 0, false, false,
        "test.s:1:20: Error: Required 1 vector register\n" },
    { "   v_interp_p1_f32 v93, v211, at26.w", 0, 0, false, false,
        "test.s:1:31: Error: Expected 'attr' keyword\n" },
    { "   v_interp_p1_f32 v93, v211, ", 0, 0, false, false,
        "test.s:1:31: Error: Expected 'attr' keyword\n" },
    { "   v_interp_p1_f32 v93, v211, attr22111.x", 0, 0, false, false,
        "test.s:1:38: Error: Number is too big\n" },
    { "   v_interp_p1_f32 v93, v211, attr64.w", 0, 0, false, false,
        "test.s:1:35: Error: Attribute number out of range (0-63)\n" },
    { "   v_interp_p1_f32 v93, v211, attr26", 0, 0, false, false,
        "test.s:1:37: Error: Expected '.' after attribute number\n" },
    { "   v_interp_p1_f32 v93, v211, attr26,", 0, 0, false, false,
        "test.s:1:37: Error: Expected '.' after attribute number\n" },
    { "   v_interp_p1_f32 v93, v211, attr26.t", 0, 0, false, false,
        "test.s:1:38: Error: Expected attribute component\n" },
    { "   v_interp_p1_f32 v93, v211, attr26.", 0, 0, false, false,
        "test.s:1:38: Error: Expected attribute component\n" },
    { "   v_interp_mov_f32 v93, xx, attr26.w", 0, 0, false, false,
        "test.s:1:26: Error: Unknown VINTRP parameter\n" },
    { "   v_interp_mov_f32 v93, xxffff, attr26.w", 0, 0, false, false,
        "test.s:1:32: Error: VINTRP parameter is too long\n" },
    /* DS encoding */
    { "   ds_add_u32  v71, v169 offset:52583", 0xd800cd67U, 0x0000a947U, true, true, "" },
    { "   ds_add_u32  v71, v169   ", 0xd8000000U, 0x0000a947U, true, true, "" },
    { "   ds_add_u32  v71, v169 offset :  52583",
        0xd800cd67U, 0x0000a947U, true, true, "" },
    { "   ds_add_u32  v71, v169", 0xd8000000U, 0x0000a947U, true, true, "" },
    { "dct=771; ds_add_u32  v71, v169 offset:dct+2",
        0xd8000305U, 0x0000a947U, true, true, "" },
    { "ds_add_u32  v71, v169 offset:dct+2; dct=771",
        0xd8000305U, 0x0000a947U, true, true, "" },
    { "   ds_add_u32  v71, v169 gds", 0xd8020000U, 0x0000a947U, true, true, "" },
    { "   ds_consume  v155 offset:52583", 0xd8f4cd67U, 0x9b000000U, true, true, "" },
    { "   ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87]"
      " offset0:103 offset1:205", 0xd9bccd67U, 0x8b56a947U, true, true, "" },
    { "   ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87]"
      " offset1:205 offset0:103", 0xd9bccd67U, 0x8b56a947U, true, true, "" },
    { "aa=32; bb=74;ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87]"
      " offset0:1+aa offset1:bb+1", 0xd9bc4b21U, 0x8b56a947U, true, true, "" },
    { "   ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87] offset0:1+aa "
        "offset1:bb+1;aa=32; bb=74", 0xd9bc4b21U, 0x8b56a947U, true, true, "" },
    { "   ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87] offset0:1+aa "
        "gds offset1:bb+1;aa=32; bb=74", 0xd9be4b21U, 0x8b56a947U, true, true, "" },
    /* DS warnings */
    { "   ds_add_u32  v71, v169 offset:52 offset:42",
        0xd800002aU, 0x0000a947U, true, true,
        "test.s:1:36: Warning: Offset is already defined\n" },
    { "   ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87] "
        "offset0:103 offset1:205 offset1:51 offset0:6",
        0xd9bc3306U, 0x8b56a947U, true, true,
        "test.s:1:89: Warning: Offset1 is already defined\n"
        "test.s:1:100: Warning: Offset0 is already defined\n" },
    /* DS errors */
    { "   ds_add_u32  v71, v169 xdff:5444", 0, 0, false, false,
        "test.s:1:26: Error: Expected 'offset'\n"
        "test.s:1:30: Error: Some garbages at DS modifier place\n" },
    { "   ds_add_u32  v71, v169 offset0:5444", 0, 0, false, false,
        "test.s:1:26: Error: Expected 'offset'\n"
        "test.s:1:33: Error: Some garbages at DS modifier place\n" },
    { "   ds_add_u32  v71, v169 offset1:5444", 0, 0, false, false,
        "test.s:1:26: Error: Expected 'offset'\n"
        "test.s:1:33: Error: Some garbages at DS modifier place\n" },
    { "   ds_wrxchg2st64_rtn_b64 v139, v[71:72], v169, v86"
      " offset0:103 offset1:205", 0, 0, false, false,
        "test.s:1:27: Error: Required 4 vector registers\n"
        "test.s:1:33: Error: Required 1 vector register\n"
        "test.s:1:43: Error: Required 2 vector registers\n"
        "test.s:1:49: Error: Required 2 vector registers\n" },
    { "   ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87] offset2:103 "
        "offsetx:205 offset:3221 oxxs", 0, 0, false, false,
        "test.s:1:65: Error: Expected 'offset', 'offset0' or 'offset1'\n"
        "test.s:1:72: Error: Some garbages at DS modifier place\n"
        "test.s:1:77: Error: Expected 'offset', 'offset0' or 'offset1'\n"
        "test.s:1:84: Error: Some garbages at DS modifier place\n"
        "test.s:1:89: Error: Expected 'offset', 'offset0' or 'offset1'\n"
        "test.s:1:95: Error: Some garbages at DS modifier place\n"
        "test.s:1:101: Error: Expected 'offset', 'offset0' or 'offset1'\n" },
    { "   ds_xor_b64  v71, v[169:170], v3 offset:52583", 0, 0, false, false,
        "test.s:1:31: Error: Some garbages at DS modifier place\n"
        "test.s:1:33: Error: Expected 'offset'\n" },
    { "   ds_gws_init  v71 offset:52583", 0, 0, false, false,
        "test.s:1:4: Error: Instruction requires GDS modifier\n" },
    { "   ds_add_u32  v71, v169 offset:,", 0, 0, false, false,
        "test.s:1:33: Error: Expected expression\n"
        "test.s:1:33: Error: Some garbages at DS modifier place\n" },
    { "   ds_add_u32  v71, v169 offset,", 0, 0, false, false,
        "test.s:1:32: Error: Expected ':' before offset\n"
        "test.s:1:32: Error: Some garbages at DS modifier place\n" },
    { "   ds_write2_b32  v71, v169, v39 offset0, offset1,",
        0, 0, false, false,
        "test.s:1:41: Error: Expected ':' before offset\n"
        "test.s:1:41: Error: Some garbages at DS modifier place\n"
        "test.s:1:50: Error: Expected ':' before offset\n"
        "test.s:1:50: Error: Some garbages at DS modifier place\n" },
    /* DS instructions */
    { "   ds_sub_u32  v71, v169 offset:52583", 0xd804cd67U, 0x0000a947U, true, true, "" },
    { "   ds_rsub_u32 v71, v169 offset:52583", 0xd808cd67U, 0x0000a947U, true, true, "" },
    { "   ds_inc_u32  v71, v169 offset:52583", 0xd80ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_dec_u32  v71, v169 offset:52583", 0xd810cd67U, 0x0000a947U, true, true, "" },
    { "   ds_min_i32  v71, v169 offset:52583", 0xd814cd67U, 0x0000a947U, true, true, "" },
    { "   ds_max_i32  v71, v169 offset:52583", 0xd818cd67U, 0x0000a947U, true, true, "" },
    { "   ds_min_u32  v71, v169 offset:52583", 0xd81ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_max_u32  v71, v169 offset:52583", 0xd820cd67U, 0x0000a947U, true, true, "" },
    { "   ds_and_b32  v71, v169 offset:52583", 0xd824cd67U, 0x0000a947U, true, true, "" },
    { "   ds_or_b32  v71, v169 offset:52583", 0xd828cd67U, 0x0000a947U, true, true, "" },
    { "   ds_xor_b32  v71, v169 offset:52583", 0xd82ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_mskor_b32  v71, v169, v39 offset:52583",
        0xd830cd67U, 0x0027a947U, true, true, "" },
    { "   ds_write_b32 v71, v169 offset:52583", 0xd834cd67U, 0x0000a947U, true, true, "" },
    { "   ds_write2_b32  v71, v169, v39 offset0:103 offset1:205",
        0xd838cd67U, 0x0027a947U, true, true, "" },
    { "   ds_write2st64_b32 v71, v169, v39 offset0:103 offset1:205",
        0xd83ccd67U, 0x0027a947U, true, true, "" },
    { "   ds_cmpst_b32  v71, v169, v39 offset:52583",
        0xd840cd67U, 0x0027a947U, true, true, "" },
    { "   ds_cmpst_f32  v71, v169, v39 offset:52583",
        0xd844cd67U, 0x0027a947U, true, true, "" },
    { "   ds_min_f32  v71, v169 offset:52583", 0xd848cd67U, 0x0000a947U, true, true, "" },
    { "   ds_max_f32  v71, v169 offset:52583", 0xd84ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_gws_init  v71 offset:52583 gds", 0xd866cd67U, 0x00000047U, true, true, "" },
    { "   ds_gws_sema_v v71 offset:52583 gds", 0xd86acd67U, 0x00000047U, true, true, "" },
    { "   ds_gws_sema_br v71 offset:52583 gds", 0xd86ecd67U, 0x00000047U, true, true, "" },
    { "   ds_gws_sema_p v71 offset:52583 gds", 0xd872cd67U, 0x00000047U, true, true, "" },
    { "   ds_gws_barrier v71 offset:52583 gds", 0xd876cd67U, 0x00000047U, true, true, "" },
    { "   ds_write_b8  v71, v169 offset:52583", 0xd878cd67U, 0x0000a947U, true, true, "" },
    { "   ds_write_b16  v71, v169 offset:52583", 0xd87ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_add_rtn_u32  v155, v71, v169 offset:52583",
        0xd880cd67U, 0x9b00a947U, true,  true, "" },
    { "   ds_sub_rtn_u32  v155, v71, v169 offset:52583",
        0xd884cd67U, 0x9b00a947U, true,  true, "" },
    { "   ds_rsub_rtn_u32  v155, v71, v169 offset:52583",
        0xd888cd67U, 0x9b00a947U, true,  true, "" },
    { "   ds_rsub_rtn_u32  v155, v71, v169 offset:52583",
        0xd888cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_inc_rtn_u32  v155, v71, v169 offset:52583",
        0xd88ccd67U, 0x9b00a947U, true, true, "" },
    { "   ds_dec_rtn_u32  v155, v71, v169 offset:52583",
        0xd890cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_min_rtn_i32  v155, v71, v169 offset:52583",
        0xd894cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_max_rtn_i32  v155, v71, v169 offset:52583",
        0xd898cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_min_rtn_u32  v155, v71, v169 offset:52583",
        0xd89ccd67U, 0x9b00a947U, true, true, "" },
    { "   ds_max_rtn_u32  v155, v71, v169 offset:52583",
        0xd8a0cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_and_rtn_b32  v155, v71, v169 offset:52583",
        0xd8a4cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_or_rtn_b32  v155, v71, v169 offset:52583",
        0xd8a8cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_xor_rtn_b32  v155, v71, v169 offset:52583",
        0xd8accd67U, 0x9b00a947U, true, true, "" },
    { "   ds_mskor_rtn_b32 v155, v71, v169, v26 offset:52583",
        0xd8b0cd67U, 0x9b1aa947U, true, true, "" },
    { "   ds_wrxchg_rtn_b32  v155, v71, v169 offset:52583",
        0xd8b4cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_wrxchg2_rtn_b32 v[155:156], v71, v169, v26 offset0:103 offset1:205",
        0xd8b8cd67U, 0x9b1aa947U, true, true, "" },
    { "   ds_wrxchg2st64_rtn_b32  v[155:156], v71, v169, v26 offset0:103 offset1:205",
        0xd8bccd67U, 0x9b1aa947U, true, true, "" },
    { "   ds_cmpst_rtn_b32  v155, v71, v169, v26 offset:52583",
        0xd8c0cd67U, 0x9b1aa947U, true, true, "" },
    { "   ds_cmpst_rtn_f32  v155, v71, v169, v26 offset:52583",
        0xd8c4cd67U, 0x9b1aa947U, true, true, "" },
    { "   ds_min_rtn_f32  v155, v71, v169 offset:52583",
        0xd8c8cd67U, 0x9b00a947U, true, true, "" },
    { "   ds_max_rtn_f32  v155, v71, v169 offset:52583",
        0xd8cccd67U, 0x9b00a947U, true, true, "" },
    { "   ds_swizzle_b32  v155, v71 offset:52583",
        0xd8d4cd67U, 0x9b000047U, true, true, "" },
    { "   ds_read_b32 v155, v71 offset:52583", 0xd8d8cd67U, 0x9b000047U, true, true, "" },
    { "   ds_read2_b32 v[155:156], v71 offset0:103 offset1:205",
        0xd8dccd67U, 0x9b000047U, true, true, "" },
    { "   ds_read2st64_b32 v[155:156], v71 offset0:103 offset1:205",
        0xd8e0cd67U, 0x9b000047U, true, true, "" },
    { "   ds_read_i8  v155, v71 offset:52583", 0xd8e4cd67U, 0x9b000047U, true, true, "" },
    { "   ds_read_u8  v155, v71 offset:52583", 0xd8e8cd67U, 0x9b000047U, true, true, "" },
    { "   ds_read_i16  v155, v71 offset:52583", 0xd8eccd67U, 0x9b000047U, true, true, "" },
    { "   ds_read_u16  v155, v71 offset:52583", 0xd8f0cd67U, 0x9b000047U, true, true, "" },
    { "   ds_consume  v155 offset:52583", 0xd8f4cd67U, 0x9b000000U, true, true, "" },
    { "   ds_append  v155 offset:52583", 0xd8f8cd67U, 0x9b000000U, true, true, "" },
    { "   ds_ordered_count v155, v71 offset:52583",
        0xd8fccd67U, 0x9b000047U, true, true, "" },
    { "   ds_add_u64  v71, v[169:170] offset:52583",
        0xd900cd67U, 0x0000a947U, true, true, "" },
    { "   ds_sub_u64  v71, v[169:170] offset:52583",
        0xd904cd67U, 0x0000a947U, true, true, "" },
    { "   ds_rsub_u64  v71, v[169:170] offset:52583",
        0xd908cd67U, 0x0000a947U, true, true, "" },
    { "   ds_inc_u64  v71, v[169:170] offset:52583",
        0xd90ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_dec_u64  v71, v[169:170] offset:52583",
        0xd910cd67U, 0x0000a947U, true, true, "" },
    { "   ds_min_i64  v71, v[169:170] offset:52583",
        0xd914cd67U, 0x0000a947U, true, true, "" },
    { "   ds_max_i64  v71, v[169:170] offset:52583",
        0xd918cd67U, 0x0000a947U, true, true, "" },
    { "   ds_min_u64  v71, v[169:170] offset:52583",
        0xd91ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_max_u64  v71, v[169:170] offset:52583",
        0xd920cd67U, 0x0000a947U, true, true, "" },
    { "   ds_and_b64  v71, v[169:170] offset:52583",
        0xd924cd67U, 0x0000a947U, true, true, "" },
    { "   ds_or_b64  v71, v[169:170] offset:52583",
        0xd928cd67U, 0x0000a947U, true, true, "" },
    { "   ds_xor_b64  v71, v[169:170] offset:52583",
        0xd92ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_mskor_b64  v71, v[169:170], v[23:24] offset:52583",
        0xd930cd67U, 0x0017a947U, true, true, "" },
    { "   ds_write_b64  v71, v[169:170] offset:52583",
        0xd934cd67U, 0x0000a947U, true, true, "" },
    { "   ds_write2_b64  v71, v[169:170], v[39:40] offset0:103 offset1:205",
        0xd938cd67U, 0x0027a947U, true, true, "" },
    { "   ds_write2st64_b64 v71, v[169:170], v[39:40] offset0:103 offset1:205",
        0xd93ccd67U, 0x0027a947U, true, true, "" },
    { "   ds_cmpst_b64  v71, v[169:170], v[39:40] offset:52583",
        0xd940cd67U, 0x0027a947U, true, true, "" },
    { "   ds_cmpst_f64  v71, v[169:170], v[39:40] offset:52583",
        0xd944cd67U, 0x0027a947U, true, true, "" },
    { "   ds_min_f64  v71, v[169:170] offset:52583",
        0xd948cd67U, 0x0000a947U, true, true, "" },
    { "   ds_max_f64  v71, v[169:170] offset:52583",
        0xd94ccd67U, 0x0000a947U, true, true, "" },
    { "   ds_add_rtn_u64  v[139:140], v71, v[169:170] offset:52583",
        0xd980cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_sub_rtn_u64  v[139:140], v71, v[169:170] offset:52583",
        0xd984cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_rsub_rtn_u64  v[139:140], v71, v[169:170] offset:52583",
        0xd988cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_inc_rtn_u64  v[139:140], v71, v[169:170] offset:52583",
        0xd98ccd67U, 0x8b00a947U, true, true, "" },
    { "   ds_dec_rtn_u64  v[139:140], v71, v[169:170] offset:52583",
        0xd990cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_min_rtn_i64  v[139:140], v71, v[169:170] offset:52583",
        0xd994cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_max_rtn_i64  v[139:140], v71, v[169:170] offset:52583",
        0xd998cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_min_rtn_u64  v[139:140], v71, v[169:170] offset:52583",
        0xd99ccd67U, 0x8b00a947U, true, true, "" },
    { "   ds_max_rtn_u64  v[139:140], v71, v[169:170] offset:52583",
        0xd9a0cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_and_rtn_b64  v[139:140], v71, v[169:170] offset:52583",
        0xd9a4cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_or_rtn_b64  v[139:140], v71, v[169:170] offset:52583",
        0xd9a8cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_xor_rtn_b64  v[139:140], v71, v[169:170] offset:52583",
        0xd9accd67U, 0x8b00a947U, true, true, "" },
    { "   ds_mskor_rtn_b64 v[139:140], v71, v[169:170], v[86:87] offset:52583",
        0xd9b0cd67U, 0x8b56a947U, true, true, "" },
    { "   ds_wrxchg_rtn_b64  v[139:140], v71, v[169:170] offset:52583",
        0xd9b4cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_wrxchg2_rtn_b64 v[139:142], v71, v[169:170], v[86:87] "
        "offset0:103 offset1:205", 0xd9b8cd67U, 0x8b56a947U, true, true, "" },
    { "   ds_wrxchg2st64_rtn_b64 v[139:142], v71, v[169:170], v[86:87] "
        "offset0:103 offset1:205", 0xd9bccd67U, 0x8b56a947U, true, true, "" },
    { "   ds_cmpst_rtn_b64 v[139:140], v71, v[169:170], v[86:87] offset:52583",
        0xd9c0cd67U, 0x8b56a947U, true, true, "" },
    { "   ds_cmpst_rtn_f64 v[139:140], v71, v[169:170], v[86:87] offset:52583",
        0xd9c4cd67U, 0x8b56a947U, true, true, "" },
    { "   ds_min_rtn_f64 v[139:140], v71, v[169:170] offset:52583",
        0xd9c8cd67U, 0x8b00a947U, true, true, "" },
    { "   ds_max_rtn_f64 v[139:140], v71, v[169:170] offset:52583",
        0xd9cccd67U, 0x8b00a947U, true, true, "" },
    { "   ds_read_b64  v[139:140], v71 offset:52583",
        0xd9d8cd67U, 0x8b000047U, true, true, "" },
    { "   ds_read2_b64  v[139:142], v71 offset0:103 offset1:205",
        0xd9dccd67U, 0x8b000047U, true, true, "" },
    { "   ds_read2st64_b64 v[139:142], v71 offset0:103 offset1:205",
        0xd9e0cd67U, 0x8b000047U, true, true, "" },
    { "   ds_add_src2_u32 v71 offset:52583", 0xda00cd67U, 0x00000047U, true, true, "" },
    { "   ds_sub_src2_u32 v71 offset:52583", 0xda04cd67U, 0x00000047U, true, true, "" },
    { "   ds_rsub_src2_u32 v71 offset:52583", 0xda08cd67U, 0x00000047U, true, true, "" },
    { "   ds_inc_src2_u32 v71 offset:52583", 0xda0ccd67U, 0x00000047U, true, true, "" },
    { "   ds_dec_src2_u32 v71 offset:52583", 0xda10cd67U, 0x00000047U, true, true, "" },
    { "   ds_min_src2_i32 v71 offset:52583", 0xda14cd67U, 0x00000047U, true, true, "" },
    { "   ds_max_src2_i32 v71 offset:52583", 0xda18cd67U, 0x00000047U, true, true, "" },
    { "   ds_min_src2_u32 v71 offset:52583", 0xda1ccd67U, 0x00000047U, true, true, "" },
    { "   ds_max_src2_u32 v71 offset:52583", 0xda20cd67U, 0x00000047U, true, true, "" },
    { "   ds_and_src2_b32 v71 offset:52583", 0xda24cd67U, 0x00000047U, true, true, "" },
    { "   ds_or_src2_b32 v71 offset:52583", 0xda28cd67U, 0x00000047U, true, true, "" },
    { "   ds_xor_src2_b32 v71 offset:52583", 0xda2ccd67U, 0x00000047U, true, true, "" },
    { "   ds_write_src2_b32 v71 offset:52583", 0xda34cd67U, 0x00000047U, true, true, "" },
    { "   ds_min_src2_f32 v71 offset:52583", 0xda48cd67U, 0x00000047U, true, true, "" },
    { "   ds_max_src2_f32 v71 offset:52583", 0xda4ccd67U, 0x00000047U, true, true, "" },
    { "   ds_add_src2_u64 v71 offset:52583", 0xdb00cd67U, 0x00000047U, true, true, "" },
    { "   ds_sub_src2_u64 v71 offset:52583", 0xdb04cd67U, 0x00000047U, true, true, "" },
    { "   ds_rsub_src2_u64 v71 offset:52583", 0xdb08cd67U, 0x00000047U, true, true, "" },
    { "   ds_inc_src2_u64 v71 offset:52583", 0xdb0ccd67U, 0x00000047U, true, true, "" },
    { "   ds_dec_src2_u64 v71 offset:52583", 0xdb10cd67U, 0x00000047U, true, true, "" },
    { "   ds_min_src2_i64 v71 offset:52583", 0xdb14cd67U, 0x00000047U, true, true, "" },
    { "   ds_max_src2_i64 v71 offset:52583", 0xdb18cd67U, 0x00000047U, true, true, "" },
    { "   ds_min_src2_u64 v71 offset:52583", 0xdb1ccd67U, 0x00000047U, true, true, "" },
    { "   ds_max_src2_u64 v71 offset:52583", 0xdb20cd67U, 0x00000047U, true, true, "" },
    { "   ds_and_src2_b64 v71 offset:52583", 0xdb24cd67U, 0x00000047U, true, true, "" },
    { "   ds_or_src2_b64 v71 offset:52583", 0xdb28cd67U, 0x00000047U, true, true, "" },
    { "   ds_xor_src2_b64 v71 offset:52583", 0xdb2ccd67U, 0x00000047U, true, true, "" },
    { "   ds_write_src2_b64 v71 offset:52583", 0xdb34cd67U, 0x00000047U, true, true, "" },
    { "   ds_min_src2_f64 v71 offset:52583", 0xdb48cd67U, 0x00000047U, true, true, "" },
    { "   ds_max_src2_f64 v71 offset:52583", 0xdb4ccd67U, 0x00000047U, true, true, "" },
    /* MUBUF */
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 "
        "offset:603 glc slc addr64 tfe", 0xe000c25bU, 0x23d43d12U, true, true, "" },
    { "    buffer_load_format_x  v61, v[18:19], s[80:83], s35 "
        "offset:603 glc slc addr64 lds", 0xe001c25bU, 0x23543d12U, true, true, "" },
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 "
        "offset:603 glc slc addr64 lds tfe", 0, 0, false, false,
        "test.s:1:5: Error: Both LDS and TFE is illegal\n" },
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], 31 "
        "offset:603 glc slc addr64 tfe", 0xe000c25bU, 0x9fd43d12U, true, true, "" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35   ",
        0xe0000000U, 0x23143d12U, true, true, "" },
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 "
        "offset:603 glc slc addr64 tfe", 0xe000c25bU, 0x23d43d12U, true, true, "" },
    { "    buffer_load_format_x  v61, v[18:19], s[80:83], s35 offset:603 glc slc addr64",
        0xe000c25bU, 0x23543d12U, true, true, "" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset:603",
        0xe000025bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset  :  603",
        0xe000025bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 OfFset  :  603",
        0xe000025bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 "
        "glc slc addr64 tfe", 0xe000c000U, 0x23d43d12U, true, true, "" },
    { "xx=5; yy=7;buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 "
        "offset:xx+yy glc slc addr64 tfe", 0xe000c00cU, 0x23d43d12U, true, true, "" },
    { "buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 offset:xx+yy "
        "glc slc addr64 tfe;xx=5; yy=7", 0xe000c00cU, 0x23d43d12U, true, true, "" },
    { "    buffer_load_format_x v[61:62], v[18:19], s[80:83], s35 offen idxen "
        "glc slc tfe", 0xe0007000U, 0x23d43d12U, true, true, "" },
    { "    buffer_load_format_x v[61:62], v18, s[80:83], s35 offset:577 idxen "
        "glc slc tfe", 0xe0006241U, 0x23d43d12U, true, true, "" },
    /* MUBUF/MTBUF offset warnings */
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset:4603",
        0xe00001fbU, 0x23143d12U, true, true,
        "test.s:1:58: Warning: Value 0x11fb truncated to 0x1fb\n" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset:x; x=4603",
        0xe00001fbU, 0x23143d12U, true, true,
        "test.s:1:58: Warning: Value 0x11fb truncated to 0x1fb\n" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset:x glc; x=4603",
        0xe00041fbU, 0x23143d12U, true, true,
        "test.s:1:58: Warning: Value 0x11fb truncated to 0x1fb\n" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset:1779 offset:603",
        0xe000025bU, 0x23143d12U, true, true,
        "test.s:1:63: Warning: Offset is already defined\n" },
    /* MUBUF errors */
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 "
        "offset:603 glc slc addr64 lds tfe idxen", 0, 0, false, false,
        "test.s:1:5: Error: Idxen and offen must be zero in 64-bit address mode\n" },
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35 "
        "offset:603 glc slc addr64 lds tfe offen", 0, 0, false, false,
        "test.s:1:5: Error: Idxen and offen must be zero in 64-bit address mode\n" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset,", 0, 0, false, false,
        "test.s:1:57: Error: Expected ':' before offset\n"
        "test.s:1:57: Error: Some garbages at MUBUF modifier place\n" },
    { "    buffer_load_format_x  v[61:63], v[18:20], s[80:83], s35 "
        "offset:603 glc slc addr64 lds tfe", 0, 0, false, false,
        "test.s:1:27: Error: Required 2 vector registers\n"
        "test.s:1:37: Error: Required 2 vector registers\n" },
    { "    buffer_load_format_x  v[61:62], v[18:19], s[80:83], s35"
        "offset:603 glc slc addr64 lds", 0, 0, false, false,
        "test.s:1:57: Error: Literal in MUBUF is illegal\n"
        "test.s:1:66: Error: Some garbages at MUBUF modifier place\n"
        "test.s:1:27: Error: Required 1 vector register\n" },
    { "    buffer_load_format_x  v61, v18, s[80:83], s35 offset:603 tfx",
        0, 0, false, false, "test.s:1:62: Error: Unknown MUBUF modifier\n" },
    { "    buffer_store_format_xyz v[61:63], v18, s[80:83], s35 idxen offset:603 tfe",
        0, 0, false, false, "test.s:1:29: Error: Required 4 vector registers\n" },
    { "    buffer_store_format_xyzw v[61:64], v18, s[80:83], s35 idxen offset:603 tfe",
        0, 0, false, false, "test.s:1:30: Error: Required 5 vector registers\n" },
    { "    buffer_load_format_x v61, v18, s[80:83], 3435 idxen offset:603",
        0, 0, false, false, "test.s:1:46: Error: Literal in MUBUF is illegal\n" },
    { "    buffer_load_format_x v61, v18, s[80:83], xx idxen offset:603",
        0, 0, false, false, "test.s:1:46: Error: Literal in MUBUF is illegal\n" },
    /* other MUBUF instructions */
    { "    buffer_load_format_x v61, v18, s[80:83], s35 idxen offset:603",
        0xe000225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_format_xy v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe004225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_format_xyz v[61:63], v18, s[80:83], s35 idxen offset:603",
        0xe008225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_format_xyzw v[61:64], v18, s[80:83], s35 idxen offset:603",
        0xe00c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_format_x v61, v18, s[80:83], s35 idxen offset:603",
        0xe010225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_format_xy v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe014225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_format_xy v[61:63], v18, s[80:83], s35 idxen offset:603 tfe",
        0xe014225bU, 0x23943d12U, true, true, "" },
    { "    buffer_store_format_xyz v[61:63], v18, s[80:83], s35 idxen offset:603",
        0xe018225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_format_xyz v[61:64], v18, s[80:83], s35 idxen offset:603 tfe",
        0xe018225bU, 0x23943d12U, true, true, "" },
    { "    buffer_store_format_xyzw v[61:64], v18, s[80:83], s35 idxen offset:603",
        0xe01c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_format_xyzw v[61:65], v18, s[80:83], s35 idxen offset:603 tfe",
        0xe01c225bU, 0x23943d12U, true, true, "" },
    { "    buffer_load_ubyte v61, v18, s[80:83], s35 idxen offset:603",
        0xe020225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_sbyte v61, v18, s[80:83], s35 idxen offset:603",
        0xe024225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_ushort v61, v18, s[80:83], s35 idxen offset:603",
        0xe028225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_sshort v61, v18, s[80:83], s35 idxen offset:603",
        0xe02c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_dword v61, v18, s[80:83], s35 idxen offset:603",
        0xe030225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_dwordx2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe034225bU, 0x23143d12U, true, true, "" },
    { "    buffer_load_dwordx4 v[61:64], v18, s[80:83], s35 idxen offset:603",
        0xe038225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_byte v61, v18, s[80:83], s35 idxen offset:603",
        0xe060225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_short v61, v18, s[80:83], s35 idxen offset:603",
        0xe068225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_dword v61, v18, s[80:83], s35 idxen offset:603",
        0xe070225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_dwordx2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe074225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_dwordx4 v[61:64], v18, s[80:83], s35 idxen offset:603",
        0xe078225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_swap v61, v18, s[80:83], s35 idxen offset:603",
        0xe0c0225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_cmpswap v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe0c4225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_add v61, v18, s[80:83], s35 idxen offset:603",
        0xe0c8225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_sub v61, v18, s[80:83], s35 idxen offset:603",
        0xe0cc225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_rsub v61, v18, s[80:83], s35 idxen offset:603",
        0xe0d0225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_smin v61, v18, s[80:83], s35 idxen offset:603",
        0xe0d4225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_umin v61, v18, s[80:83], s35 idxen offset:603",
        0xe0d8225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_smax v61, v18, s[80:83], s35 idxen offset:603",
        0xe0dc225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_umax v61, v18, s[80:83], s35 idxen offset:603",
        0xe0e0225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_and v61, v18, s[80:83], s35 idxen offset:603",
        0xe0e4225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_or v61, v18, s[80:83], s35 idxen offset:603",
        0xe0e8225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_xor v61, v18, s[80:83], s35 idxen offset:603",
        0xe0ec225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_inc v61, v18, s[80:83], s35 idxen offset:603",
        0xe0f0225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_dec v61, v18, s[80:83], s35 idxen offset:603",
        0xe0f4225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_fcmpswap v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe0f8225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_fmin v61, v18, s[80:83], s35 idxen offset:603",
        0xe0fc225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_fmax v61, v18, s[80:83], s35 idxen offset:603",
        0xe100225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_swap_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe140225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_cmpswap_x2 v[61:64], v18, s[80:83], s35 idxen offset:603",
        0xe144225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_add_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe148225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_sub_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe14c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_rsub_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe150225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_smin_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe154225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_umin_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe158225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_smax_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe15c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_umax_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe160225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_and_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe164225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_or_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe168225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_xor_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe16c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_inc_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe170225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_dec_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe174225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_fcmpswap_x2 v[61:64], v18, s[80:83], s35 idxen offset:603",
        0xe178225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_fmin_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe17c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_atomic_fmax_x2 v[61:62], v18, s[80:83], s35 idxen offset:603",
        0xe180225bU, 0x23143d12U, true, true, "" },
    { "    buffer_wbinvl1_sc", 0xe1c00000U, 0x00000000U, true, true, "" },
    { "    buffer_wbinvl1_sc idxen offset:603 lds",
        0xe1c1225bU, 0x00000000U, true, true, "" },
    { "    buffer_wbinvl1", 0xe1c40000U, 0x00000000U, true, true, "" },
    /* MTBUF encoding */
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[8,sint]", 0xea8867d4U, 0x23d43d12U, true, true, "" },
    /* formats */
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe", 0xe80867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[sint]", 0xea8867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_8,BUF_NUM_FORMAT_SINT]",
        0xea8867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[16]", 0xe81067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[8_8]", 0xe81867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[32]", 0xe82067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[16_16]", 0xe82867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[10_11_11]", 0xe83067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[11_11_10]", 0xe83867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[10_10_10_2]", 0xe84067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[2_10_10_10]", 0xe84867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[8_8_8_8]", 0xe85067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[32_32]", 0xe85867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[16_16_16_16]", 0xe86067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[32_32_32]", 0xe86867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[32_32_32_32]", 0xe87067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_16]",
        0xe81067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_8_8]",
        0xe81867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_32]",
        0xe82067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_16_16]",
        0xe82867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_10_11_11]",
        0xe83067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_11_11_10]",
        0xe83867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_10_10_10_2]",
        0xe84067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_2_10_10_10]",
        0xe84867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_8_8_8_8]",
        0xe85067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_32_32]",
        0xe85867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_16_16_16_16]",
        0xe86067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_32_32_32]",
        0xe86867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_32_32_32_32]",
        0xe87067d4U, 0x23d43d12U, true, true, "" },
    
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[unorm]", 0xe80867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[snorm]", 0xe88867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[uscaled]", 0xe90867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[sscaled]", 0xe98867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[uint]", 0xea0867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[sint]", 0xea8867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[snorm_ogl]", 0xeb0867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[float]", 0xeb8867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_UNORM]",
        0xe80867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_SNORM]",
        0xe88867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_USCALED]",
        0xe90867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_SSCALED]",
        0xe98867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_UINT]",
        0xea0867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_SINT]",
        0xea8867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_SNORM_OGL]",
        0xeb0867d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_NUM_FORMAT_FLOAT]",
        0xeb8867d4U, 0x23d43d12U, true, true, "" },
    
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[10_11_11,sscaled]", 0xe9b067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format  : [  10_11_11 , sscaled  ]",
        0xe9b067d4U, 0x23d43d12U, true, true, "" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[BUF_DATA_FORMAT_10_11_11,BUF_NUM_FORMAT_SSCALED]",
        0xe9b067d4U, 0x23d43d12U, true, true, "" },
    /* MTBUF format errors */
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[]", 0, 0, false, false,
        "test.s:1:94: Error: Some garbages at data/number format place\n"
        "test.s:1:95: Error: Unterminated format modifier\n" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[xxx]", 0, 0, false, false,
        "test.s:1:94: Error: Unknown data/number format\n" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[16_16,aaa]", 0, 0, false, false,
        "test.s:1:100: Error: Unknown number format\n" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[16_16", 0, 0, false, false,
        "test.s:1:99: Error: Unterminated format modifier\n" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe format:[xx,yy", 0, 0, false, false,
        "test.s:1:94: Error: Unknown data/number format\n"
        "test.s:1:97: Error: Unknown number format\n"
        "test.s:1:99: Error: Unterminated format modifier\n" },
    /* MTBUF errors */
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe lds", 0, 0, false, false,
        "test.s:1:86: Error: Unknown MTBUF modifier\n" },
    { "    tbuffer_load_format_x v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc tfe offenx", 0, 0, false, false,
        "test.s:1:86: Error: Unknown MTBUF modifier\n" },
    /* MTBUF instructions */
    { "    tbuffer_load_format_xy v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc", 0xe80967d4U, 0x23543d12U, true, true, "" },
    { "    tbuffer_load_format_xyz v[61:63], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc", 0xe80a67d4U, 0x23543d12U, true, true, "" },
    { "    tbuffer_load_format_xyzw v[61:64], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc", 0xe80b67d4U, 0x23543d12U, true, true, "" },
    { "    tbuffer_store_format_x v61, v18, s[80:83], s35 idxen offset:2004 "
        "glc slc", 0xe80c67d4U, 0x23543d12U, true, true, "" },
    { "    tbuffer_store_format_xy v[61:62], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc", 0xe80d67d4U, 0x23543d12U, true, true, "" },
    { "    tbuffer_store_format_xyz v[61:63], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc", 0xe80e67d4U, 0x23543d12U, true, true, "" },
    { "    tbuffer_store_format_xyzw v[61:64], v18, s[80:83], s35 idxen offset:2004 "
        "glc slc", 0xe80f67d4U, 0x23543d12U, true, true, "" },
    /* MIMG encoding */
    { "    image_load  v[157:160], v[121:124], s[84:87] dmask:11 unorm glc slc r128 "
        "tfe lwe da", 0xf203fb00U, 0x00159d79U, true, true, "" },
    { "    image_load      v[157:158], v[121:124], s[84:87] unorm glc slc r128 tfe lwe da",
        0xf203f100U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc slc r128 "
        "lwe da", 0xf202fb00U, 0x00159d79U, true, true, "" },
    { "dmask=5;image_load  v[157:158], v[121:124], s[84:87] dmask:dmask unorm glc "
        "slc r128 lwe da", 0xf202f500U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:91]    ",
        0xf0000100U, 0x00159d79U, true, true, "" },
    /* dmask */
    { "    image_load  v157, v[121:124], s[84:87] dmask:1 unorm glc slc r128 lwe da",
        0xf202f100U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:87] dmask:2 unorm glc slc r128 lwe da",
        0xf202f200U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:3 unorm glc slc r128 "
        "lwe da", 0xf202f300U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:87] dmask:4 unorm glc slc r128 lwe da",
        0xf202f400U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:5 unorm glc slc r128 "
        "lwe da", 0xf202f500U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:6 unorm glc slc r128 "
        "lwe da", 0xf202f600U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:159], v[121:124], s[84:87] dmask:7 unorm glc slc r128 "
        "lwe da", 0xf202f700U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:87] dmask:8 unorm glc slc r128 lwe da",
        0xf202f800U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:9 unorm glc slc r128 "
        "lwe da", 0xf202f900U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:10 unorm glc slc r128 "
        "lwe da", 0xf202fa00U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:0xa unorm glc slc r128 "
        "lwe da", 0xf202fa00U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:160], v[121:124], s[84:87] dmask:15 unorm glc slc r128 "
        "lwe da", 0xf202ff00U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:160], v[121:124], s[84:87] dmask:0xf unorm glc slc r128 "
        "lwe da", 0xf202ff00U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:161], v[121:124], s[84:87] dmask:0xf unorm glc tfe slc r128 "
        "lwe da", 0xf203ff00U, 0x00159d79U, true, true, "" },
    /* MIMG flags */
    { "    image_load  v157, v[121:124], s[84:91]",
        0xf0000100U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:91] unorm",
        0xf0001100U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:91] glc",
        0xf0002100U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:91] da",
        0xf0004100U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:87] r128",
        0xf0008100U, 0x00159d79U, true, true, "" },
    { "    image_load  v[157:158], v[121:124], s[84:91] tfe",
        0xf0010100U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:91] lwe",
        0xf0020100U, 0x00159d79U, true, true, "" },
    { "    image_load  v157, v[121:124], s[84:91] slc",
        0xf2000100U, 0x00159d79U, true, true, "" },
    /* MIMG errors */
    { "    image_load  v157, v[121:124], s[84:91] sxc", 0, 0, false, false,
        "test.s:1:44: Error: Unknown MIMG modifier\n" },
    { "    image_load  v157, v[121:124], s[84:91] g53 aa3 daa db", 0, 0, false, false,
        "test.s:1:44: Error: Unknown MIMG modifier\n"
        "test.s:1:48: Error: Unknown MIMG modifier\n"
        "test.s:1:52: Error: Unknown MIMG modifier\n"
        "test.s:1:56: Error: Unknown MIMG modifier\n" },
    { "    image_load  v157, v[121:124], s[84:91] dmask:", 0, 0, false, false,
        "test.s:1:50: Error: Expected expression\n" },
    { "    image_load  v157, v[121:124], s[84:91] dmask", 0, 0, false, false,
        "test.s:1:49: Error: Expected ':' before dmask\n" },
    { "    image_load  v157, v[121:124], s[84:90]", 0, 0, false, false,
        "test.s:1:35: Error: Required 8 scalar registers\n" },
    { "    image_load  v157, v[121:124], s[84:90] r128", 0, 0, false, false,
        "test.s:1:35: Error: Required 4 scalar registers\n" },
    { "    image_store v157, v[121:124], s[84:91]", 0, 0, false, false,
        "test.s:1:5: Error: Unorm is not set for store or atomic instruction\n" },
    { "    image_atomic_add v157, v[121:124], s[84:91]", 0, 0, false, false,
        "test.s:1:5: Error: Unorm is not set for store or atomic instruction\n" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:8 unorm r128 lwe da",
        0, 0, false, false, "test.s:1:17: Error: Required 1 vector register\n" },
    { "    image_load  v[157:158], v[121:124], s[84:87] dmask:11 unorm r128 lwe da",
        0, 0, false, false, "test.s:1:17: Error: Required 3 vector registers\n" },
    { "    image_load  v157, v[121:124], s[84:87] dmask:0 unorm glc slc r128 lwe da",
        0, 0, false, false, "test.s:1:50: Error: Zero in dmask is illegal\n" },
    { "    image_load  v157, v[121:124], s[84:87] dmask:0x1ab0 unorm glc slc r128 lwe da",
        0, 0, false, false, "test.s:1:50: Warning: Dmask out of range (0-15)\n"
        "test.s:1:50: Error: Zero in dmask is illegal\n" },
    /* MIMG warnings */
    { "    image_load  v[157:161], v[121:124], s[84:87] dmask:0xdaf unorm glc tfe slc r128 "
        "lwe da", 0xf203ff00U, 0x00159d79U, true, true,
        "test.s:1:56: Warning: Dmask out of range (0-15)\n" },
    /* MIMG instructions */
    { "    image_load_mip  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf004fb00U, 0x00159d79U, true, true, "" },
    { "    image_load_pck  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf008fb00U, 0x00159d79U, true, true, "" },
    { "    image_load_pck_sgn  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf00cfb00U, 0x00159d79U, true, true, "" },
    { "    image_load_mip_pck  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf010fb00U, 0x00159d79U, true, true, "" },
    { "    image_load_mip_pck_sgn  v[157:159], v[121:124], s[84:87] dmask:11 unorm "
        "glc r128 da", 0xf014fb00U, 0x00159d79U, true, true, "" },
    { "    image_store  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf020fb00U, 0x00159d79U, true, true, "" },
    { "    image_store_mip  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf024fb00U, 0x00159d79U, true, true, "" },
    { "    image_store_pck  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf028fb00U, 0x00159d79U, true, true, "" },
    { "    image_store_mip_pck  v[157:159], v[121:124], s[84:87] dmask:11 unorm "
        "glc r128 da", 0xf02cfb00U, 0x00159d79U, true, true, "" },
    { "    image_get_resinfo  v[157:159], v[121:124], s[84:87] dmask:11 unorm "
        "glc r128 da", 0xf038fb00U, 0x00159d79U, true, true, "" },
    { "    image_get_resinfo  v[157:159], v[121:124], s[84:87] dmask:11 glc r128 da",
        0xf038eb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_swap  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf03cfb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_cmpswap  v[157:159], v[121:124], s[84:87] dmask:11 unorm "
        "glc r128 da", 0xf040fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_add  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf044fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_sub  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf048fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_rsub  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf04cfb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_smin  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf050fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_umin  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf054fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_smax  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf058fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_umax  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf05cfb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_and  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf060fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_or v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf064fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_xor  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf068fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_inc  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf06cfb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_dec  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf070fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_fcmpswap  v[157:159], v[121:124], s[84:87] dmask:11 unorm "
        "glc r128 da", 0xf074fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_fmin  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf078fb00U, 0x00159d79U, true, true, "" },
    { "    image_atomic_fmax  v[157:159], v[121:124], s[84:87] dmask:11 unorm glc r128 da",
        0xf07cfb00U, 0x00159d79U, true, true, "" },
    { "    image_sample  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf080fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_cl  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf084fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_d  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf088fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_d_cl  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf08cfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_l  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf090fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_b  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf094fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_b_cl  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf098fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_lz  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf09cfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0a0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_cl  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0a4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_d  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0a8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_d_cl  v[157:159], v[121:125], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0acfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_l  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0b0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_b  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0b4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_b_cl  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0b8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_lz  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0bcfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0c0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_cl_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0c4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_d_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0c8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_d_cl_o  v[157:159], v[121:125], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0ccfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_l_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0d0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_b_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0d4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_b_cl_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0d8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_lz_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0dcfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0e0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_cl_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0e4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_d_o  v[157:159], v[121:125], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0e8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_d_cl_o  v[157:159], v[121:126], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf0ecfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_l_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0f0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_b_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0f4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_b_cl_o  v[157:159], v[121:125], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf0f8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_lz_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf0fcfb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf100fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_cl  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf104fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_l  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf110fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_b  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf114fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_b_cl  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf118fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_lz  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf11cfb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf120fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_cl  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf124fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_l  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf130fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_b  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf134fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_b_cl  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf138fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_lz  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf13cfb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf140fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_cl_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf144fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_l_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf150fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_b_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf154fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_b_cl_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf158fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_lz_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf15cfb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf160fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_cl_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf164fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_l_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf170fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_b_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf174fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_b_cl_o  v[157:160], v[121:125], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf178fb00U, 0x02559d79U, true, true, "" },
    { "    image_gather4_c_lz_o  v[157:160], v[121:124], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf17cfb00U, 0x02559d79U, true, true, "" },
    { "    image_get_lod  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf180fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_cd  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf1a0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_cd_cl  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf1a4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_cd  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 unorm "
        "glc r128 da", 0xf1a8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_cd_cl  v[157:159], v[121:125], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf1acfb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_cd_o  v[157:159], v[121:124], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf1b0fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_cd_cl_o  v[157:159], v[121:125], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf1b4fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_cd_o  v[157:159], v[121:125], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf1b8fb00U, 0x02559d79U, true, true, "" },
    { "    image_sample_c_cd_cl_o  v[157:159], v[121:126], s[84:87], s[72:75] dmask:11 "
        "unorm glc r128 da", 0xf1bcfb00U, 0x02559d79U, true, true, "" },
    /* EXP encoding */
    { "exp  param5, v116, v93, v27, v124 done vm",
        0xf8001a5fU, 0x7c1b5d74U, true, true, "" },
    { "exp  param25, v116, v93, v27, v124 done vm",
        0xf8001b9fU, 0x7c1b5d74U, true, true, "" },
    { "exp  param0025, v116, v93, v27, v124 done vm",
        0xf8001b9fU, 0x7c1b5d74U, true, true, "" },
    { "exp  mrt0, v116, v93, v27, v124 done vm",
        0xf800180fU, 0x7c1b5d74U, true, true, "" },
    { "exp  mrt6, v116, v93, v27, v124 done vm",
        0xf800186fU, 0x7c1b5d74U, true, true, "" },
    { "exp  pos0, v116, v93, v27, v124 done vm",
        0xf80018cfU, 0x7c1b5d74U, true, true, "" },
    { "exp  pos1, v116, v93, v27, v124 done vm",
        0xf80018dfU, 0x7c1b5d74U, true, true, "" },
    { "exp  pos2, v116, v93, v27, v124 done vm",
        0xf80018efU, 0x7c1b5d74U, true, true, "" },
    { "exp  pos02, v116, v93, v27, v124 done vm",
        0xf80018efU, 0x7c1b5d74U, true, true, "" },
    { "exp  pos3, v116, v93, v27, v124 done vm",
        0xf80018ffU, 0x7c1b5d74U, true, true, "" },
    { "exp  null, v116, v93, v27, v124 done vm",
        0xf800189fU, 0x7c1b5d74U, true, true, "" },
    { "exp  mrtz, v116, v93, v27, v124 done vm",
        0xf800188fU, 0x7c1b5d74U, true, true, "" },
    /* EXP encoding: flags */
    { "exp  param5, v116, v93, v27, v124 done", 0xf8000a5fU, 0x7c1b5d74U, true, true, "" },
    { "exp  param5, v116, v93, v27, v124", 0xf800025fU, 0x7c1b5d74U, true, true, "" },
    { "exp  param5, v116, v93, v27, v124   ", 0xf800025fU, 0x7c1b5d74U, true, true, "" },
    { "exp  mrt0, v116, v116, v93, off done compr vm",
        0xf8001c07U, 0x00005d74U, true, true, "" },
    { "exp  mrt0, v116, v116, v93, v93 done compr vm",
        0xf8001c0fU, 0x00005d74U, true, true, "" },
    /* EXP encoding src enables */
    { "exp  param5, v116, v93, v27, off done", 0xf8000a57U, 0x001b5d74U, true, true, "" },
    { "exp  param5, v116, v93, off, v124 done", 0xf8000a5bU, 0x7c005d74U, true, true, "" },
    { "exp  param5, v116, off, off, v124 done", 0xf8000a59U, 0x7c000074U, true, true, "" },
    { "exp  param5, off, v93, v27, off done", 0xf8000a56U, 0x001b5d00U, true, true, "" },
    { "exp  param5, off, v93, off, off done", 0xf8000a52U, 0x00005d00U, true, true, "" },
    { "exp  mrt0, v116, v116, v93, off done compr vm",
        0xf8001c07U, 0x00005d74U, true, true, "" },
    { "exp  mrt0, off , v116, v93, v93 done compr vm",
        0xf8001c0eU, 0x00005d00U, true, true, "" },
    { "exp  mrt0, off , v116, off, v93 done compr vm", 0xf8001c0aU, 0, true, true, "" },
    { "exp  mrt0, v116, off, v93, off done compr vm",
        0xf8001c05U, 0x00005d74U, true, true, "" },
    { "exp  mrt0, off, off, v93, off done compr vm",
        0xf8001c04U, 0x00005d00U, true, true, "" },
    { "exp  mrt0, v116, off, off, off done compr vm",
        0xf8001c01U, 0x00000074U, true, true, "" },
    { "exp  param5, v116, v93, off, off",
        0xf8000253U, 0x00005d74U, true, true, "" },
    /* EXP encoding errors */
    { "exp  param5, v116, v93, v27, v124 xxx", 0, 0, false, false,
        "test.s:1:35: Error: Unknown EXP modifier\n" },
    { "exp  param32, v116, v93, v27, v124 done vm", 0, 0, false, false,
        "test.s:1:6: Error: Param number out of range (0-31)\n" },
    { "exp  pos4, v116, v93, v27, v124 done vm", 0, 0, false, false,
        "test.s:1:6: Error: Pos number out of range (0-3)\n" },
    { "exp  mrt8, v116, v93, v27, v124 done vm", 0, 0, false, false,
        "test.s:1:6: Error: MRT number out of range (0-7)\n" },
    { "exp  mrt0, v116, v117, v93, v93 done compr vm", 0, 0, false, false,
        "test.s:1:18: Error: VSRC1 must be equal to VSRC0 in compr mode\n" },
    { "exp  mrt0, v116, v116, v93, v91 done compr vm", 0, 0, false, false,
        "test.s:1:29: Error: VSRC3 must be equal to VSRC2 in compr mode\n" },
    { "exp  paxxx, v116, v93, v27, v124 done vm", 0, 0, false, false,
        "test.s:1:6: Error: Unknown EXP target\n" },
    { "exp  param, v116, v93, v27, v124 done vm", 0, 0, false, false,
        "test.s:1:6: Error: Missing number\n" },
    { nullptr, 0, 0, false, false, 0 }
};

const GCNAsmOpcodeCase encGCN11OpcodeCases[] =
{   /* flat registers */
    { "    s_add_u32  flat_scratch_lo, s4, s61", 0x80683d04U, 0, false, true, "" },
    { "    s_add_u32  flat_scratch_hi, s4, s61", 0x80693d04U, 0, false, true, "" },
    { "    s_xor_b64 s[22:23], flat_scratch, s[62:63]\n",
        0x89963e68U, 0, false, true, "" },
    /* SOPP encoding */
    { "    s_setkill  0x32b", 0xbf8b032bU, 0, false, true, "" },
    { "xxx: s_cbranch_cdbgsys xxx+16", 0xbf970003U, 0, false, true, "" },
    { "xxx: s_cbranch_cdbguser xxx+16", 0xbf980003U, 0, false, true, "" },
    { "xxx: s_cbranch_cdbgsys_or_user xxx+16", 0xbf990003U, 0, false, true, "" },
    { "xxx: s_cbranch_cdbgsys_and_user xxx+16", 0xbf9a0003U, 0, false, true, "" },
    /* SMRD */
    { "   s_dcache_inv_vol", 0xc7400000U, 0, false, true, "" },
    /* VOP1 encoding */
    { "    v_trunc_f64  v[158:159], v[79:80]", 0x7f3c2f4fU, 0, false, true, "" },
    { "    v_ceil_f64  v[158:159], v[79:80]", 0x7f3c314fU, 0, false, true, "" },
    { "    v_rndne_f64  v[158:159], v[79:80]", 0x7f3c334fU, 0, false, true, "" },
    { "    v_floor_f64  v[158:159], v[79:80]", 0x7f3c354fU, 0, false, true, "" },
    { "    v_log_legacy_f32 v158, v79", 0x7f3c8b4fU, 0, false, true, "" },
    { "    v_exp_legacy_f32 v158, v79", 0x7f3c8d4fU, 0, false, true, "" },
    /* VOP1 in VOP3 */
    { "    v_trunc_f64  v[55:56], v[27:28] vop3",
        0xd32e0037U, 0x0000011bU, true, true, "" },
    { "    v_ceil_f64  v[55:56], v[27:28] vop3",
        0xd3300037U, 0x0000011bU, true, true, "" },
    { "    v_rndne_f64  v[55:56], v[27:28] vop3",
        0xd3320037U, 0x0000011bU, true, true, "" },
    { "    v_floor_f64  v[55:56], v[27:28] vop3",
        0xd3340037U, 0x0000011bU, true, true, "" },
    { "    v_log_legacy_f32 v55, v27 vop3",
        0xd38a0037U, 0x0000011bU, true, true, "" },
    { "    v_exp_legacy_f32 v55, v27 vop3",
        0xd38c0037U, 0x0000011bU, true, true, "" },
    /* VOP3 encoding */
    { "   v_msad_u8  v55, v79, v166, v229", 0xd2e20037U, 0x07974d4fU, true, true, "" },
    { "    v_qsad_pk_u16_u8  v[55:56], v[79:80], v166, v[229:230]",
        0xd2e40037U, 0x07974d4fU, true, true, "" },
    { "    v_mqsad_pk_u16_u8  v[55:56], v[79:80], v166, v[229:230]",
        0xd2e60037U, 0x07974d4fU, true, true, "" },
    { "    v_mqsad_u32_u8  v[55:56], v[79:80], v166, v[229:230]",
        0xd2ea0037U, 0x07974d4fU, true, true, "" },
    { "    v_mad_u64_u32  v[55:56], s[46:47], v79, v166, v[229:230]",
        0xd2ec2e37U, 0x07974d4fU, true, true, "" },
    { "    v_mad_i64_i32  v[55:56], s[46:47], v79, v166, v[229:230]",
        0xd2ee2e37U, 0x07974d4fU, true, true, "" },
    /* DS encoding */
    { "    ds_nop  v71 offset:52583", 0xd850cd67U, 0x00000047U, true, true, "" },
    { "    ds_gws_sema_release_all v71 gds offset:52583",
        0xd862cd67U, 0x00000047U, true, true, "" },
    { "    ds_wrap_rtn_b32  v155, v71, v169, v86 offset:52583",
        0xd8d0cd67U, 0x9b56a947U, true, true, "" },
    { "   ds_condxchg32_rtn_b64 v[155:156], v71, v[169:170] offset:52583",
        0xd9f8cd67U, 0x9b00a947U, true, true, "" },
    { "    ds_write_b96  v71, v[169:171] offset:52583",
        0xdb78cd67U, 0x0000a947U, true, true, "" },
    { "    ds_write_b128  v71, v[169:172] offset:52583",
        0xdb7ccd67U, 0x0000a947U, true, true, "" },
    { "    ds_condxchg32_rtn_b128  v[155:158] offset:52583",
        0xdbf4cd67U, 0x9b000000U, true, true, "" },
    { "    ds_read_b96  v[155:157], v71 offset:52583",
        0xdbf8cd67U, 0x9b000047U, true, true, "" },
    { "    ds_read_b128  v[155:158], v71 offset:52583",
        0xdbfccd67U, 0x9b000047U, true, true, "" },
    /* MUBUF encoding */
    { "    buffer_load_dwordx3 v[61:63], v18, s[80:83], s35 idxen offset:603",
        0xe03c225bU, 0x23143d12U, true, true, "" },
    { "    buffer_store_dwordx3 v[61:63], v18, s[80:83], s35 idxen offset:603",
        0xe07c225bU, 0x23143d12U, true, true, "" },
    /* MUBUF unknown instrunctions */
    { "    buffer_atomic_rsub v61, v18, s[80:83], s35 idxen offset:603",
        0, 0, false, false, "test.s:1:5: Error: Unknown instruction\n" },
    { "    buffer_atomic_rsub v[61:62], v18, s[80:83], s35 idxen offset:603",
        0, 0, false, false, "test.s:1:5: Error: Unknown instruction\n" },
    /* FLAT encoding */
    { "flat_load_ubyte v[47:48], v[187:188] glc slc tfe",
        0xdc230000U, 0x2f8000bbU, true, true, "" },
    { "flat_load_ubyte v47, v[187:188] glc slc",
        0xdc230000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_ubyte v47, v[187:188] glc", 0xdc210000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_ubyte v47, v[187:188] glc   ", 0xdc210000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_ubyte v47, v[187:188]", 0xdc200000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_ubyte v47, v[187:188]   ", 0xdc200000U, 0x2f0000bbU, true, true, "" },
    /* FLAT errors */
    { "flat_load_ubyte v47, v[187:188] glc slcx", 0, 0, false, false,
        "test.s:1:37: Error: Unknown FLAT modifier\n" },
    { "flat_load_ubyte v[47:48], v[187:188] glc", 0, 0, false, false,
        "test.s:1:17: Error: Required 1 vector register\n" },
    { "flat_load_ubyte v47, v[187:188] glc slc tfe", 0, 0, false, false,
        "test.s:1:17: Error: Required 2 vector registers\n" },
    /* FLAT instructions */
    { "flat_load_sbyte  v47, v[187:188]", 0xdc240000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_ushort  v47, v[187:188]", 0xdc280000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_sshort  v47, v[187:188]", 0xdc2c0000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_dword  v47, v[187:188]", 0xdc300000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_dwordx2  v[47:48], v[187:188]",
        0xdc340000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_dwordx4  v[47:50], v[187:188]",
        0xdc380000U, 0x2f0000bbU, true, true, "" },
    { "flat_load_dwordx3  v[47:49], v[187:188]",
        0xdc3c0000U, 0x2f0000bbU, true, true, "" },
    { "flat_store_byte v[191:192], v84 glc slc tfe",
        0xdc630000U, 0x008054bfU, true, true, "" },
    { "flat_store_short v[191:192], v84 glc slc tfe",
        0xdc6b0000U, 0x008054bfU, true, true, "" },
    { "flat_store_dword v[191:192], v84", 0xdc700000U, 0x000054bfU, true, true, "" },
    { "flat_store_dwordx2 v[191:192], v[84:85]", 0xdc740000U, 0x000054bfU, true, true, "" },
    { "flat_store_dwordx4 v[191:192], v[84:87]", 0xdc780000U, 0x000054bfU, true, true, "" },
    { "flat_store_dwordx3 v[191:192], v[84:86]", 0xdc7c0000U, 0x000054bfU, true, true, "" },
    { "flat_atomic_swap  v[47:48], v[187:188], v65 glc slc tfe",
        0xdcc30000U, 0x2f8041bbU, true, true, "" },
    { "flat_atomic_swap  v47, v[187:188], v65 glc slc",
        0xdcc30000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_cmpswap v47, v[187:188], v[65:66] glc slc",
        0xdcc70000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_add v47, v[187:188], v65", 0xdcc80000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_sub v47, v[187:188], v65", 0xdccc0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_smin v47, v[187:188], v65", 0xdcd40000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_umin v47, v[187:188], v65", 0xdcd80000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_smax v47, v[187:188], v65", 0xdcdc0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_umax v47, v[187:188], v65", 0xdce00000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_and v47, v[187:188], v65", 0xdce40000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_or v47, v[187:188], v65", 0xdce80000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_xor v47, v[187:188], v65", 0xdcec0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_inc v47, v[187:188], v65", 0xdcf00000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_dec v47, v[187:188], v65", 0xdcf40000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_fcmpswap v47, v[187:188], v[65:66] glc slc",
        0xdcfb0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_fmin v47, v[187:188], v65", 0xdcfc0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_fmax v47, v[187:188], v65", 0xdd000000U, 0x2f0041bbU, true, true, "" },
    
    { "flat_atomic_swap_x2  v[47:49], v[187:188], v[65:66] glc slc tfe",
        0xdd430000U, 0x2f8041bbU, true, true, "" },
    { "flat_atomic_swap_x2  v[47:48], v[187:188], v[65:66] glc slc",
        0xdd430000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_cmpswap_x2 v[47:48], v[187:188], v[65:68] glc slc",
        0xdd470000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_add_x2 v[47:48], v[187:188], v[65:66]",
        0xdd480000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_sub_x2 v[47:48], v[187:188], v[65:66]",
        0xdd4c0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_smin_x2 v[47:48], v[187:188], v[65:66]",
        0xdd540000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_umin_x2 v[47:48], v[187:188], v[65:66]",
        0xdd580000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_smax_x2 v[47:48], v[187:188], v[65:66]",
        0xdd5c0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_umax_x2 v[47:48], v[187:188], v[65:66]",
        0xdd600000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_and_x2 v[47:48], v[187:188], v[65:66]",
        0xdd640000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_or_x2 v[47:48], v[187:188], v[65:66]",
        0xdd680000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_xor_x2 v[47:48], v[187:188], v[65:66]",
        0xdd6c0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_inc_x2 v[47:48], v[187:188], v[65:66]",
        0xdd700000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_dec_x2 v[47:48], v[187:188], v[65:66]",
        0xdd740000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_fcmpswap_x2 v[47:48], v[187:188], v[65:68] glc slc",
        0xdd7b0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_fmin_x2 v[47:48], v[187:188], v[65:66]",
        0xdd7c0000U, 0x2f0041bbU, true, true, "" },
    { "flat_atomic_fmax_x2 v[47:48], v[187:188], v[65:66]",
        0xdd800000U, 0x2f0041bbU, true, true, "" },
    { nullptr, 0, 0, false, false, 0 }
};
